/*****************************************************************************/
/*                                                                           */
/*   Function    : fAnalogTick()                                             */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : This function emulates the analog functions of the        */
/*                 vectrex. These functions produce the X,Y,Z deflection     */
/*                 signals for the CRT and the COMPARE signal for the        */
/*                 jostick AtoD system. The registers and values it needs    */
/*                 are taken from the 6522A Port A and B registers. The      */
/*                 value on port A is used as the DAC value an port B        */
/*                 is used for a number of the control lines that drive      */
/*                 the block such as SW,SEL0,SEL1,RAMP,BLANK.                */
/*                                                                           */
/*   Accepts     : WORD tNoOfTicks, the number of unit cycles to run         */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

//void fAnalogTick(void)
{
 fDebug(LOW,("fAnalogTick() - Processing analog event"));

 /*********************************/
 /* Emulate zero/ramping circuits */
 /*********************************/
 if (tSIGDEF_ZEROB)
 {
  if (!tLastRamp)
  {
   lwTickMark=lwGblSystemTickTotal-lwTickMark;
   lSIGDEF_XOUTPUT=lSIGDEF_XOUTPUT+lXSampleHold*lwTickMark;
   lSIGDEF_YOUTPUT=lSIGDEF_YOUTPUT+lYSampleHold*lwTickMark;
  }
 }
 else
 {
  lSIGDEF_XOUTPUT=0;
  lSIGDEF_YOUTPUT=0;
 }
 lwTickMark=lwGblSystemTickTotal;
 tLastRamp=tSIGDEF_RAMPB;

 /*****************************/
 /* Emulate blanking circuits */
 /*****************************/
 if (tSIGDEF_BLANKB) /* ZOUTPUT on */
 {
  tSIGDEF_ZOUTPUT=tZSampleHold;

  /***************************/
  /* PEN just 'switched' on? */
  /***************************/
  if (tOldBlank!=tSIGDEF_BLANKB)
  {
   lX1=lSIGDEF_XOUTPUT;
   lY1=lSIGDEF_YOUTPUT;
   tBright=tSIGDEF_ZOUTPUT;
   tOldBlank=tSIGDEF_BLANKB;
  }
  else
  {
#ifdef LIGHTPEN
   if (tOldBlank)
   {
    /*******************************************************/
    /* 'drawing' right now, might check for ramp==1 too... */
    /*******************************************************/
    if (mouse_button)
    {
     if (((labs(lSIGDEF_XOUTPUT-vmouse_x)<0x100)&&(labs(lSIGDEF_YOUTPUT-vmouse_y)<0x100)))
     {
      fDebug(HIGH,("fAnalogTick() - Correct Lightpen position reached PC=<%04x>",tsCPURegisters.tPC));
      /*********************/
      /* set bit 2 on d00d */
      /*********************/
      t6522IFLG|=E6522IFLG_CA1;
      if ((t6522IER&0x02)&& !tsCPURegisters.uCC.sCC.bI)
      {
       fDebug(HIGH,("f6522Tick() - IRQ asserted by 6522 CA1 Reg at PC=<%04x>",tsCPURegisters.tPC));
       tsCPURegisters.tIRQFlag=TRUE;
      }
     }
    }
   }
#endif
  }
 }
 else /* ZOUTPUT off */
 {
  tSIGDEF_ZOUTPUT=0;
  /*******************************************************************/
  /* Check for a vector draw completion and signal it to the display */
  /*******************************************************************/
  if (tOldBlank!=tSIGDEF_BLANKB)
  {
//    fDisplayLineDraw(lX1,lY1,lSIGDEF_XOUTPUT,lSIGDEF_YOUTPUT,tBright);
   SLONG lX2=lSIGDEF_XOUTPUT;
   SLONG lY2=lSIGDEF_YOUTPUT;

#ifdef ANALOG_IMAGER
 #define DISPLAY_TICK_IMAGER

  if (wGblImagerMode==6)
  {
   lX1+=wGblImagerXOffset;
   lX2+=wGblImagerXOffset;
//  printf("\nlX1:%i, lX2:%i",lX1,lX2);
  }

#endif

   #ifdef ANALOG_TICK_LINEAR_SHIFT
   #define DISPLAY_LINE_LINEAR_SHIFT
   #include "dline.cc"
   #undef DISPLAY_LINE_LINEAR_SHIFT
   #endif

   #ifdef ANALOG_TICK_LINEAR_DIV
   #define DISPLAY_LINE_LINEAR_DIV
   #include "dline.cc"
   #undef DISPLAY_LINE_LINEAR_DIV
   #endif

   #ifdef ANALOG_TICK_BANKED_SHIFT
   #define DISPLAY_LINE_BANKED_SHIFT
   #include "dline.cc"
   #undef DISPLAY_LINE_BANKED_SHIFT
   #endif

   #ifdef ANALOG_TICK_BANKED_DIV
   #define DISPLAY_LINE_BANKED_DIV
   #include "dline.cc"
   #undef DISPLAY_LINE_BANKED_DIV
   #endif

#ifdef ANALOG_IMAGER
 #undef DISPLAY_TICK_IMAGER
#endif

   tOldBlank=tSIGDEF_BLANKB;
  }
 }
 /**********************************/
 /* Convert DAC to a signed number */
 /**********************************/
 lXSampleHold=(SBYTE)tSIGDEF_DACBUS;//DAC_lookup[tSIGDEF_DACBUS];

 /******************************/
 /* Emulate switching circuits */
 /******************************/
 if (!tSIGDEF_SAMPHOLD) // Only use analog switch if enabled
 {
  switch (tSIGDEF_AMUXSEL)
  {
   case 0: /* Y AXIS */
    lYSampleHold=(SBYTE)tSIGDEF_DACBUS;//DAC_lookup[tSIGDEF_DACBUS];
    //lYSampleHold=DAC_lookup[tSIGDEF_DACBUS];
    break;
   case 4: /* Z AXIS - BRIGHT */
    tZSampleHold=(SBYTE)tSIGDEF_DACBUS;
    break;
//   case 2: /* ZERO OFFSET - Appparently for calibration only */
//    break;
//   case 6: /* SOUND */
//    break;
   default:
//    fDebug(VERROR,("fAnalogTick() Illegal analog mux setting"));
    break;
  }
 }

/*****************************************/
/* DEBUGING CODE TO OUTPUT USEFUL VALUES */
/*****************************************/
#ifdef DEBUG_MSG
 if(tGblDebugLevel==LOW)
 {
  static SLONG lOldXSH=-1,lOldYSH=-1;
  static UBYTE tOldZSH=-1;

  static SLONG lOldSigX=-1,lOldSigY=-1;
  static UBYTE tOldSigZ=-1;

  static UBYTE tOldBlank=123;
  static UBYTE tOldRamp=123;
  static UBYTE tOldZero=123;
  static UBYTE tOldSel=123;
  static UBYTE tOldShold=123;

  /******************/
  /* Debug printing */
  /******************/
  if(lOldXSH!=lXSampleHold)  fIOPrintf(20,100,BRIGHT15,BRIGHT0,FONT_SMALL,"lXSampleHold    = %08lx",lXSampleHold);
  if(lOldYSH!=lYSampleHold)  fIOPrintf(20,110,BRIGHT15,BRIGHT0,FONT_SMALL,"lYSampleHold    = %08lx",lYSampleHold);
  if(tOldZSH!=tZSampleHold)  fIOPrintf(20,120,BRIGHT15,BRIGHT0,FONT_SMALL,"tZSampleHold    = %02x",tZSampleHold);

  if(lOldSigX!=lSIGDEF_XOUTPUT) fIOPrintf(20,160,BRIGHT15,BRIGHT0,FONT_SMALL,"tSIGDEF_XOUTPUT = %08lx",lSIGDEF_XOUTPUT);
  if(lOldSigY!=lSIGDEF_YOUTPUT) fIOPrintf(20,170,BRIGHT15,BRIGHT0,FONT_SMALL,"tSIGDEF_YOUTPUT = %08lx",lSIGDEF_YOUTPUT);
  if(tOldSigZ!=tSIGDEF_ZOUTPUT) fIOPrintf(20,180,BRIGHT15,BRIGHT0,FONT_SMALL,"tSIGDEF_ZOUTPUT = %02x",tSIGDEF_ZOUTPUT);

  if(tOldBlank!=tSIGDEF_BLANKB)  fIOPrintf(20,200,BRIGHT15,BRIGHT0,FONT_SMALL,"tSIGDEF_BLANKB  = %02x",tSIGDEF_BLANKB );
  if(tOldRamp !=tSIGDEF_RAMPB)   fIOPrintf(20,210,BRIGHT15,BRIGHT0,FONT_SMALL,"tSIGDEF_RAMPB   = %02x",tSIGDEF_RAMPB  );
  if(tOldZero !=tSIGDEF_ZEROB)   fIOPrintf(20,220,BRIGHT15,BRIGHT0,FONT_SMALL,"tSIGDEF_ZEROB   = %02x",tSIGDEF_ZEROB  );
  if(tOldSel  !=tSIGDEF_AMUXSEL) fIOPrintf(20,230,BRIGHT15,BRIGHT0,FONT_SMALL,"tSIGDEF_AMUXSEL = %02x",tSIGDEF_AMUXSEL);
  if(tOldShold!=tSIGDEF_SAMPHOLD)fIOPrintf(20,240,BRIGHT15,BRIGHT0,FONT_SMALL,"tSIGDEF_SAMPHOLD= %02x",tSIGDEF_SAMPHOLD);

  lOldXSH=lXSampleHold;
  lOldYSH=lYSampleHold;
  tOldZSH=tZSampleHold;

  lOldSigX=lSIGDEF_XOUTPUT;
  lOldSigY=lSIGDEF_YOUTPUT;
  tOldSigZ=tSIGDEF_ZOUTPUT;

  tOldBlank=tSIGDEF_BLANKB;
  tOldRamp=tSIGDEF_RAMPB;
  tOldZero=tSIGDEF_ZEROB;
  tOldSel=tSIGDEF_AMUXSEL;
  tOldShold=tSIGDEF_SAMPHOLD;
 }
#endif
}
