/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : 6809 Emulator code functions per instruction               */
/*                                                                           */
/*   File Name  : COD6809.H                                                  */
/*                                                                           */
/*   Author     : Keith Wilkins                                              */
/*                                                                           */
/*   Version    : 1.00                                                       */
/*                                                                           */
/*   Desciption : This file contains the function code to execute any given  */
/*                processor instruction.                                     */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    02/07/95  K.W  Creation of empty stub file, description only   */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*                                                                           */
/*****************************************************************************/

//
// REGISTER MACROS
//
#define PC     (tsCPURegisters.tPC)
#define DP     (tsCPURegisters.tDP)
#define X      (tsCPURegisters.tX)
#define Y      (tsCPURegisters.tY)
#define D      (tsCPURegisters.uABD.tD)
#define A      (tsCPURegisters.uABD.sAB.tA)
#define B      (tsCPURegisters.uABD.sAB.tB)
#define S      (tsCPURegisters.tHSP)
#define U      (tsCPURegisters.tUSP)
#define CC     (tsCPURegisters.uCC.tCC)
#define CC_C   (tsCPURegisters.uCC.sCC.bC)
#define CC_V   (tsCPURegisters.uCC.sCC.bV)
#define CC_Z   (tsCPURegisters.uCC.sCC.bZ)
#define CC_N   (tsCPURegisters.uCC.sCC.bN)
#define CC_I   (tsCPURegisters.uCC.sCC.bI)
#define CC_H   (tsCPURegisters.uCC.sCC.bH)
#define CC_F   (tsCPURegisters.uCC.sCC.bF)
#define CC_E   (tsCPURegisters.uCC.sCC.bE)


// Interrupt function prototypes

UWORD  f6809_FIRQ(void);
UWORD  f6809_IRQ(void);
                  

// Register definitions for the above
//
// [DL] f6809AddressRead [AX]
//
// [DL] f6809AddressWrite [AX] [DL]


UWORD  f6809_P1_00(void);

UWORD  f6809_P1_03(void);
UWORD  f6809_P1_04(void);

UWORD  f6809_P1_06(void);
UWORD  f6809_P1_07(void);
UWORD  f6809_P1_08(void);
UWORD  f6809_P1_09(void);
UWORD  f6809_P1_0A(void);

UWORD  f6809_P1_0C(void);
UWORD  f6809_P1_0D(void);
UWORD  f6809_P1_0E(void);
UWORD  f6809_P1_0F(void);

UWORD  f6809_P1_10(void);
UWORD  f6809_P1_11(void);

UWORD  f6809_P1_12(void);
UWORD  f6809_P1_13(void);

UWORD  f6809_P1_16(void);
UWORD  f6809_P1_17(void);

UWORD  f6809_P1_19(void);

UWORD  f6809_P1_1A(void);

UWORD  f6809_P1_1C(void);
UWORD  f6809_P1_1D(void);
UWORD  f6809_P1_1E(void);
UWORD  f6809_P1_1F(void);

UWORD  f6809_P1_20(void);
UWORD  f6809_P1_21(void);
UWORD  f6809_P1_22(void);
UWORD  f6809_P1_23(void);
UWORD  f6809_P1_24(void);
UWORD  f6809_P1_25(void);
UWORD  f6809_P1_26(void);
UWORD  f6809_P1_27(void);
UWORD  f6809_P1_28(void);
UWORD  f6809_P1_29(void);
UWORD  f6809_P1_2A(void);
UWORD  f6809_P1_2B(void);
UWORD  f6809_P1_2C(void);
UWORD  f6809_P1_2D(void);
UWORD  f6809_P1_2E(void);
UWORD  f6809_P1_2F(void);

UWORD  f6809_P1_30(void);
UWORD  f6809_P1_31(void);
UWORD  f6809_P1_32(void);
UWORD  f6809_P1_33(void);
UWORD  f6809_P1_34(void);
UWORD  f6809_P1_35(void);
UWORD  f6809_P1_36(void);
UWORD  f6809_P1_37(void);

UWORD  f6809_P1_39(void);
UWORD  f6809_P1_3A(void);
UWORD  f6809_P1_3B(void);
UWORD  f6809_P1_3C(void);
UWORD  f6809_P1_3D(void);

UWORD  f6809_P1_3F(void);

UWORD  f6809_P1_40(void);

UWORD  f6809_P1_43(void);
UWORD  f6809_P1_44(void);

UWORD  f6809_P1_46(void);
UWORD  f6809_P1_47(void);
UWORD  f6809_P1_48(void);
UWORD  f6809_P1_49(void);
UWORD  f6809_P1_4A(void);

UWORD  f6809_P1_4C(void);
UWORD  f6809_P1_4D(void);        

UWORD  f6809_P1_4F(void);

UWORD  f6809_P1_50(void);

UWORD  f6809_P1_53(void);
UWORD  f6809_P1_54(void);

UWORD  f6809_P1_56(void);
UWORD  f6809_P1_57(void);
UWORD  f6809_P1_58(void);
UWORD  f6809_P1_59(void);

UWORD  f6809_P1_5A(void);

UWORD  f6809_P1_5C(void);
UWORD  f6809_P1_5D(void);

UWORD  f6809_P1_5F(void);

UWORD  f6809_P1_60(void);

UWORD  f6809_P1_63(void);
UWORD  f6809_P1_64(void);

UWORD  f6809_P1_66(void);
UWORD  f6809_P1_67(void);
UWORD  f6809_P1_68(void);
UWORD  f6809_P1_69(void);
UWORD  f6809_P1_6A(void);

UWORD  f6809_P1_6C(void);
UWORD  f6809_P1_6D(void);
UWORD  f6809_P1_6E(void);
UWORD  f6809_P1_6F(void);

UWORD  f6809_P1_70(void);

UWORD  f6809_P1_73(void);
UWORD  f6809_P1_74(void);

UWORD  f6809_P1_76(void);
UWORD  f6809_P1_77(void);
UWORD  f6809_P1_78(void);
UWORD  f6809_P1_79(void);
UWORD  f6809_P1_7A(void);

UWORD  f6809_P1_7C(void);
UWORD  f6809_P1_7D(void);

UWORD  f6809_P1_7E(void);
UWORD  f6809_P1_7F(void);

UWORD  f6809_P1_80(void);
UWORD  f6809_P1_81(void);
UWORD  f6809_P1_82(void);
UWORD  f6809_P1_83(void);
UWORD  f6809_P1_84(void);
UWORD  f6809_P1_85(void);
UWORD  f6809_P1_86(void);

UWORD  f6809_P1_88(void);
UWORD  f6809_P1_89(void);
UWORD  f6809_P1_8A(void);
UWORD  f6809_P1_8B(void);
UWORD  f6809_P1_8C(void);
UWORD  f6809_P1_8D(void);
UWORD  f6809_P1_8E(void);

UWORD  f6809_P1_90(void);
UWORD  f6809_P1_91(void);
UWORD  f6809_P1_92(void);
UWORD  f6809_P1_93(void);
UWORD  f6809_P1_94(void);
UWORD  f6809_P1_95(void);
UWORD  f6809_P1_96(void);
UWORD  f6809_P1_97(void);
UWORD  f6809_P1_98(void);
UWORD  f6809_P1_99(void);
UWORD  f6809_P1_9A(void);
UWORD  f6809_P1_9B(void);
UWORD  f6809_P1_9C(void);
UWORD  f6809_P1_9D(void);
UWORD  f6809_P1_9E(void);
UWORD  f6809_P1_9F(void);

UWORD  f6809_P1_A0(void);
UWORD  f6809_P1_A1(void);
UWORD  f6809_P1_A2(void);
UWORD  f6809_P1_A3(void);
UWORD  f6809_P1_A4(void);
UWORD  f6809_P1_A5(void);
UWORD  f6809_P1_A6(void);
UWORD  f6809_P1_A7(void);
UWORD  f6809_P1_A8(void);
UWORD  f6809_P1_A9(void);
UWORD  f6809_P1_AA(void);
UWORD  f6809_P1_AB(void);
UWORD  f6809_P1_AC(void);
UWORD  f6809_P1_AD(void);

UWORD  f6809_P1_AE(void);
UWORD  f6809_P1_AF(void);

UWORD  f6809_P1_B0(void);
UWORD  f6809_P1_B1(void);
UWORD  f6809_P1_B2(void);
UWORD  f6809_P1_B3(void);
UWORD  f6809_P1_B4(void);
UWORD  f6809_P1_B5(void);
UWORD  f6809_P1_B6(void);
UWORD  f6809_P1_B7(void);
UWORD  f6809_P1_B8(void);
UWORD  f6809_P1_B9(void);
UWORD  f6809_P1_BA(void);
UWORD  f6809_P1_BB(void);
UWORD  f6809_P1_BC(void);
UWORD  f6809_P1_BD(void);
UWORD  f6809_P1_BE(void);
UWORD  f6809_P1_BF(void);

UWORD  f6809_P1_C0(void);
UWORD  f6809_P1_C1(void);
UWORD  f6809_P1_C2(void);
UWORD  f6809_P1_C3(void);
UWORD  f6809_P1_C4(void);
UWORD  f6809_P1_C5(void);
UWORD  f6809_P1_C6(void);

UWORD  f6809_P1_C8(void);
UWORD  f6809_P1_C9(void);
UWORD  f6809_P1_CA(void);
UWORD  f6809_P1_CB(void);
UWORD  f6809_P1_CC(void);

UWORD  f6809_P1_CE(void);

UWORD  f6809_P1_D0(void);
UWORD  f6809_P1_D1(void);
UWORD  f6809_P1_D2(void);
UWORD  f6809_P1_D3(void);
UWORD  f6809_P1_D4(void);
UWORD  f6809_P1_D5(void);
UWORD  f6809_P1_D6(void);
UWORD  f6809_P1_D7(void);
UWORD  f6809_P1_D8(void);
UWORD  f6809_P1_D9(void);
UWORD  f6809_P1_DA(void);
UWORD  f6809_P1_DB(void);
UWORD  f6809_P1_DC(void);
UWORD  f6809_P1_DD(void);
UWORD  f6809_P1_DE(void);
UWORD  f6809_P1_DF(void);

UWORD  f6809_P1_E0(void);
UWORD  f6809_P1_E1(void);
UWORD  f6809_P1_E2(void);
UWORD  f6809_P1_E3(void);
UWORD  f6809_P1_E4(void);
UWORD  f6809_P1_E5(void);
UWORD  f6809_P1_E6(void);
UWORD  f6809_P1_E7(void);
UWORD  f6809_P1_E8(void);
UWORD  f6809_P1_E9(void);
UWORD  f6809_P1_EA(void);
UWORD  f6809_P1_EB(void);
UWORD  f6809_P1_EC(void);
UWORD  f6809_P1_ED(void);
UWORD  f6809_P1_EE(void);
UWORD  f6809_P1_EF(void);

UWORD  f6809_P1_F0(void);
UWORD  f6809_P1_F1(void);
UWORD  f6809_P1_F2(void);
UWORD  f6809_P1_F3(void);
UWORD  f6809_P1_F4(void);
UWORD  f6809_P1_F5(void);
UWORD  f6809_P1_F6(void);
UWORD  f6809_P1_F7(void);
UWORD  f6809_P1_F8(void);
UWORD  f6809_P1_F9(void);
UWORD  f6809_P1_FA(void);
UWORD  f6809_P1_FB(void);
UWORD  f6809_P1_FC(void);
UWORD  f6809_P1_FD(void);
UWORD  f6809_P1_FE(void);
UWORD  f6809_P1_FF(void);

UWORD  f6809_P2_21(void);
UWORD  f6809_P2_22(void);
UWORD  f6809_P2_23(void);
UWORD  f6809_P2_24(void);
UWORD  f6809_P2_25(void);
UWORD  f6809_P2_26(void);
UWORD  f6809_P2_27(void);
UWORD  f6809_P2_28(void);
UWORD  f6809_P2_29(void);
UWORD  f6809_P2_2A(void);
UWORD  f6809_P2_2B(void);
UWORD  f6809_P2_2C(void);
UWORD  f6809_P2_2D(void);
UWORD  f6809_P2_2E(void);
UWORD  f6809_P2_2F(void);

UWORD  f6809_P2_3F(void);

UWORD  f6809_P2_83(void);

UWORD  f6809_P2_8C(void);

UWORD  f6809_P2_8E(void);

UWORD  f6809_P2_93(void);

UWORD  f6809_P2_9C(void);

UWORD  f6809_P2_9E(void);
UWORD  f6809_P2_9F(void);

UWORD  f6809_P2_A3(void);

UWORD  f6809_P2_AC(void);

UWORD  f6809_P2_AE(void);
UWORD  f6809_P2_AF(void);

UWORD  f6809_P2_B3(void);

UWORD  f6809_P2_BC(void);

UWORD  f6809_P2_BE(void);
UWORD  f6809_P2_BF(void);

UWORD  f6809_P2_CE(void);

UWORD  f6809_P2_DE(void);
UWORD  f6809_P2_DF(void);

UWORD  f6809_P2_EE(void);
UWORD  f6809_P2_EF(void);

UWORD  f6809_P2_FE(void);
UWORD  f6809_P2_FF(void);

UWORD  f6809_P3_3F(void);

UWORD  f6809_P3_83(void);

UWORD  f6809_P3_8C(void);

UWORD  f6809_P3_93(void);

UWORD  f6809_P3_9C(void);

UWORD  f6809_P3_A3(void);

UWORD  f6809_P3_AC(void);

UWORD  f6809_P3_B3(void);

UWORD  f6809_P3_BC(void);


/* END OF FILE */

