/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : 6809 Disassembler Module                                   */
/*                                                                           */
/*   File Name  : dis6809.c                                                  */
/*                                                                           */
/*   Author     : Keith Wilkins                                              */
/*                                                                           */
/*   Version    : 1.00                                                       */
/*                                                                           */
/*   Desciption : This file contains the 6809 dissasembler code              */
/*                                                                           */
/*   Functions  : f6809Disassemble() - Return string of instruction code     */
/*                f6809CycleCount()  - Count 6809 machine cycles per inst    */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    11/04/95  K.W  Creation                                        */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*                                                                           */
/*****************************************************************************/

#define DIS6809_C

#include "vectrex.h"

/* Include opcode table & local references */

#include "tab6809.h"
#include "mon_cmds.h"

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6809Dissasemble()                                        */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Dissasemble 6809 code, passed an addres it returns a      */
/*                 string containing one dissasembled instruction and then   */
/*                 returns the addres of the next UBYTE.                      */
/*                                                                           */
/*   Accepts     : address & string pointer                                  */
/*                                                                           */
/*   Returns     : new address                                               */
/*                                                                           */
/*****************************************************************************/
                              
UWORD f6809Disassemble(UWORD wAddress,UBYTE *pcOutString)
{
   UBYTE	acTmpStr[DISAS_MAX_STRING];
   UWORD	wOpCode,wTemp,wLoop;
   UBYTE	bTemp,bFirstFlag;
        
	/* Read the 1st instruction BYTE */
		
	wOpCode=(UWORD)f6809AddressRead(wAddress++);
		
	strcpy(pcOutString,as6809Dis[wOpCode].acInstruction);

	/* Sort out 16 bit instructions before we do the real decode */
		
	switch(as6809Dis[wOpCode].bAddressMode)
	{		
		case PAGE2:
			wOpCode=(UWORD)f6809AddressRead(wAddress++);
			wOpCode+=256;
			strcpy(pcOutString,as6809Dis[wOpCode].acInstruction);
			break;
		case PAGE3:
			wOpCode=(UWORD)f6809AddressRead(wAddress++);
			wOpCode+=512;
			strcpy(pcOutString,as6809Dis[wOpCode].acInstruction);
			break;
		default:
			break;
	}
	strcat(pcOutString," ");
			
	/* Now we have the correct opcode generate the operand */
		
	switch(as6809Dis[wOpCode].bAddressMode)
	{
		case ILLEGAL:
			strcpy(pcOutString,"???");
			break;
				
		case INHERENT:
			break;
				
		case DIRECT:
			sprintf(acTmpStr,"$%02x",f6809AddressRead(wAddress++));
			strcat(pcOutString,acTmpStr);
			break;
				
		case EXTENDED:
			sprintf(acTmpStr,"$%02x%",f6809AddressRead(wAddress++));
			strcat(pcOutString,acTmpStr);
			sprintf(acTmpStr,"%02x",f6809AddressRead(wAddress++));
			strcat(pcOutString,acTmpStr);
			break;
				
		case RELATIVE8:
			wTemp=(UWORD)f6809AddressRead(wAddress++);
			if(wTemp>=128) wTemp=wTemp|0xff00;
			wTemp=wAddress+wTemp;
			sprintf(acTmpStr,"$%04x%",wTemp);
			strcat(pcOutString,acTmpStr);
			break;
				
		case RELATIVE16:
			wTemp=(UWORD)f6809AddressRead(wAddress++);
			wTemp=wTemp<<8;
			wTemp=wTemp | (UWORD)f6809AddressRead(wAddress++);
			wTemp=wAddress+wTemp;
			sprintf(acTmpStr,"$%04x%",wTemp);
			strcat(pcOutString,acTmpStr);
			break;
				
		case IMMEDIATE8:
			sprintf(acTmpStr,"#$%02x",f6809AddressRead(wAddress++));
			strcat(pcOutString,acTmpStr);
			break;
				
		case IMMEDIATE16:
			sprintf(acTmpStr,"#$%02x",f6809AddressRead(wAddress++));
			strcat(pcOutString,acTmpStr);
			sprintf(acTmpStr,"%02x",f6809AddressRead(wAddress++));
			strcat(pcOutString,acTmpStr);
			break;
				
		case REGISTER1:
			bTemp=f6809AddressRead(wAddress++);
			sprintf(acTmpStr,"%s,%s",asRegNames1[bTemp>>4],asRegNames1[bTemp&0x0f]);
			strcat(pcOutString,acTmpStr);
			break;
				
		case REGISTER2:
			wTemp=(UWORD)f6809AddressRead(wAddress++);
			bTemp=0;
			bFirstFlag=TRUE;
			for(wLoop=1;wLoop<129;wLoop=wLoop<<1)
			{
				if((wTemp&wLoop)!=0)
				{
					if(bFirstFlag==FALSE) strcat(pcOutString,",");
					strcat(pcOutString,asRegNames2[bTemp]);
					bFirstFlag=FALSE;
				}
				bTemp++;
			}
			break;
				
		case INDEXED:
		
			bTemp=f6809AddressRead(wAddress++);
			
			/* Now we must decode the index UBYTE */
		
			switch(asIndexDecode[bTemp].wWordSize)
			{
				case 0:
					strcat(pcOutString,asIndexDecode[bTemp].acFormat);
					break;
				case 1:
					sprintf(acTmpStr,asIndexDecode[bTemp].acFormat,f6809AddressRead(wAddress++));
					strcat(pcOutString,acTmpStr);
					break;
				case 2:
					wTemp=(UWORD)f6809AddressRead(wAddress++)<<8;
					wTemp+=(UWORD)f6809AddressRead(wAddress++);
					sprintf(acTmpStr,asIndexDecode[bTemp].acFormat,wTemp);
					strcat(pcOutString,acTmpStr);
					break;
				default:
					strcat(pcOutString,"Error in Index Table");
					break;
			}
			break;
				
    	case PAGE2:
		case PAGE3:
		default:
			strcat(pcOutString," Bad decode table entry");
			break;
	}

	/* Return the next valid address */
			
	return (wAddress);
}



/*****************************************************************************/
/*                                                                           */
/*   Function    : f6809CycleCount()                                         */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : This function counts the number of cycles taken to exec   */
/*                 the instruction at wAddr.                                 */
/*                                                                           */
/*   Accepts     : Address of instruction operand                            */
/*                                                                           */
/*   Returns     : number of CPU E clock cycles to execute instruction       */
/*                                                                           */
/*****************************************************************************/

UBYTE	f6809CycleCount(UWORD wAddr)
{
   wAddr=0;
	return(1);
}




/* End of DIS6809.C */                              
