/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : Vector Display driver software module                      */
/*                                                                           */
/*   File Name  : DISPLAY.C                                                  */
/*                                                                           */
/*   Author     : Keith Wilkins / C.S.                                       */
/*                                                                           */
/*   Version    : 1.20                                                       */
/*                                                                           */
/*   Desciption : This file contains the functions that drive the PC display */
/*                and set the modes up etc etc. A base plot routine is       */
/*                provided for displaying the vectrex screen.                */
/*                These functions only init the VECTOR based screen not the  */
/*                PC screen which is initialised in MAIN().                  */
/*                                                                           */
/*                The vector display functions use the fIO primatives from   */
/*                IO.C to plot pixels, this function maps from Vextrex       */
/*                pixels to REAL pixels. It also Plots pixels into the       */
/*                persistance handler area.                                  */
/*                                                                           */
/*                -Some function are now 'inline' functions                  */
/*                                                                           */
/*   Functions  : fVectorDisplayInit()    - Init the video display           */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    14/03/93  K.W  Creation of empty file and descriptions         */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*    1.20    20/10/96  C.S  Some function moved to inline status            */
/*                           Overlay stuff added, hash function...           */
/*                           new resolutions, div/shift...                   */
/*                                                                           */
/*****************************************************************************/

#define DISPLAY_C

#include "vectrex.h"
#include "palette.h"

#define BORDER_COLOUR       15  /* White border */
#define BACKGROUND_COLOUR   0   /* Black Background */

void fBorderTriangleDraw(SLONG xl0, SLONG yl0, SLONG xl1, SLONG yl1, SLONG xl2, SLONG yl2,UWORD ink);
void fOverlayTriangleDraw(SLONG xl0, SLONG yl0, SLONG xl1, SLONG yl1, SLONG xl2, SLONG yl2,UWORD ink);
void fOverlayDrawLine(SWORD x1,SWORD y1,SWORD x2, SWORD y2, UWORD iColor);

/************************************************/
/* 640x480,800x600,1024x768,1280x1024,1600x1200 */
/************************************************/
DISPLAYINFO    display_infox[RESOLUTION_COUNT]=
{
/******************************************************/
/* DivShftA   DivshftB   error scaled  error to 40000 */
/******************************************************/

          7,         8,  // error 12 : 996
          7,         7,  // error 24 : 1584
          6,         8,  // error 13 : 676
          6,         7,  // error 87 : 3393
          6,         6,  // error 50 : 1650
};

SWORD divider_x[RESOLUTION_COUNT]={
          83,            // error  1.92 : 159
          66,            // error  6.06 : 400
          52,            // error  1.23 : 64
          39,            // error  1.64 : 64
          33,            // error 12.12 : 400
};
DISPLAYINFO    display_infoy[RESOLUTION_COUNT]=
{
/******************************************************/
/* DivShftA   DivshftB   error scaled  error to 40000 */
/******************************************************/

          7,         8,  // error 12 : 996
          7,         7,  // error 24 : 1584
          6,         8,  // error 13 : 676
          6,         7,  // error 87 : 3393
          6,         6,  // error 50 : 1650
};

SWORD divider_y[RESOLUTION_COUNT]={
          83,            // error  1.92 : 159
          66,            // error  6.06 : 400
          52,            // error  1.23 : 64
          39,            // error  1.64 : 64
          33,            // error 12.12 : 400
};

SWORD dividerx;
SWORD dividery;

int mausxmin=0;
int mausxmax=0;
int mausymin=0;
int mausymax=0;
/*******************************************/
/* Decoded copies of all configs for speed */
/*******************************************/
SWORD iDivShiftAx;
SWORD iDivShiftBx;
SWORD iDivShiftAy;
SWORD iDivShiftBy;

SWORD iXOffset;
SWORD iYOffset;

SWORD iX1clip;
SWORD iY1clip;
SWORD iX2clip;
SWORD iY2clip;

/*****************************************/
/* Variables for refresh timing & adjust */
/*****************************************/
SWORD iLineSelected=-1;
SWORD iLineSelector=LINE_REFRESH_LIMIT;
ULONG lwRefreshMarker=0;

/**************************/
/* Line store definitions */
/**************************/
SDLINE line_store[LINESTORE_DEPTH];

/*****************************/
/* Hashing table definitions */
/*****************************/
SWORD line_freelist=0;
SWORD line_rootnode=-1;

SWORD hash_rootnodes[HASH_TABLE_SIZE];
SWORD hash_freelist=0;
SHASH hash_store[LINESTORE_DEPTH];



/***********************/
/* Overlay definitions */
/***********************/
UBYTE *overlay_line_starts[OVERLAY_LINE_COUNT];

/***********************************************/
/* prototype function for shift or div scaling */
/***********************************************/
void (* fDisplayLineDraw)(SLONG,SLONG,SLONG,SLONG,UBYTE);
void (* scale)(SWORD*,SWORD*,SWORD*,SWORD*,SLONG,SLONG,SLONG,SLONG);

void fDisplayLineDraw_div(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tBright);
void fDisplayLineDraw_shift(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tBright);



/**************************************************************/
/* just 2 scaling functions... added for convinient DIV/SHIFT */
/* handling within overlay/border funtions                    */
/**************************************************************/
void scale_shift(SWORD *iX1, SWORD *iY1,  SWORD *iX2,  SWORD *iY2,
                 SLONG lX1, SLONG lY1, SLONG lX2, SLONG lY2)
{
 *iX1=(lX1>>iDivShiftAx) + (lX1>>iDivShiftBx);
 *iY1=(lY1>>iDivShiftAy) + (lY1>>iDivShiftBy);
 *iX2=(lX2>>iDivShiftAx) + (lX2>>iDivShiftBx);
 *iY2=(lY2>>iDivShiftAy) + (lY2>>iDivShiftBy);
}
void scale_div(SWORD *iX1, SWORD *iY1,  SWORD *iX2,  SWORD *iY2,
               SLONG lX1, SLONG lY1, SLONG lX2, SLONG lY2)
{
 *iX1=lX1/dividerx;
 *iY1=lY1/dividery;
 *iX2=lX2/dividerx;
 *iY2=lY2/dividery;
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : fDisplayInit()                                            */
/*                                                                           */
/*   Author      : K.W / C.S.                                                */
/*                                                                           */
/*   Description : Clear the vector screen area and draw the border around   */
/*                 it to mark it out from the screen. Initialise all the     */
/*                 other bits and pieces.                                    */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : BOOLEAN TRUE if initialised OK else FALSE                 */
/*                                                                           */
/*****************************************************************************/

BOOL fDisplayInit(void)
{
 ULONG loop;
 UBYTE *ptr;
 PIXEL screen;

 screen=fIOGetScreenRes();

 /****************************/
 /* Initialise the line data */
 /****************************/
 if (bGblDisplayExactScaling==FALSE)
 {
  iDivShiftAx=display_infox[wGblDisplayMode].iDivShiftA;
  iDivShiftBx=display_infox[wGblDisplayMode].iDivShiftB;
  iDivShiftAy=display_infoy[wGblDisplayMode].iDivShiftA;
  iDivShiftBy=display_infoy[wGblDisplayMode].iDivShiftB;
  scale=scale_shift;
 }
 else
 {
  dividerx=divider_x[wGblDisplayMode];
  dividery=divider_y[wGblDisplayMode];
  scale=scale_div;
 }
 if ((ptr=fIOGetOverlayBuffer())==NULL)
  fFatal("fIOGetOverlayBuffer() returned NULL");
 for(loop=0;loop<OVERLAY_LINE_COUNT;loop++)
 {
  overlay_line_starts[loop]=ptr;
  ptr+=screen.iXpos;
 }
  fDisplayReset();
 return(TRUE);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : fDisplayReset()                                           */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Reset the vector display function                         */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/
void fDisplayReset(void)
{
 SWORD loop;
 PIXEL screen;
 ULONG screenmem;

 for (loop=0;loop<256;loop++)
 {
  palette[loop][0]=palette_reset[loop][0];
  palette[loop][1]=palette_reset[loop][1];
  palette[loop][2]=palette_reset[loop][2];
  palette[loop][3]=palette_reset[loop][3];
 }

 bGblDisplayFullRefresh=FALSE;
 bGblGoggleGame=FALSE;
 ImagerColorNow=255;
 wGblColor1Degrees=60;
 wGblColor2Degrees=60;
 wGblColor3Degrees=60;
 wGblImagerMode=0;
 wGblImagerXOffset=0;
 wGblImagerXOffset1=0;
 wGblImagerXOffset2=0;

 bGblLightpenEnable=FALSE;
 bGblOverlayEnable=FALSE;
 rectengle_clipping_enable=TRUE;

 divider_x[0]=83;
 divider_y[0]=83;

 divider_x[0]=66;
 divider_y[0]=66;

 divider_x[0]=52;
 divider_y[0]=52;

 divider_x[0]=39;
 divider_y[0]=39;

 divider_x[0]=33;
 divider_y[0]=33;

 display_infox[0].iDivShiftA=7;
 display_infox[0].iDivShiftB=8;
 display_infoy[0].iDivShiftA=7;
 display_infoy[0].iDivShiftB=8;

 display_infox[1].iDivShiftA=7;
 display_infox[1].iDivShiftB=7;
 display_infoy[1].iDivShiftA=7;
 display_infoy[1].iDivShiftB=7;

 display_infox[2].iDivShiftA=6;
 display_infox[2].iDivShiftB=8;
 display_infoy[2].iDivShiftA=6;
 display_infoy[2].iDivShiftB=8;

 display_infox[3].iDivShiftA=6;
 display_infox[3].iDivShiftB=7;
 display_infoy[3].iDivShiftA=6;
 display_infoy[3].iDivShiftB=7;

 display_infox[4].iDivShiftA=6;
 display_infox[4].iDivShiftB=6;
 display_infoy[4].iDivShiftA=6;
 display_infoy[4].iDivShiftB=6;

 screen=fIOGetScreenRes();
 iXOffset=screen.iXpos/2;
 iYOffset=screen.iYpos/2;
 if (bGblDisplayExactScaling==FALSE)
 {
  iX1clip=(lwGblClipX>>iDivShiftAx) + (lwGblClipX>>iDivShiftBx);
  iY1clip=(lwGblClipY>>iDivShiftAy) + (lwGblClipY>>iDivShiftBy);
  iX2clip=(-lwGblClipX>>iDivShiftAx) + (-lwGblClipX>>iDivShiftBx);
  iY2clip=(-lwGblClipY>>iDivShiftAy) + (-lwGblClipY>>iDivShiftBy);

  mausxmin=iXOffset-((20000>>iDivShiftAx)+(20000>>iDivShiftBx));
  mausxmax=iXOffset+((20000>>iDivShiftAx)+(20000>>iDivShiftBx));
  mausymin=iYOffset-((20000>>iDivShiftAy)+(20000>>iDivShiftBy));
  mausymax=iYOffset+((20000>>iDivShiftAy)+(20000>>iDivShiftBy));
 }
 else
 {
  iX1clip=lwGblClipX/dividerx;
  iY1clip=lwGblClipY/dividery;
  iX2clip=-lwGblClipX/dividerx;
  iY2clip=-lwGblClipY/dividery;

  mausxmin=iXOffset-20000/dividerx; // just in case a lightpen
  mausxmax=iXOffset+20000/dividerx; // will be enabled
  mausymin=iYOffset-20000/dividery;
  mausymax=iYOffset+20000/dividery;
 }
 iX1clip+=iXOffset;
 iY1clip+=iYOffset;
 iX2clip+=iXOffset;
 iY2clip+=iYOffset;

 /**************************************/
 /* Initialise Line store & hash table */
 /**************************************/
 for(loop=0;loop<LINESTORE_DEPTH-1;loop++)
 {
  line_store[loop].next_vector=loop+1;
  hash_store[loop].next_hash=loop+1;
 }
 line_store[loop].next_vector=-1;
 hash_store[loop].next_hash=-1;

 line_freelist=0;
 hash_freelist=0;

 line_rootnode=-1;

 for(loop=0;loop<HASH_TABLE_SIZE;loop++)
 {
  hash_rootnodes[loop]=-1;
 }

 if(bGblDisplayAutoRefresh)
  wGblDisplayLinePeriod=1;

 fIOSetClipping(-1,-1,-1,-1);
 fDisplayClear();

 /********************************************/
 /* Initialise overlay memory to default ink */
 /********************************************/
 screen=fIOGetScreenRes();
 if ((overlay_memory)==NULL)
  fFatal("fIOGetOverlayBuffer() returned NULL");
 screenmem=(ULONG)screen.iXpos*(ULONG)screen.iYpos;
 memset(overlay_memory,wGblDisplayBaseColour<<4,screenmem);

 /***********************************************************/
 /* Now screen is reset then reload the Overlay if required */
 /***********************************************************/
 bGblDisplayFullRefresh=FALSE;
 bGblLightpenEnable=FALSE;
 rectengle_clipping_enable=TRUE;
 fDisplayLoadVOL();
if (wGblImagerMode!=0)
{
/* initialise degrees of color wheel in system ticks */
/* in lwGblWheelTicks is 1/2 turn of the wheel in ticks */
/* wGblColor1Degrees+wGblColor2Degrees+wGblColor3Degrees should be 180 degrees */

 ULONG ltmp_all=wGblColor1Degrees+wGblColor2Degrees+wGblColor3Degrees;
 wGblColor1Degrees = (lwGblWheelTicks*wGblColor1Degrees)/ltmp_all;
 wGblColor2Degrees = (lwGblWheelTicks*wGblColor2Degrees)/ltmp_all;
 wGblColor3Degrees = (lwGblWheelTicks*wGblColor3Degrees)/ltmp_all;
 if (lwGblWheelTicks!=(wGblColor1Degrees+wGblColor2Degrees+wGblColor3Degrees))
 {
  wGblColor1Degrees+=lwGblWheelTicks-(wGblColor1Degrees+wGblColor2Degrees+wGblColor3Degrees);
 }

 if (wGblImagerMode==6)
 {
 if (bGblDisplayExactScaling==FALSE)
 {
  wGblImagerXOffset1=-(iXOffset+(-30000>>iDivShiftAx) + (-30000>>iDivShiftBx));
  wGblImagerXOffset2=screen.iXpos-((30000>>iDivShiftAx) + (30000>>iDivShiftBx))-iXOffset ;
  wGblImagerXOffset1=(wGblImagerXOffset1<<iDivShiftAx)+(wGblImagerXOffset1<<iDivShiftBx);
  wGblImagerXOffset2=(wGblImagerXOffset2<<iDivShiftAx)+(wGblImagerXOffset2<<iDivShiftBx);
 }
 else
 {
  wGblImagerXOffset1=-(iXOffset+(-30000/dividerx));
  wGblImagerXOffset2=screen.iXpos-(30000/dividerx)-iXOffset;
  wGblImagerXOffset1*=dividerx;
  wGblImagerXOffset2*=dividerx;
// printf("iXOffset:%i, (20000/dividerx):%i, screen.iXpos:%i",iXOffset,(20000/dividerx),screen.iXpos);
 }
/*
 0 muss so sein, dass ich noch -20000 darstellen kann

 und

 0 muss so sein, dass ich noch +20000 darstellen kann
*/

 }
//printf("\nxo1:%i, xo2:%i",wGblImagerXOffset1,wGblImagerXOffset2);

}

 /*************************************************/
 /* Before exiting set the vector clipping region */
 /*************************************************/
#ifndef DEBUG_MON
#ifndef DEBUG_MSG
 fIOSetClipping(iX1clip,iY1clip,iX2clip,iY2clip);
#endif
#endif

}

void fBorderLineDraw(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tPen)
{
 SWORD iX1,iY1,iX2,iY2;

 /**************************/
 /* Scale to screen coords */
 /**************************/
 lY1=-lY1;
 lY2=-lY2;
 scale(&iX1,&iY1,&iX2,&iY2,lX1,lY1,lX2,lY2);
 iX1+=iXOffset;
 iY1+=iYOffset;
 iX2+=iXOffset;
 iY2+=iYOffset;
 fIODrawLine(iX1,iY1,iX2,iY2,(SWORD)tPen);
}

void fBorderBlockDraw(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tPen)
{
 SWORD iX1,iY1,iX2,iY2;

 /**************************/
 /* Scale to screen coords */
 /**************************/
 lY1=-lY1;
 lY2=-lY2;
 scale(&iX1,&iY1,&iX2,&iY2,lX1,lY1,lX2,lY2);
 iX1+=iXOffset;
 iY1+=iYOffset;
 iX2+=iXOffset;
 iY2+=iYOffset;
 fIODrawBox(iX1,iY1,iX2,iY2,(SWORD)tPen,TRUE);
}

void fBorderElipseDraw(SLONG lX1,SLONG lY1,SLONG lMajor,SLONG lMinor,UBYTE tPen)
{
 SWORD   iX1,iY1,iMajor,iMinor,row,result;
 double  lsmaj2,lsmin2,ly2;

 /**************************/
 /* Scale to screen coords */
 /**************************/
 lY1=-lY1;
 scale(&iX1,&iY1,&iMajor,&iMinor,lX1,lY1,lMajor,lMinor);
 iX1+=iXOffset;
 iY1+=iYOffset;

 lsmaj2=pow(iMajor,2.0);
 lsmin2=pow(iMinor,2.0);

 for(row=0;row<=iMinor;row++)
 {
  ly2=pow((double)row,2.0);
  result=floor(sqrt(lsmaj2-((ly2*lsmaj2)/lsmin2)));
  fIODrawLine(iX1-result,iY1-row,iX1+result,iY1-row,tPen);
  fIODrawLine(iX1-result,iY1+row,iX1+result,iY1+row,tPen);
 }
}

void fOverlayBlockDraw(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tPen)
{
 SWORD iX1,iY1,iX2,iY2;
 SWORD        loop;
 PIXEL   screen;

 screen=fIOGetScreenRes();

 /**************************/
 /* Scale to screen coords */
 /**************************/
 lY1=-lY1;
 lY2=-lY2;
 scale(&iX1,&iY1,&iX2,&iY2,lX1,lY1,lX2,lY2);
 iX1+=iXOffset;
 iY1+=iYOffset;
 iX2+=iXOffset;
 iY2+=iYOffset;
 if(iX1>iX2)
  fSwapWord(&iX1,&iX2);
 if(iY1>iY2)
  fSwapWord(&iY1,&iY2);

 for(;iY1<=iY2;iY1++)
 {
  if(iY1>=0 && iY1<screen.iYpos)
  {
   for(loop=iX1;loop<=iX2;loop++)
   {
    if(loop>=0 && loop<screen.iXpos)
    {
     *(overlay_line_starts[iY1]+loop)=tPen;
    }
   }
  }
 }
}

void fOverlayElipseDraw(SLONG lX1,SLONG lY1,SLONG lMajor,SLONG lMinor,UBYTE tPen)
{
 SWORD   iX1,iY1,iMajor,iMinor,row,result,loop;
 double  lsmaj2,lsmin2,ly2;
 PIXEL   screen;

 screen=fIOGetScreenRes();

 /**************************/
 /* Scale to screen coords */
 /**************************/
 lY1=-lY1;
 scale(&iX1,&iY1,&iMajor,&iMinor,lX1,lY1,lMajor,lMinor);
 iX1+=iXOffset;
 iY1+=iYOffset;
 lsmaj2=pow(iMajor,2.0);
 lsmin2=pow(iMinor,2.0);

 for(row=0;row<=iMinor;row++)
 {
  ly2=pow((double)row,2.0);
  result=floor(sqrt(lsmaj2-((ly2*lsmaj2)/lsmin2)));
  if(iY1>=0 && iY1<screen.iYpos)
  {
   for(loop=iX1-result;loop<=iX1+result;loop++)
   {
    if(loop>=0 && loop<screen.iXpos)
    {
     *(overlay_line_starts[iY1-row]+loop)=tPen;
     *(overlay_line_starts[iY1+row]+loop)=tPen;
    }
   }
  }
 }
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : fDisplayClear()                                           */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Cleans the screen area inside the vectrex screen area to  */
/*                 colour 0 (Background)                                     */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

void fDisplayClear(void)
{
 fIOAreaClear(-1,-1,-1,-1,0);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : fDisplayLoadVOL()                                         */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Load/Initialise VOL (Vectrex OverLay) file                */
/*                                                                           */
/*   Accepts     : Char* pointer to file to load from                        */
/*                                                                           */
/*   Returns     : BOOL    TRUE=Init OK, FALSE=Init Fail                     */
/*                                                                           */
/*****************************************************************************/

BOOL fDisplayLoadVOL(void)
{
 UBYTE   filename[STRING_LENGTH];

 FILE    *pfINIfile;
 UBYTE   acBuffer[1024];

 UBYTE   *pcCommandName;
 UBYTE   *pcCommandType;
 UBYTE   *pcArgStr;

 ULONG   arglist[MAX_ARGS];
 ULONG   loop,argcount;

 SWORD   line=1;
 BOOL    debug=FALSE;
 BOOL    screen_fix=FALSE;

// File syntax definitions

// OVERLAY BLOCK <PEN> <X1> <Y1> <X2> <Y2>
// OVERLAY ELIPSE <PEN> <X1> <Y1> <major> <minor>
// overlay triangle pen x1 y1 x2 y2 x3 y3

// OVERLAY <ENABLE/DISABLE>

// OVERLAY_PCX0 FILENAME (PCX 256 color file with exact resolution! for resolution 0)
// OVERLAY_PCX1 FILENAME (PCX 256 color file with exact resolution! for resolution 1)
// OVERLAY_PCX2 FILENAME (PCX 256 color file with exact resolution! for resolution 2)
// OVERLAY_PCX3 FILENAME (PCX 256 color file with exact resolution! for resolution 1)
// OVERLAY_PCX4 FILENAME (PCX 256 color file with exact resolution! for resolution 2)

// LIGHTPEN
// FULLREFRESH

// EXTENDED_RAM

// BORDER BLOCK   <PEN> <INTENSITY> <X1> <Y1> <X2> <Y2>
// BORDER LINE    <PEN> <INTENSITY> <X1> <Y1> <X2> <Y2>
// BORDER ELIPSE  <PEN> <INTENSITY> <X1> <Y1> <major> <minor>
// border triangle pen INTENSITY x1 y1 x2 y2 x3 y3

// SCREEN CLIP    <X1> <Y1> <X2> <Y2>
// SCREEN OFFSET  <X1> <Y1>
// SCREEN COLOUR  <PEN> <INTENSITY> <RED> <GREEN> <BLUE>

// SCREEN DIVX   <RESOLUTION> <DIV>
// SCREEN DIVY   <RESOLUTION> <DIV>
// SCREEN SHIFTX   <RESOLUTION> <SHIFT A> <SHIFT B>
// SCREEN SHIFTY   <RESOLUTION> <SHIFT A> <SHIFT B>


// DEBUG

// Create the filename

 if(strcmp(acGblDefaultCartFileName,""))
  strcpy(filename,acGblDefaultCartFileName);
 else
  strcpy(filename,acGblROMImageFileName);
 if(strrchr(filename,'.')!=NULL)
  *(strrchr(filename,'.'))='\0';
 strcat(filename,".vol");

 for(loop=0;loop<MAX_ARGS;loop++)
  arglist[loop]=DUFF_VALUE;

 // Now use the INI file to overload the values

 if((pfINIfile=fopen(filename,"rt"))!=NULL)
 {
  while(fgets(acBuffer,1024,pfINIfile)!=NULL)
  {
   strupr(acBuffer);

   if((pcCommandType=strtok(acBuffer," ,=:\n"))==NULL)
    strcpy(pcCommandType,"#");
   pcCommandName=strtok(NULL," ,=:\n");

   argcount=0;
   while ((pcArgStr=strtok(NULL," ,=:\n"))!=NULL)
   {
    arglist[argcount++]=atol(pcArgStr);
   }
   // Decode data
   if (strcmp(pcCommandType,"NO_SHADES")==0)
   {
    bGblNoShades=TRUE;
   }
   else if (strcmp(pcCommandType,"3D_IMAGER_GAME")==0)
   {
    if (bGblLightpenEnable==TRUE)
    {
     fFatal("Overlay init ERROR! No game has Lightpen + Imagers...");
    }
    bGblGoggleGame=TRUE;
    if (framebuffer==FB_LINEAR) /* linear framebuffer */
    {
     if (bGblDisplayExactScaling==FALSE) /* SHIFT */
     {
      fAnalogTick=fAnalogTick_linear_shift_imager;
     }
     else /* DIV */
     {
      fAnalogTick=fAnalogTick_linear_div_imager;
     }
    }
    else /* banked framebuffer */
    {
     if (bGblDisplayExactScaling==FALSE) /* SHIFT */
     {
      fAnalogTick=fAnalogTick_bank_shift_imager;
     }
     else /* DIV */
     {
      fAnalogTick=fAnalogTick_bank_div_imager;
     }
    }
    if (framebuffer==FB_LINEAR) /* linear framebuffer */
    {
     fVectrexEmulate=fVectrexEmulate_linear_imager;
    }
    else /* banked framebuffer */
    {
     fVectrexEmulate=fVectrexEmulate_banked_imager;
    }
   }
   else if(strcmp(pcCommandType,"WHEEL_TICKS")==0)
   {
    lwGblWheelTicks=atol(pcCommandName)/2;
   }
   else if(strcmp(pcCommandType,"COLOR_1_DEGREES")==0)
   {
    wGblColor1Degrees=atol(pcCommandName);
   }
   else if(strcmp(pcCommandType,"COLOR_2_DEGREES")==0)
   {
    wGblColor2Degrees=atol(pcCommandName);
   }
   else if(strcmp(pcCommandType,"COLOR_3_DEGREES")==0)
   {
    wGblColor3Degrees=atol(pcCommandName);
   }
   else if(strcmp(pcCommandType,"IMAGER_MODE")==0)
   {
    wGblImagerMode=atol(pcCommandName);
   }
   else if (strcmp(pcCommandType,"LOAD_COLORS_EXACT")==0)
   {
    bGblLoadColorsExact=TRUE;
   }
   else if (strcmp(pcCommandType,"EXTENDED_RAM")==0)
   {
    bGblExtendedRam=TRUE;          /* Animaction Ram 2K */
   }
   else if (strcmp(pcCommandType,"FULLREFRESH")==0)
   {
    bGblDisplayFullRefresh=TRUE;
   }
   else if (strcmp(pcCommandType,"LIGHTPEN")==0)
   {
    if (bGblGoggleGame==TRUE)
    {
     fFatal("Overlay init ERROR! No game has Lightpen + Imagers...");
    }
    bGblLightpenEnable=TRUE;
    fIOInitMouse();
    f6522Addr0DRead=f6522Addr0DReadLightpen;

    if (framebuffer==FB_LINEAR) /* linear framebuffer */
    {
     fVectrexEmulate=fVectrexEmulate_linear_lightpen;
     if (bGblDisplayExactScaling==FALSE) /* SHIFT */
     {
      fAnalogTick=fAnalogTick_linear_shift_lightpen;
     }
     else /* DIV */
     {
      fAnalogTick=fAnalogTick_linear_div_lightpen;
     }
    }
    else /* banked framebuffer */
    {
     fVectrexEmulate=fVectrexEmulate_banked_lightpen;
     if (bGblDisplayExactScaling==FALSE) /* SHIFT */
     {
      fAnalogTick=fAnalogTick_bank_shift_lightpen;
     }
     else /* DIV */
     {
      fAnalogTick=fAnalogTick_bank_div_lightpen;
     }
    }
   }
   else if (strcmp(pcCommandType,"OVERLAY_PCX0")==0)
   {
    if (wGblDisplayMode==0)
    {
     if (fIOload_pcx_overlay(pcCommandName)!=TRUE)
     {
      fFatal("Error loading PCX Overlay file");
     }
     rectengle_clipping_enable=FALSE;
    }
   }
   else if (strcmp(pcCommandType,"OVERLAY_PCX1")==0)
   {
    if (wGblDisplayMode==1)
    {
     if (fIOload_pcx_overlay(pcCommandName)!=TRUE)
     {
      fFatal("Error loading PCX Overlay file");
     }
     rectengle_clipping_enable=FALSE;
    }
   }
   else if (strcmp(pcCommandType,"OVERLAY_PCX2")==0)
   {
    if (wGblDisplayMode==2)
    {
     if (fIOload_pcx_overlay(pcCommandName)!=TRUE)
     {
      fFatal("Error loading PCX Overlay file");
     }
     rectengle_clipping_enable=FALSE;
    }
   }
   else if (strcmp(pcCommandType,"OVERLAY_PCX3")==0)
   {
    if (wGblDisplayMode==3)
    {
     if (fIOload_pcx_overlay(pcCommandName)!=TRUE)
     {
      fFatal("Error loading PCX Overlay file");
     }
     rectengle_clipping_enable=FALSE;
    }
   }
   else if (strcmp(pcCommandType,"OVERLAY_PCX4")==0)
   {
    if (wGblDisplayMode==4)
    {
     if (fIOload_pcx_overlay(pcCommandName)!=TRUE)
     {
      fFatal("Error loading PCX Overlay file");
     }
     rectengle_clipping_enable=FALSE;
    }
   }
   else if(strcmp(pcCommandType,"OVERLAY")==0)
   {
    if (strcmp(pcCommandName,"TRIANGLE")==0)
    {
     UWORD ink=((arglist[0]<<4)&0xf0);

     int x0=arglist[1];
     int y0=arglist[2];
     int x1=arglist[3];
     int y1=arglist[4];
     int x2=arglist[5];
     int y2=arglist[6];
     if (debug)
      fBorderTriangleDraw(x0,y0,x1,y1,x2,y2,ink);
     fOverlayTriangleDraw(x0,y0,x1,y1,x2,y2,ink);
    }
    else if(strcmp(pcCommandName,"BLOCK")==0)
    {
     UWORD   ink;
     if(argcount!=5)
      fFatal("OVERLAY BLOCK, incorrect number of paramaters, line %d of %s,",line,filename);
     ink=((arglist[0]<<4)&0xf0);
     if(debug)
      fBorderBlockDraw(arglist[1],arglist[2],arglist[3],arglist[4],ink+15);
     fOverlayBlockDraw(arglist[1],arglist[2],arglist[3],arglist[4],ink);
    }
    else if(strcmp(pcCommandName,"ELIPSE")==0)
    {
     UWORD   ink;

     if(argcount!=5)
      fFatal("OVERLAY ELIPSE, incorrect number of paramaters, line %d of %s,",line,filename);

     ink=((arglist[0]<<4)&0xf0);

     if(debug)
      fBorderElipseDraw(arglist[1],arglist[2],arglist[3],arglist[4],ink+15);
     fOverlayElipseDraw(arglist[1],arglist[2],arglist[3],arglist[4],ink);
    }
    else if(strcmp(pcCommandName,"ENABLE")==0)
    {
     bGblOverlayEnable=TRUE;
    }
    else if(strcmp(pcCommandName,"DISABLE")==0)
    {
     bGblOverlayEnable=FALSE;
    }
    else
    {
     fFatal("Bad/Missing command type for OVERLAY (%s) line %d of %s",pcCommandName,line,filename);
    }
   }
   else if(strcmp(pcCommandType,"BORDER")==0)
   {
    if (strcmp(pcCommandName,"TRIANGLE")==0)
    {
     UWORD ink=((arglist[0]<<4)&0xf0)+(arglist[1]&0x0f);

     int x0=arglist[2];
     int y0=arglist[3];
     int x1=arglist[4];
     int y1=arglist[5];
     int x2=arglist[6];
     int y2=arglist[7];
     fBorderTriangleDraw(x0,y0,x1,y1,x2,y2,ink);
    }
    else if(strcmp(pcCommandName,"BLOCK")==0)
    {
     UWORD   ink;

     if(argcount!=6)
      fFatal("BORDER BLOCK, incorrect number of paramaters, line %d of %s,",line,filename);
     ink=((arglist[0]<<4)&0xf0)+(arglist[1]&0x0f);

     fBorderBlockDraw(arglist[2],arglist[3],arglist[4],arglist[5],ink);
    }
    else if(strcmp(pcCommandName,"LINE")==0)
    {
     UWORD   ink;

     if(argcount!=6)
      fFatal("BORDER LINE, incorrect number of paramaters, line %d of %s,",line,filename);

     ink=((arglist[0]<<4)&0xf0)+(arglist[1]&0x0f);

     fBorderLineDraw(arglist[2],arglist[3],arglist[4],arglist[5],ink);
    }
    else if(strcmp(pcCommandName,"ELIPSE")==0)
    {
     UWORD   ink;

     if(argcount!=6) fFatal("BORDER LINE, incorrect number of paramaters, line %d of %s,",line,filename);

     ink=((arglist[0]<<4)&0xf0)+(arglist[1]&0x0f);

     fBorderElipseDraw(arglist[2],arglist[3],arglist[4],arglist[5],ink);
    }
    else
    {
     fFatal("Bad/Missing command type for BORDER (%s) line %d of %s",pcCommandName,line,filename);
    }
   }
   else if(strcmp(pcCommandType,"SCREEN")==0)
   {
    if(strcmp(pcCommandName,"CLIP")==0)
    {
     if(argcount!=4) fFatal("SCREEN CLIP, incorrect number of paramaters, line %d of %s,",line,filename);

     scale(&iX1clip,&iY1clip,&iX2clip,&iY2clip,arglist[0],arglist[1],arglist[2],arglist[3]);

     if(iX1clip>iX2clip)
      fSwapWord(&iX1clip,&iX2clip);
     if(iY1clip>iY2clip)
      fSwapWord(&iY1clip,&iY2clip);

     iX1clip+=iXOffset;
     iY1clip+=iYOffset;
     iX2clip+=iXOffset;
     iY2clip+=iYOffset;
    }
    else if(strcmp(pcCommandName,"COLOUR")==0)
    {
     UWORD   ink;

     if(argcount!=5)
      fFatal("SCREEN COLOUR, incorrect number of paramaters, line %d of %s,",line,filename);

     ink=((arglist[0]<<4)&0xf0)+(arglist[1]&0x0f);
     fIOsetink(ink,arglist[2],arglist[3],arglist[4]);
    }
    else if(strcmp(pcCommandName,"OFFSET")==0)
    {
     SWORD  xoffs,yoffs,dummy;

     if(argcount!=2)
      fFatal("SCREEN OFFSET, incorrect number of paramaters, line %d of %s,",line,filename);

     scale(&xoffs,&yoffs,&dummy,&dummy,arglist[0],arglist[1],arglist[0],arglist[1]);
     iXOffset+=xoffs;
     iYOffset+=yoffs;
     iX1clip+=xoffs;
     iY1clip+=yoffs;
     iX2clip+=xoffs;
     iY2clip+=yoffs;
     if (bGblDisplayExactScaling==FALSE)
     {
      mausxmin=iXOffset-((20000>>iDivShiftAx)+(20000>>iDivShiftBx));
      mausxmax=iXOffset+((20000>>iDivShiftAx)+(20000>>iDivShiftBx));
      mausymin=iYOffset-((20000>>iDivShiftAy)+(20000>>iDivShiftBy));
      mausymax=iYOffset+((20000>>iDivShiftAy)+(20000>>iDivShiftBy));
     }
     else
     {
      mausxmin=iXOffset-20000/dividerx;
      mausxmax=iXOffset+20000/dividerx;
      mausymin=iYOffset-20000/dividery;
      mausymax=iYOffset+20000/dividery;
     }
    }
    else if(strcmp(pcCommandName,"DIVX")==0)
    {
     if(argcount!=2)
      fFatal("SCREEN DIVX, incorrect number of paramaters, line %d of %s,",line,filename);
     if (arglist[0]>RESOLUTION_COUNT)
      fFatal("SCREEN DIVX, incorrect Resolution, line %d of %s,",line,filename);
     divider_x[arglist[0]]=arglist[1];
     screen_fix=TRUE;
    }
    else if(strcmp(pcCommandName,"DIVY")==0)
    {
     if(argcount!=2)
      fFatal("SCREEN DIVY, incorrect number of paramaters, line %d of %s,",line,filename);
     if (arglist[0]>RESOLUTION_COUNT)
      fFatal("SCREEN DIVY, incorrect Resolution, line %d of %s,",line,filename);

     divider_y[arglist[0]]=arglist[1];
     screen_fix=TRUE;
    }

    else if(strcmp(pcCommandName,"SHIFTX")==0)
    {
     if(argcount!=3)
      fFatal("SCREEN SHIFTX, incorrect number of paramaters, line %d of %s,",line,filename);
     if (arglist[0]>RESOLUTION_COUNT)
      fFatal("SCREEN SHIFTX, incorrect Resolution, line %d of %s,",line,filename);

     display_infox[arglist[0]].iDivShiftA=arglist[1];
     display_infox[arglist[0]].iDivShiftB=arglist[2];
     screen_fix=TRUE;
    }
    else if(strcmp(pcCommandName,"SHIFTY")==0)
    {
     if(argcount!=2)
      fFatal("SCREEN SHIFTY, incorrect number of paramaters, line %d of %s,",line,filename);
     if (arglist[0]>RESOLUTION_COUNT)
      fFatal("SCREEN SHIFTY, incorrect Resolution, line %d of %s,",line,filename);

     display_infoy[arglist[0]].iDivShiftA=arglist[1];
     display_infoy[arglist[0]].iDivShiftB=arglist[2];
     screen_fix=TRUE;
    }
    else
    {
     fFatal("Bad/Missing command type for SCREEN (%s) line %d of %s",pcCommandName,line,filename);
    }
   }
   else if(strcmp(pcCommandType,"DEBUG")==0)
   {
    debug=TRUE;
   }
   else if(pcCommandType[0]=='#')
   {
    // Text comment or empty line
   }
   else
   {
    fFatal("Unrecognized token in %s line %d (%s)",filename,line,pcCommandType);
   }
   line++;
  }
  fclose(pfINIfile);
 }
 fIOsetpalette(); /* in case new colors have been set... */
 if (screen_fix==TRUE)
 {
  if (bGblDisplayExactScaling==FALSE)
  {

   iDivShiftAx=display_infox[wGblDisplayMode].iDivShiftA;
   iDivShiftBx=display_infox[wGblDisplayMode].iDivShiftB;
   iDivShiftAy=display_infoy[wGblDisplayMode].iDivShiftA;
   iDivShiftBy=display_infoy[wGblDisplayMode].iDivShiftB;

   iX1clip=(lwGblClipX>>iDivShiftAx) + (lwGblClipX>>iDivShiftBx);
   iY1clip=(lwGblClipY>>iDivShiftAy) + (lwGblClipY>>iDivShiftBy);
   iX2clip=(-lwGblClipX>>iDivShiftAx) + (-lwGblClipX>>iDivShiftBx);
   iY2clip=(-lwGblClipY>>iDivShiftAy) + (-lwGblClipY>>iDivShiftBy);

   mausxmin=iXOffset-((20000>>iDivShiftAx)+(20000>>iDivShiftBx));
   mausxmax=iXOffset+((20000>>iDivShiftAx)+(20000>>iDivShiftBx));
   mausymin=iYOffset-((20000>>iDivShiftAy)+(20000>>iDivShiftBy));
   mausymax=iYOffset+((20000>>iDivShiftAy)+(20000>>iDivShiftBy));
  }
  else
  {
   dividerx=divider_x[wGblDisplayMode];
   dividery=divider_y[wGblDisplayMode];

   iX1clip=lwGblClipX/dividerx;
   iY1clip=lwGblClipY/dividery;
   iX2clip=-lwGblClipX/dividerx;
   iY2clip=-lwGblClipY/dividery;

   mausxmin=iXOffset-20000/dividerx; // just in case a lightpen
   mausxmax=iXOffset+20000/dividerx; // will be enabled
   mausymin=iYOffset-20000/dividery;
   mausymax=iYOffset+20000/dividery;
  }
  iX1clip+=iXOffset;
  iY1clip+=iYOffset;
  iX2clip+=iXOffset;
  iY2clip+=iYOffset;
 }
 return TRUE;
}


SWORD get_x_line(SWORD x0,SWORD y0,SWORD x1,SWORD y1, SWORD x_searched);


void fBorderTriangleDraw(SLONG xl0, SLONG yl0, SLONG xl1, SLONG yl1, SLONG xl2, SLONG yl2,UWORD ink)
{
 SWORD x0,x1,x2,y0,y1,y2;

 SWORD bottom_x,top_x,mid_v_x;
 SWORD bottom_y,top_y,mid_v_y;
 int i;

 /**************************/
 /* Scale to screen coords */
 /**************************/
// yl0=-yl0;
// yl1=-yl1;
// yl2=-yl2;

 scale(&x0,&y0,&x1,&y1,xl0,yl0,xl1,yl1);
 scale(&x0,&y0,&x2,&y2,xl0,yl0,xl2,yl2);

 x0+=iXOffset;
 y0+=iYOffset;
 x1+=iXOffset;
 y1+=iYOffset;
 x2+=iXOffset;
 y2+=iYOffset;


 if ((y0>=y1)&&(y0>=y2))
 {
  bottom_y=y0;
  bottom_x=x0;
  if ((y1<=y0)&&(y1<=y2))
  {
   top_y=y1;
   top_x=x1;
   mid_v_y=y2;
   mid_v_x=x2;
  }
  else if ((y2<=y0)&&(y2<=y1))
  {
   top_y=y2;
   top_x=x2;
   mid_v_y=y1;
   mid_v_x=x1;
  }
 }
 else if ((y1>=y0)&&(y1>=y2))
 {
  bottom_y=y1;
  bottom_x=x1;
  if ((y0<=y1)&&(y0<=y2))
  {
   top_y=y0;
   top_x=x0;
   mid_v_y=y2;
   mid_v_x=x2;
  }
  else if ((y2<=y0)&&(y2<=y1))
  {
   top_y=y2;
   top_x=x2;
   mid_v_y=y0;
   mid_v_x=x0;
  }
 }
 else if ((y2>=y0)&&(y2>=y1))
 {
  bottom_y=y2;
  bottom_x=x2;
  if ((y0<=y1)&&(y0<=y2))
  {
   top_y=y0;
   top_x=x0;
   mid_v_y=y1;
   mid_v_x=x1;
  }
   else if ((y1<=y0)&&(y1<=y2))
  {
   top_y=y1;
   top_x=x1;
   mid_v_y=y0;
   mid_v_x=x0;
  }
 }
 for (i=top_y;i<=bottom_y;i++)
 {
  int first_x,second_x;
  first_x=get_x_line(top_x,top_y,bottom_x,bottom_y,i);
  if (i<mid_v_y)
   second_x=get_x_line(top_x,top_y,mid_v_x,mid_v_y,i);
  else
   second_x=get_x_line(mid_v_x,mid_v_y,bottom_x,bottom_y,i);
 if (first_x<second_x)
  fIODrawLine(first_x,i,second_x,i,ink);
 else
  fIODrawLine(second_x,i,first_x,i,ink);
 }
}

SWORD get_x_line(SWORD iX1,SWORD iY1,SWORD iX2,SWORD iY2, SWORD y_searched)
{
 SWORD   dx=0,dy=0,err=0,inc=0;

/**************************/
/* Do straight line cases */
/**************************/
 if (iX1==iX2)
 {
  return iX1;
 }
 else if(iY1==iY2)
 {
  return (iX1+iX2)/2;
 }
 else
 {
  /*************/
  /* X Variant */
  /*************/
  dx=iX2-iX1;
  dy=iY2-iY1;
  if(abs(dx)>=abs(dy))
  {
   if(iX1>iX2)
   {
    fSwap(&iX1,&iX2);
    fSwap(&iY1,&iY2);
   }
   dx=iX2-iX1;
   dy=iY2-iY1;
   err=-(dx>>1);
   if(dy>0)
   {
    inc=1;
   }
   else
   {
    inc=-1;
    dy=-dy;
   }
   do
   {
    /**************************/
    /* Calulate byte position */
    /**************************/
    if (y_searched==iY1)
     return iX1;
    /**********************/
    /* Check error status */
    /**********************/
    err+=dy;
    if(err>0)
    {
     err-=dx;
     iY1+=inc;
    }
   }
   while(iX1++!=iX2);
  }
  else
  {
   if(iY1>iY2)
   {
    fSwap(&iX1,&iX2);
    fSwap(&iY1,&iY2);
   }
   dx=iX2-iX1;
   dy=iY2-iY1;
   err=-(dy>>1);
   if(dx>0)
   {
    inc=1;
   }
   else
   {
    inc=-1;
    dx=-dx;
   }
   do
   {
    /**************************/
    /* Calulate byte position */
    /**************************/
    if (y_searched==iY1)
     return iX1;
    /**********************/
    /* Check error status */
    /**********************/
    err+=dx;
    if(err>=0)
    {
     err-=dy;
     iX1+=inc;
    }
   }
   while(iY1++!=iY2);
  }
 }
 return iX2;
}

void fOverlayTriangleDraw(SLONG xl0, SLONG yl0, SLONG xl1, SLONG yl1, SLONG xl2, SLONG yl2,UWORD ink)
{
 SWORD x0,x1,x2,y0,y1,y2;

 SWORD bottom_x,top_x,mid_v_x;
 SWORD bottom_y,top_y,mid_v_y;
 int i;

 /**************************/
 /* Scale to screen coords */
 /**************************/
// yl0=-yl0;
// yl1=-yl1;
// yl2=-yl2;

 scale(&x0,&y0,&x1,&y1,xl0,yl0,xl1,yl1);
 scale(&x0,&y0,&x2,&y2,xl0,yl0,xl2,yl2);

 x0+=iXOffset;
 y0+=iYOffset;
 x1+=iXOffset;
 y1+=iYOffset;
 x2+=iXOffset;
 y2+=iYOffset;

 if ((y0>=y1)&&(y0>=y2))
 {
  bottom_y=y0;
  bottom_x=x0;
  if ((y1<=y0)&&(y1<=y2))
  {
   top_y=y1;
   top_x=x1;
   mid_v_y=y2;
   mid_v_x=x2;
  }
  else if ((y2<=y0)&&(y2<=y1))
  {
   top_y=y2;
   top_x=x2;
   mid_v_y=y1;
   mid_v_x=x1;
  }
 }
 else if ((y1>=y0)&&(y1>=y2))
 {
  bottom_y=y1;
  bottom_x=x1;
  if ((y0<=y1)&&(y0<=y2))
  {
   top_y=y0;
   top_x=x0;
   mid_v_y=y2;
   mid_v_x=x2;
  }
  else if ((y2<=y0)&&(y2<=y1))
  {
   top_y=y2;
   top_x=x2;
   mid_v_y=y0;
   mid_v_x=x0;
  }
 }
 else if ((y2>=y0)&&(y2>=y1))
 {
  bottom_y=y2;
  bottom_x=x2;
  if ((y0<=y1)&&(y0<=y2))
  {
   top_y=y0;
   top_x=x0;
   mid_v_y=y1;
   mid_v_x=x1;
  }
   else if ((y1<=y0)&&(y1<=y2))
  {
   top_y=y1;
   top_x=x1;
   mid_v_y=y0;
   mid_v_x=x0;
  }
 }
 for (i=top_y;i<=bottom_y;i++)
 {
  int first_x,second_x;
  first_x=get_x_line(top_x,top_y,bottom_x,bottom_y,i);
  if (i<mid_v_y)
   second_x=get_x_line(top_x,top_y,mid_v_x,mid_v_y,i);
  else
   second_x=get_x_line(mid_v_x,mid_v_y,bottom_x,bottom_y,i);
  if (first_x<second_x)
   fOverlayDrawLine(first_x,i,second_x,i,ink);
  else
   fOverlayDrawLine(second_x,i,first_x,i,ink);
 }
}

void fOverlayDrawLine(SWORD iX1,SWORD iY1,SWORD iX2,SWORD iY2,SWORD iColour)
{
 UBYTE *dest=(UBYTE*)overlay_memory;
 SWORD   dx=0,dy=0,err=0,inc=0;

/**************************/
/* Do straight line cases */
/**************************/
 if (iX1==iX2)
 {
  if(iY1>iY2)
  {
   fSwap(&iY1,&iY2);
  }
  do
  {
   /******************************************************************************************/
   /* not to be put out of loop, offset can get to high... (high iY1 are pointing to maxline)*/
   /******************************************************************************************/
   offset=linestartpos[(UWORD)iY1]+(ULONG)iX1;
   *(dest+offset)=(UBYTE)iColour;
  }
  while(iY1++!=iY2);
 }
 else if(iY1==iY2)
 {
  if(iX1>iX2)
  {
   fSwap(&iX1,&iX2);
  }
  offset=linestartpos[(UWORD)iY1]+(ULONG)iX1;
  do
  {
   *(dest+offset)=(UBYTE)iColour;
   offset++;
  }
  while(iX1++!=iX2);
 }
 else
 {
  /*************/
  /* X Variant */
  /*************/
  dx=iX2-iX1;
  dy=iY2-iY1;
  if(abs(dx)>=abs(dy))
  {
   if(iX1>iX2)
   {
    fSwap(&iX1,&iX2);
    fSwap(&iY1,&iY2);
   }
   dx=iX2-iX1;
   dy=iY2-iY1;
   err=-(dx>>1);
   if(dy>0)
   {
    inc=1;
   }
   else
   {
    inc=-1;
    dy=-dy;
   }
   do
   {
    /**************************/
    /* Calulate byte position */
    /**************************/
    offset=linestartpos[(UWORD)iY1]+(ULONG)iX1;
    *(dest+offset)=(UBYTE)iColour;
    /**********************/
    /* Check error status */
    /**********************/
    err+=dy;
    if(err>0)
    {
     err-=dx;
     iY1+=inc;
    }
   }
   while(iX1++!=iX2);
  }
  else
  {
   if(iY1>iY2)
   {
    fSwap(&iX1,&iX2);
    fSwap(&iY1,&iY2);
   }
   dx=iX2-iX1;
   dy=iY2-iY1;
   err=-(dy>>1);
   if(dx>0)
   {
    inc=1;
   }
   else
   {
    inc=-1;
    dx=-dx;
   }
   do
   {
    /**************************/
    /* Calulate byte position */
    /**************************/
    offset=linestartpos[(UWORD)iY1]+(ULONG)iX1;
    *(dest+offset)=(UBYTE)iColour;
    /**********************/
    /* Check error status */
    /**********************/
    err+=dx;
    if(err>=0)
    {
     err-=dy;
     iX1+=inc;
    }
   }
   while(iY1++!=iY2);
  }
 }
}

/* End of DISPLAY.C */
