/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : Vector display driver module header file                   */
/*                                                                           */
/*   File Name  : DISPLAY.H                                                  */
/*                                                                           */
/*   Author     : Keith Wilkins                                              */
/*                                                                           */
/*   Version    : 1.00                                                       */
/*                                                                           */
/*   Desciption : This file contains the functions definitions & prototypes  */
/*                for the vector display driver software module.             */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    14/03/93  K.W  Creation of empty file and descriptions         */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*                                                                           */
/*****************************************************************************/

#if !defined DISPLAY_H
    #define DISPLAY_H


typedef struct sDline
{
   SWORD   iX1;
   SWORD   iY1;
   SWORD   iX2;
   SWORD   iY2;
   SWORD   brightness;
   SWORD   imager_color;
   SWORD   age;
   SWORD   next_vector;
   SWORD   hash_index;
}SDLINE;


typedef struct sHash
{
   SWORD   line_index;
   SWORD   next_hash;
   SWORD   prev_hash;
   SWORD   hash_code;
}SHASH;


typedef struct displayinfo
{
   SWORD iDivShiftA;
   SWORD iDivShiftB;
}DISPLAYINFO;


/* Define function prototypes */


BOOL    fDisplayInit();
void    fDisplayReset();
extern void    (* fDisplayTick)(void);
//void    fDisplayLineDraw(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tBright);
extern void (* fDisplayLineDraw)(SLONG,SLONG,SLONG,SLONG,UBYTE);

void    fBorderLineDraw(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tPen);
void    fBorderBlockDraw(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tPen);
void    fBorderElipseDraw(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tPen);

void    fOverlayBlockDraw(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tPen);
void    fOverlayElipseDraw(SLONG lX1,SLONG lY1,SLONG lX2,SLONG lY2,UBYTE tPen);

void    fDisplayClear(void);
BOOL    fDisplayLoadVOL(void);


extern DISPLAYINFO    display_infox[RESOLUTION_COUNT];
extern SWORD divider_x[RESOLUTION_COUNT];
extern DISPLAYINFO    display_infoy[RESOLUTION_COUNT];
extern SWORD divider_y[RESOLUTION_COUNT];

extern SWORD dividerx;
extern SWORD dividery;

extern int mausxmin;
extern int mausxmax;
extern int mausymin;
extern int mausymax;


/*******************************************/
/* Decoded copies of all configs for speed */
/*******************************************/
extern SWORD iDivShiftAx;
extern SWORD iDivShiftBx;
extern SWORD iDivShiftAy;
extern SWORD iDivShiftBy;

extern SWORD iXOffset;
extern SWORD iYOffset;

extern SWORD iX1clip;
extern SWORD iY1clip;
extern SWORD iX2clip;
extern SWORD iY2clip;

/*****************************************/
/* Variables for refresh timing & adjust */
/*****************************************/
#define    LINE_REFRESH_LIMIT  16  // Number of refreshes for a line before it can be selected
#define    REFRESH_PERIOD_ADJ  500
extern SWORD iLineSelected;
extern SWORD iLineSelector;
extern ULONG lwRefreshMarker;

/**************************/
/* Line store definitions */
/**************************/
#define    LINESTORE_DEPTH     4000
extern SDLINE line_store[LINESTORE_DEPTH];

/*****************************/
/* Hashing table definitions */
/*****************************/
extern SWORD    line_freelist;
extern SWORD    line_rootnode;

#define HASH_TABLE_SIZE    1511
#define HASH_MASK          0x03ff

extern SWORD  hash_rootnodes[HASH_TABLE_SIZE];
extern SWORD  hash_freelist;
extern SHASH  hash_store[LINESTORE_DEPTH];



/***********************/
/* Overlay definitions */
/***********************/
#define OVERLAY_LINE_COUNT 1600
#define DUFF_VALUE         -99999999
#define MAX_ARGS           16

extern UBYTE *overlay_line_starts[OVERLAY_LINE_COUNT];




#endif

/* End of DISPLAY.H */



