/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : 6522A PIA Software emulation module                        */
/*                                                                           */
/*   File Name  : EMU6522.C                                                  */
/*                                                                           */
/*   Author     : Keith Wilkins / C.S.                                       */
/*                                                                           */
/*   Version    : 1.20                                                       */
/*                                                                           */
/*   Desciption : This file contains the functions that emulate the 6522 PIA */
/*                Chip. Any emulation functions calling the read write       */
/*                functions will cause an immediate update to the PIA and    */
/*                and all associated variables. Changes occur instantly so   */
/*                there is no direct time based simulation.                  */
/*                                                                           */
/*   Functions  : f6522AInit()           - Init the 6522 PIA                 */
/*                f6522AReset()          - Reset the PIA                     */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    14/03/93  K.W  Creation of empty file and descriptions         */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*    1.20    20/10/96  C.S  Small changes here and there                    */
/*                                                                           */
/*****************************************************************************/

#define EMU6522_C

#include "vectrex.h"

/*********************************/
/* 6522 Internal state variables */
/*********************************/

#ifdef DIGI_SOUND_HACK
 short int digisound=false;
#endif


UBYTE      t6522ORB_0=0;           // Output register B - Bit 0 - Sample/Hold
UBYTE      t6522IRB_0=0;           // Input  register B - Bit 0 - Sample/Hold
UBYTE      t6522IRB_0L=0;          // Input  register B - Bit 0 - Sample/Hold (Latched)

UBYTE      t6522ORB_1=0;           // Output register B - Bit 1/2 - Analog mux select bit 0
UBYTE      t6522IRB_1=0;           // Input  register B - Bit 1/2 - Analog mux select
UBYTE      t6522IRB_1L=0;          // Input  register B - Bit 1/2 - Analog mux (Latched)

UBYTE      t6522ORB_2=0;           // Output register B - Bit 1/2 - Analog mux select bit 1
UBYTE      t6522IRB_2=0;           // Input  register B - Bit 1/2 - Analog mux select
UBYTE      t6522IRB_2L=0;          // Input  register B - Bit 1/2 - Analog mux (Latched)

UBYTE      t6522ORB_3=0;           // Output register B - Bit 3 - Sound Chip BC1
UBYTE      t6522IRB_3=0;           // Input  register B - Bit 3 - Sound Chip BC1
UBYTE      t6522IRB_3L=0;          // Input  register B - Bit 3 - Sound Chip BC1 (Latched)

UBYTE      t6522ORB_4=0;           // Output register B - Bit 4 - Sound Chip BDIR
UBYTE      t6522IRB_4=0;           // Input  register B - Bit 4 - Sound Chip BDIR
UBYTE      t6522IRB_4L=0;          // Input  register B - Bit 4 - Sound Chip BDIR (Latched)

UBYTE      t6522ORB_5=0;           // Output register B - Bit 5 - Compare
UBYTE      t6522IRB_5=0;           // Input  register B - Bit 5 - Compare
UBYTE      t6522IRB_5L=0;          // Input  register B - Bit 5 - Compare (Latched)

UBYTE      t6522ORB_6=0;           // Output register B - Bit 6 - Cartridge
UBYTE      t6522IRB_6=0;           // Input  register B - Bit 6 - Cartridge
UBYTE      t6522IRB_6L=0;          // Input  register B - Bit 6 - Cartridge (Latched)

UBYTE      t6522ORB_7REG=0;        // Output register B - Bit 6 - Ramp (register only copy)
UBYTE      t6522ORB_7=0;           // Output register B - Bit 6 - Ramp (Actual output bit)
UBYTE      t6522IRB_7=0;           // Input  register B - Bit 6 - Ramp
UBYTE      t6522IRB_7L=0;          // Input  register B - Bit 6 - Ramp (Latched)

UBYTE      t6522ORA=0;             // Output register A
UBYTE      t6522IRA=0;             // Input register A
UBYTE      t6522IRAL=0;            // Input register A latch

UBYTE      t6522DDRB=0;            // Data Direction reg B     0x00
UBYTE      t6522DDRA=0;            // Data Direction reg A     0x01

UBYTE      t6522SHFTREG=0;         // Shift Register           0x0a
UBYTE      t6522SHFTREGACT=0;

TIMER      u6522Timer1;            // Timer structures
TIMER      u6522Timer2;
BOOL       bTimer2OneShot=FALSE;

UBYTE      t6522AUX_T1CTRL=0;      // Aux register T1 Control
UBYTE      t6522AUX_T2CTRL=0;      // Aux register T2 Control
UBYTE      t6522AUX_SHFTCTRL=0;    // Aux register shift register control
UBYTE      t6522AUX_PA_LE=0;       // Aux register port A latch enable
UBYTE      t6522AUX_PB_LE=0;       // Aux register port B latch enable

UBYTE      t6522PCR_CB2CTRL=0;     // Peripheral ctrl reg CB2
UBYTE      t6522PCR_CB1CTRL=0;     // Peripheral ctrl reg CB1
UBYTE      t6522PCR_CA2CTRL=0;     // Peripheral ctrl reg CA2
UBYTE      t6522PCR_CA1CTRL=0;     // Peripheral ctrl reg CA1

UBYTE      t6522IER=0;

UBYTE      t6522IFLG=0;

UBYTE      t6522CA1=0;             // CA1 Pin
UBYTE      t6522CA2=0;             // CA2 Pin
UBYTE      t6522CB1=0;             // CB1 Pin
UBYTE      t6522CB2=0;             // CB2 Pin

/**************************/
/* some new global data's */
/**************************/
extern CPUREGISTERS  tsCPURegisters;
SWORD iToggle=0;
long tcount=0;
UBYTE (* f6522Addr0DRead)(UWORD tAddress);

/***********************************************************************/
/* Dirty little helper, small but effectiv (mainly for spike, it's the */
/* only program I know which uses T1 Interrupt on PIA)                 */
/***********************************************************************/
int t1int=FALSE;

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Init()                                               */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Initialise the 6522A emulation and register it with the   */
/*                 CPU as a memory mapped device.                            */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : BYTE TRUE/FALSE depending on init success/fail            */
/*                                                                           */
/*****************************************************************************/

BOOL f6522Init(void)
{
 UWORD        loop;
 static UBYTE tFirstTime=TRUE;

 if(tFirstTime==TRUE)
 {
  tFirstTime=FALSE;
  for(loop=0;loop<128;loop++)
  {
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR00,1,f6522Addr00Read,f6522Addr00Write)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR01,1,f6522Addr01Read,f6522Addr01Write)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR02,1,f6522Addr02Read,f6522Addr02Write)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR03,1,f6522Addr03Read,f6522Addr03Write)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR04,1,f6522Addr04Read,f6522Addr04Write)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR05,1,f6522Addr05Read,f6522Addr05Write)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR06,1,f6522Addr06Read,f6522Addr06Write)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR07,1,f6522Addr07Read,f6522Addr07Write)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR08,1,f6522Addr08Read,f6522Addr08Write)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR09,1,f6522Addr09Read,f6522Addr09Write)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR0A,1,f6522Addr0ARead,f6522Addr0AWrite)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR0B,1,f6522Addr0BRead,f6522Addr0BWrite)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR0C,1,f6522Addr0CRead,f6522Addr0CWrite)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR0D,1,f6522Addr0DRead,f6522Addr0DWrite)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR0E,1,f6522Addr0ERead,f6522Addr0EWrite)==FALSE) return(FALSE);
   if(f6809DeclareAddress((loop*16)+PIA6522_ADDR0F,1,f6522Addr0FRead,f6522Addr0FWrite)==FALSE) return(FALSE);
  }
 }
 f6522Reset();
 return(TRUE);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Reset()                                              */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Perform a hardware reset of the 6522A PIA                 */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

void f6522Reset(void)
{
 t6522ORB_0=0;
 t6522IRB_0=0;
 t6522IRB_0L=0;

 t6522ORB_1=0;
 t6522IRB_1=0;
 t6522IRB_1L=0;

 t6522ORB_2=0;
 t6522IRB_2=0;
 t6522IRB_2L=0;

 t6522ORB_3=0;
 t6522IRB_3=0;
 t6522IRB_3L=0;

 t6522ORB_4=0;
 t6522IRB_4=0;
 t6522IRB_4L=0;

 t6522ORB_5=0;
 t6522IRB_5=0;
 t6522IRB_5L=0;

 t6522ORB_6=0;
 t6522IRB_6=0;
 t6522IRB_6L=0;

 t6522ORB_7=0;
 t6522IRB_7=0;
 t6522IRB_7L=0;

 t6522ORA=0;
 t6522IRA=0;
 t6522IRAL=0;
 t6522DDRB=0;
 t6522DDRA=0;
 t6522SHFTREG=0;
 t6522SHFTREGACT=0;

 u6522Timer1.uLatch.tW=0xffff;
 u6522Timer1.uCount.tW=0;
 u6522Timer2.uLatch.tW=0xffff;
 u6522Timer2.uCount.tW=0;
 bTimer2OneShot=FALSE;

 t6522AUX_T1CTRL=0;
 t6522AUX_T2CTRL=0;
 t6522AUX_SHFTCTRL=0;
 t6522AUX_PA_LE=0;
 t6522AUX_PB_LE=0;

 t6522PCR_CB2CTRL=0;
 t6522PCR_CB1CTRL=0;
 t6522PCR_CA2CTRL=0;
 t6522PCR_CA1CTRL=0;

 t6522IER=0;
// t6522IER_CA2=0;
// t6522IER_CA1=0;
// t6522IER_SHFT=0;
// t6522IER_CB2=0;
// t6522IER_CB1=0;
// t6522IER_TIMER1=0;
// t6522IER_TIMER2=0;

 t6522IFLG=0;
// t6522IFLG_CA2=0;
// t6522IFLG_CA1=0;
// t6522IFLG_SHFT=0;
// t6522IFLG_CB2=0;
// t6522IFLG_CB1=0;
// t6522IFLG_TIMER1=0;
// t6522IFLG_TIMER2=0;

 t6522CA1=0;
 t6522CA2=0;
 t6522CB1=0;
 t6522CB2=0;
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Tick()                                               */
/*                                                                           */
/*   Author      : K.W / C.S.                                                */
/*                                                                           */
/*   Description : Perform a single tick of the 6522 counters                */
/*                 Sometimes inline...                                       */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

void f6522Tick(void)
{
 // CODE MOVED TO SEQUENCE.C MAIN LOOP
    #define PIA_TIMER1
    #define PIA_TIMER2
    #define PIA_SHIFT
#include "pia.cc"
    #undef PIA_TIMER1
    #undef PIA_TIMER2
    #undef PIA_SHIFT
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr00Read/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 'B' input/ouput register               */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr00Read(UWORD tAddress)
{
 UBYTE   tData;
 // Before we do the read update the compare register

 fAnalogJoystickUpdate();

 // Build the read byte

 tData=0;
 tData|=t6522ORB_0 & t6522DDRB;         // Add output bits
 tData|=t6522ORB_1 & t6522DDRB;         // Add output bits
 tData|=t6522ORB_2 & t6522DDRB;         // Add output bits
 tData|=t6522ORB_3 & t6522DDRB;         // Add output bits
 tData|=t6522ORB_4 & t6522DDRB;         // Add output bits
 tData|=t6522ORB_5 & t6522DDRB;         // Add output bits
 tData|=t6522ORB_6 & t6522DDRB;         // Add output bits
 tData|=t6522ORB_7 & t6522DDRB;         // Add output bits

 // Check if latched version should be used

 if (t6522AUX_PB_LE)
 {
  tData|=t6522IRB_0L & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_1L & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_2L & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_3L & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_4L & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_5L & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_6L & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_7L & (t6522DDRB^0xff);  // Add input bits
 }
 else
 {
  tData|=t6522IRB_0 & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_1 & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_2 & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_3 & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_4 & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_5 & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_6 & (t6522DDRB^0xff);  // Add input bits
  tData|=t6522IRB_7 & (t6522DDRB^0xff);  // Add input bits
 }
 fDebug(MID,("f6522Addr00Read()  - Input/Output register B read (%02x)",tData));

 // Clear interrupt flag

 if (t6522PCR_CB2CTRL!=1 && t6522PCR_CB2CTRL!=3)
  t6522IFLG&=(E6522IFLG_CB2^0xff);
 t6522IFLG&=(E6522IFLG_CB1^0xff);
 return(tData);
}

UBYTE f6522Addr00Write(UWORD tAddress,UBYTE tData)
{
 t6522ORB_0=TMPL_RB_0 & tData;      // Add output bits
 t6522ORB_1=TMPL_RB_1 & tData;      // Add output bits
 t6522ORB_2=TMPL_RB_2 & tData;      // Add output bits
 t6522ORB_3=TMPL_RB_3 & tData;      // Add output bits
 t6522ORB_4=TMPL_RB_4 & tData;      // Add output bits
 t6522ORB_5=TMPL_RB_5 & tData;      // Add output bits
 t6522ORB_6=TMPL_RB_6 & tData;      // Add output bits


//printf("\n0=%i,1=%i,2=%i",t6522ORB_0,t6522ORB_1,t6522ORB_2);


 //-- Special case of PB7 & Timer 1 overide

 t6522ORB_7REG=TMPL_RB_7 & tData;      // Always copy reg value

 // Only send to PB7 if allowed

 if ((t6522AUX_T1CTRL&0x02)==0) t6522ORB_7=t6522ORB_7REG;

 fDebug(MID,("f6522Addr00Write() - Input/Output register B write (%02x)",tData));

 // Clear interrupt flag

 if (t6522PCR_CB2CTRL!=1 && t6522PCR_CB2CTRL!=3)
  t6522IFLG&=(E6522IFLG_CB2^0xff);
 t6522IFLG&=(E6522IFLG_CB1^0xff);

 // Update analog hardware & sound chip

#ifdef DIGI_SOUND_HACK
 if (/*t6522ORB_0+*/t6522ORB_1+t6522ORB_2==2+4)
 {
  //printf("toll");
  digisound=true;
 }
 else
  digisound=false;
#endif

 fAnalogTick();
 fSoundTick();
 return(tData);
}



/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr01Read/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 'A' input/ouput register               */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr01Read(UWORD tAddress)
{
   UBYTE   tData;

   tData=t6522ORA&t6522DDRA;           // Add output bits

   // Check if latched input should be used

   if(t6522AUX_PA_LE)
   {
       tData|=t6522IRAL&(t6522DDRA^0xff);   // Add input bits
   }
   else
   {
       tData|=t6522IRA&(t6522DDRA^0xff);   // Add input bits
   }

   fDebug(MID,("f6522Addr01Read()  - Input/Output register A read (%02x)",tData));


//   *** ADD HANDSHAKING ***


   // Clear interrupt flag

   if (t6522PCR_CA2CTRL!=1 && t6522PCR_CA2CTRL!=3)
    t6522IFLG&=(E6522IFLG_CA2^0xff);
   t6522IFLG&=(E6522IFLG_CA1^0xff);

   return(tData);
}


UBYTE f6522Addr01Write(UWORD tAddress,UBYTE tData)
{

   t6522ORA=tData;

   fDebug(MID,("f6522Addr01Write() - Input/Output register A write (%02x)",tData));

   // Clear interrupt flag

   if(t6522PCR_CA2CTRL!=1 && t6522PCR_CA2CTRL!=3)
    t6522IFLG&=(E6522IFLG_CA2^0xff);
   t6522IFLG&=(E6522IFLG_CA1^0xff);

   // Update analog hardware & sound chip

   fAnalogTick();
   fSoundTick();
   return(tData);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr02Read/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 'B' data direction register            */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr02Read(UWORD tAddress)
{
   fDebug(MID,("f6522Addr02Read()  - Data Direction B read (%02x)",t6522DDRB));
   return(t6522DDRB);
}


UBYTE f6522Addr02Write(UWORD tAddress,UBYTE tData)
{
 t6522DDRB=tData;
 fDebug(MID,("f6522Addr02Write() - Data Direction B write (%02x)",t6522DDRB));
 return(t6522DDRB);
}



/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr03Read/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 'A' data direction register            */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr03Read(UWORD tAddress)
{
// tAddress=0; // Stop compiler warning
   fDebug(MID,("f6522Addr03Read()  - Data Direction A read (%02x)",t6522DDRA));
   return(t6522DDRA);
}


UBYTE f6522Addr03Write(UWORD tAddress,UBYTE tData)
{
// tAddress=0; // Stop compiler warning
   t6522DDRA=tData;
   fDebug(MID,("f6522Addr03Write() - Data Direction A write (%02x)",t6522DDRA));
   return(t6522DDRA);
}




/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr04Read/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 T1 low order latches/counter           */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr04Read(UWORD tAddress)
{
   fDebug(MID,("f6522Addr04Read()  - T1 Low order counter read (%02x)",u6522Timer1.uCount.sW.tL));

   // Clear interrupt flag

   t6522IFLG&=(E6522IFLG_TIMER1^0xff);

   return(u6522Timer1.uCount.sW.tL);
}

UBYTE f6522Addr04Write(UWORD tAddress,UBYTE tData)
{
   fDebug(MID,("f6522Addr04Write() - T1 Low order latch write (%02x)",tData));
   u6522Timer1.uLatch.sW.tL=tData;
   return(u6522Timer1.uLatch.sW.tL);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr05Read/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 T1 high order counter                  */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr05Read(UWORD tAddress)
{
   fDebug(MID,("f6522Addr05Read()  - T1 High order counter read (%02x)",u6522Timer1.uCount.sW.tH));
   return(u6522Timer1.uCount.sW.tH);
}


UBYTE f6522Addr05Write(UWORD tAddress,UBYTE tData)
{
   fDebug(MID,("f6522Addr05Write() - T1 High order latch write (%02x)",tData));
   u6522Timer1.uLatch.sW.tH=tData;
   u6522Timer1.uCount.tW=u6522Timer1.uLatch.tW;

   // Setup  timer modes if required

   switch(t6522AUX_T1CTRL)
   {
       case 2:             // ONE SHOT MODE ON PB7
           t6522ORB_7=0;
           break;

       case 3:             // FREE RUN MODE
           t6522ORB_7^=TMPL_RB_7;
           break;

       default:
           break;
   }

   // Clear interrupt flag

   t6522IFLG&=(E6522IFLG_TIMER1^0xff);

   // Update analog hardware


 fAnalogTick();
   return(u6522Timer1.uLatch.sW.tH);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr06Read/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 T1 low order latches                   */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr06Read(UWORD tAddress)
{
   fDebug(MID,("f6522Addr06Read()  - T1 Low order latch read (%02x)",u6522Timer1.uLatch.sW.tL));
   return(u6522Timer1.uLatch.sW.tL);
}

UBYTE f6522Addr06Write(UWORD tAddress,UBYTE tData)
{
   fDebug(MID,("f6522Addr06Write() - T1 Low order latch write (%02x)",tData));
   u6522Timer1.uLatch.sW.tL=tData;
   return(u6522Timer1.uLatch.sW.tL);
}




/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr07Read/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 T1 high order latches                  */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr07Read(UWORD tAddress)
{
   fDebug(MID,("f6522Addr07Read()  - T1 High order latch read (%02x)",u6522Timer1.uLatch.sW.tH));
   return(u6522Timer1.uLatch.sW.tH);
}


UBYTE f6522Addr07Write(UWORD tAddress,UBYTE tData)
{
   fDebug(MID,("f6522Addr07Write() - T1 High order latch write (%02x)",tData));
   u6522Timer1.uLatch.sW.tH=tData;
   return(u6522Timer1.uLatch.sW.tH);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr08Read/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 T2 low order latches/counters          */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr08Read(UWORD tAddress)
{
   fDebug(MID,("f6522Addr08Read()  - T2 Low order counter read (%02x)",u6522Timer2.uCount.sW.tL));

   // Clear interrupt flag

   t6522IFLG&=(E6522IFLG_TIMER2^0xff);

   return(u6522Timer2.uCount.sW.tL);
}


UBYTE f6522Addr08Write(UWORD tAddress,UBYTE tData)
{
   fDebug(MID,("f6522Addr08Write() - T2 Low order latch write (%02x)",tData));
   u6522Timer2.uLatch.sW.tL=tData;
   return(u6522Timer2.uLatch.sW.tL);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr09Read/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 T2 high order counter                  */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr09Read(UWORD tAddress)
{
   fDebug(MID,("f6522Addr06Read()  - T2 High order counter read (%02x)",u6522Timer2.uCount.sW.tH));
   return(u6522Timer2.uCount.sW.tH);
}

UBYTE f6522Addr09Write(UWORD tAddress,UBYTE tData)
{
   fDebug(MID,("f6522Addr09Write() - T2 High order counter write (%02x)",tData));
   u6522Timer2.uCount.sW.tH=tData;
   u6522Timer2.uCount.sW.tL=u6522Timer2.uLatch.sW.tL;
   bTimer2OneShot=TRUE;

   // Clear interrupt flag

   t6522IFLG&=(E6522IFLG_TIMER2^0xff);

   return(u6522Timer2.uCount.sW.tH);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr0ARead/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 Shift register                         */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr0ARead(UWORD tAddress)
{
   fDebug(MID,("f6522Addr0ARead()  - Shift register read (%02x)",t6522SHFTREG));
   // Clear interrupt flag
   t6522IFLG&=(E6522IFLG_SHFT^0xff);
   return(t6522SHFTREG);
}


UBYTE f6522Addr0AWrite(UWORD tAddress,UBYTE tData)
{
   t6522SHFTREG=tData;
   t6522SHFTREGACT=8;

   fDebug(MID,("f6522Addr0AWrite()  - Shift register write (%02x)",t6522SHFTREG));

   // Clear interrupt flag

   t6522IFLG&=(E6522IFLG_SHFT^0xff);

   return(t6522SHFTREG);
}



/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr0BRead/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 Auxiliary control register             */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr0BRead(UWORD tAddress)
{
   UBYTE tTmp;

   tTmp=(t6522AUX_T1CTRL<<6) | (t6522AUX_T2CTRL<<5) | (t6522AUX_SHFTCTRL<<2) | (t6522AUX_PB_LE<<1) | t6522AUX_PA_LE;

   fDebug(MID,("f6522Addr0BRead()  - Aux control register read (%02x)",tTmp));

   return(tTmp);
}


UBYTE f6522Addr0BWrite(UWORD tAddress,UBYTE tData)
{
   fDebug(MID,("f6522Addr0BWrite() - Aux control register write (%02x)",tData));

   t6522AUX_T1CTRL=(tData>>6)&3;
   t6522AUX_T2CTRL=(tData>>5)&1;
   t6522AUX_SHFTCTRL=(tData>>2)&7;

   // If we've switched to one-shot then PB7 must be set

   if((t6522AUX_T1CTRL&0x02)) t6522ORB_7=TMPL_RB_7; else t6522ORB_7=t6522ORB_7REG;

   // Check for an invalid shift mode

   if(t6522AUX_SHFTCTRL==3 || t6522AUX_SHFTCTRL==7)
   {
       fDebug(VERROR,("f6522Addr0BRead() - Bad shiftregister mode selected"));
   }

   t6522AUX_PB_LE=(tData>>1)&1;
   t6522AUX_PA_LE=tData&1;

   return(tData);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr0CRead/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 Peripheral control register            */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr0CRead(UWORD tAddress)
{
   UBYTE tTmp;

   tTmp=(t6522PCR_CB2CTRL<<5) | (t6522PCR_CB1CTRL<<4) | (t6522PCR_CA2CTRL<<1) | t6522PCR_CA1CTRL;

#ifdef DEBUG_MSG
   if (t6522PCR_CA1CTRL)
#endif
    fDebug(HIGH,("f6522Addr0CRead()  - Periperal control register read (%02x)",tTmp));

   return(tTmp);
}


UBYTE f6522Addr0CWrite(UWORD tAddress,UBYTE tData)
{
#ifdef DEBUG_MSG
   if (tData&1)
#endif
   fDebug(HIGH,("f6522Addr0CWrite() - Peripheral control register write (%02x)",tData));

   t6522PCR_CB2CTRL=(tData>>5)&7;
   t6522PCR_CB1CTRL=(tData>>4)&1;
   t6522PCR_CA2CTRL=(tData>>1)&7;
   t6522PCR_CA1CTRL=tData&1;

   switch(t6522PCR_CB2CTRL)
   {
       case 6:
           t6522CB2=0;
           break;
       case 7:
           t6522CB2=1;
           break;
       default:
           fDebug(VERROR,("f6522Addr0CWrite() - Illegal CB2 control mode %d",t6522PCR_CB2CTRL));
           break;
   }

   switch(t6522PCR_CA2CTRL)
   {
       case 6:
           t6522CA2=0;
           break;
       case 7:
           t6522CA2=1;
           break;
       default:
           fDebug(VERROR,("f6522Addr0CWrite() - Illegal CA2 control mode %d",t6522PCR_CA2CTRL));
           break;
   }


 fAnalogTick();

   return(tData);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr0DRead/Write()                                   */
/*                                                                           */
/*   Author      : K.W / C.S.                                                */
/*                                                                           */
/*   Description : Read/Write to 6522 Interrupt flag register                */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr0DReadNoLightpen(UWORD tAddress)
{
   UBYTE tTmp=0;

// tAddress=0; // Stop compiler warning

// tTmp =t6522IFLG_CA2&t6522IER_CA2;
// tTmp|=t6522IFLG_CA1&t6522IER_CA1;
// tTmp|=t6522IFLG_SHFT&t6522IER_SHFT;
// tTmp|=t6522IFLG_CB2&t6522IER_CB2;
// tTmp|=t6522IFLG_CB1&t6522IER_CB1;
// tTmp|=t6522IFLG_TIMER1&t6522IER_TIMER1;
// tTmp|=t6522IFLG_TIMER2&t6522IER_TIMER2;

// tTmp=(tTmp<<7) | (t6522IFLG_TIMER1<<6) | (t6522IFLG_TIMER2<<5) | (t6522IFLG_CB1<<4);
// tTmp|=tTmp  | (t6522IFLG_CB2<<3) | (t6522IFLG_SHFT<<2) | (t6522IFLG_CA1<<1) | t6522IFLG_CA2;


// Mask off unused interrupts

// If any interrupts are pending then set bit 8
   if (t6522IFLG&t6522IER)
    tTmp=0x80;

#ifdef DEBUG_MSG
   if (t6522IFLG&2)
    fDebug(MID,("f6522Addr0DRead()  - Interrupt Flag register read (read 0x2==CA1)(%02x)",tTmp));
#endif


   tTmp=tTmp|t6522IFLG;

   fDebug(MID,("f6522Addr0DRead()  - Interrupt Flag register read (%02x)",tTmp));

   return(tTmp);
}




UBYTE f6522Addr0DReadLightpen(UWORD tAddress)
{
 UBYTE tTmp=0;
 if (mouse_button)
  fAnalogTick();
// tAddress=0; // Stop compiler warning

// tTmp =t6522IFLG_CA2&t6522IER_CA2;
// tTmp|=t6522IFLG_CA1&t6522IER_CA1;
// tTmp|=t6522IFLG_SHFT&t6522IER_SHFT;
// tTmp|=t6522IFLG_CB2&t6522IER_CB2;
// tTmp|=t6522IFLG_CB1&t6522IER_CB1;
// tTmp|=t6522IFLG_TIMER1&t6522IER_TIMER1;
// tTmp|=t6522IFLG_TIMER2&t6522IER_TIMER2;

// tTmp=(tTmp<<7) | (t6522IFLG_TIMER1<<6) | (t6522IFLG_TIMER2<<5) | (t6522IFLG_CB1<<4);
// tTmp|=tTmp  | (t6522IFLG_CB2<<3) | (t6522IFLG_SHFT<<2) | (t6522IFLG_CA1<<1) | t6522IFLG_CA2;


// Mask off unused interrupts

// If any interrupts are pending then set bit 8
 if (t6522IFLG&t6522IER)
  tTmp=0x80;

#ifdef DEBUG_MSG
 if (t6522IFLG&2)
  fDebug(MID,("f6522Addr0DRead()  - Interrupt Flag register read (read 0x2==CA1)(%02x)",tTmp));
#endif

 tTmp=tTmp|t6522IFLG;
 fDebug(MID,("f6522Addr0DRead()  - Interrupt Flag register read (%02x)",tTmp));
 return(tTmp);
}


UBYTE f6522Addr0DWrite(UWORD tAddress,UBYTE tData)
{
// tAddress=0; // Stop compiler warning
   fDebug(MID,("f6522Addr0DWrite()  - Interrupt Flag register write (%02x)",tData));
   t6522IFLG&=(tData^0xff);
   return(tData);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr0ERead/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 Interrupt enable register              */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr0ERead(UWORD tAddress)
{
   UBYTE tTmp;

// tAddress=0; // Stop compiler warning

   tTmp=0x80 | t6522IER;

// tTmp=(1<<7) | (t6522IER_TIMER1<<6) | (t6522IER_TIMER2<<5) | (t6522IER_CB1<<4);
// tTmp|=tTmp  | (t6522IER_CB2<<3) | (t6522IER_SHFT<<2) | (t6522IER_CA1<<1) | t6522IER_CA2;

   fDebug(MID,("f6522Addr0ERead()  - Interrupt Enable register read (%02x)",tTmp));

   return(tTmp);
}


UBYTE f6522Addr0EWrite(UWORD tAddress,UBYTE tData)
{
// tAddress=0; // Stop compiler warning

//   fDebug(HIGH,("f6522Addr0EWrite()  - Interrupt Enable register write (%02x)",tData));

#ifdef DEBUG_MSG
   if (tData&E6522IFLG_CA1) // CA1 == Lightpen
   {
    fDebug(MID,("f6522Addr0EWrite()  - CA1 Interrupt Enable register write (%02x)",tData));
   }
#endif
   if(tData&0x80)
   {
    // Hi bits enable the relevant IE
    t6522IER|=(tData&0x7f);
   }
   else
   {
    // Hi bits disable the relevant IE
    // fixme
    if (bGblLightpenEnable==TRUE)
     t6522IER&=((tData&0x7f)^(0xff-2));
    else
     t6522IER&=((tData&0x7f)^(0xff));
    /* -2 for artmaster 1 only, since the disable CA1 is done */
    /* by clearing d00e, which is not the way it should be done */
    /* this patch should not cause any trouble with anything else */
   }

   return(tData);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Addr0FRead/Write()                                   */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Read/Write to 6522 Input/Output regster A*                */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE f6522Addr0FRead(UWORD tAddress)
{
   UBYTE   tData;

   tData=t6522ORA&t6522DDRA;           // Add output bits

   tData|=t6522IRA&(t6522DDRA^0xff);   // Add input bits

   fDebug(MID,("f6522Addr0FRead()  - Input/Output register A read (%02x)",tData));

   return(tData);
}


UBYTE f6522Addr0FWrite(UWORD tAddress,UBYTE tData)
{
   t6522ORA=tData;

   fDebug(MID,("f6522Addr0FWrite() - Input/Output register A write (%02x)",tData));

 fAnalogTick();
   fSoundTick();

   return(tData);
}

/* End of EMU6522.C */


