/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : 6522A Emulation module header file                         */
/*                                                                           */
/*   File Name  : EMU6522.H                                                  */
/*                                                                           */
/*   Author     : Keith Wilkins / C.S.                                       */
/*                                                                           */
/*   Version    : 1.20                                                       */
/*                                                                           */
/*   Desciption : This file contains the function prototypes and definitions */
/*                for the functions that emulates the 6522 PIA Chip          */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    14/03/93  K.W  Creation of empty file and descriptions         */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*    1.20    20/10/96  C.S  DVE V1.20                                       */
/*                                                                           */
/*****************************************************************************/

#if !defined EMU6522_H
# define EMU6522_H


// Typedefs

typedef struct
{
 union
 {
  UWORD   tW;
  struct
  {
   UBYTE   tL;
   UBYTE   tH;
  }sW;
 }uLatch;
 union
 {
  UWORD   tW;
  struct
  {
   UBYTE   tL;
   UBYTE   tH;
  }sW;
 }uCount;
}TIMER;

#ifndef EMU6522_C

// Define externals for other functions


#ifdef DIGI_SOUND_HACK
extern short int digisound;
#endif


extern UBYTE      t6522ORB_0;  // Output register B - Bit 0 - Sample/Hold
extern UBYTE      t6522IRB_0;  // Input  register B - Bit 0 - Sample/Hold
extern UBYTE      t6522IRB_0L;  // Input  register B - Bit 0 - Sample/Hold

extern UBYTE      t6522ORB_1;  // Output register B - Bit 1/2 - Analog mux select bit 0
extern UBYTE      t6522IRB_1;  // Input  register B - Bit 1/2 - Analog mux select
extern UBYTE      t6522IRB_1L;  // Input  register B - Bit 1/2 - Analog mux select

extern UBYTE      t6522ORB_2;  // Output register B - Bit 1/2 - Analog mux select bit 1
extern UBYTE      t6522IRB_2;  // Input  register B - Bit 1/2 - Analog mux select
extern UBYTE      t6522IRB_2L;  // Input  register B - Bit 1/2 - Analog mux select

extern UBYTE      t6522ORB_3;  // Output register B - Bit 3 - Sound Chip BC1
extern UBYTE      t6522IRB_3;  // Input  register B - Bit 3 - Sound Chip BC1
extern UBYTE      t6522IRB_3L;  // Input  register B - Bit 3 - Sound Chip BC1

extern UBYTE      t6522ORB_4;  // Output register B - Bit 4 - Sound Chip BDIR
extern UBYTE      t6522IRB_4;  // Input  register B - Bit 4 - Sound Chip BDIR
extern UBYTE      t6522IRB_4L;  // Input  register B - Bit 4 - Sound Chip BDIR

extern UBYTE      t6522ORB_5;  // Output register B - Bit 5 - Compare
extern UBYTE      t6522IRB_5;  // Input  register B - Bit 5 - Compare
extern UBYTE      t6522IRB_5L;  // Input  register B - Bit 5 - Compare

extern UBYTE      t6522ORB_6;  // Output register B - Bit 6 - Cartridge
extern UBYTE      t6522IRB_6;  // Input  register B - Bit 6 - Cartridge
extern UBYTE      t6522IRB_6L;  // Input  register B - Bit 6 - Cartridge

extern UBYTE      t6522ORB_7REG;  // Output register B - Bit 6 - Ramp
extern UBYTE      t6522ORB_7;  // Output register B - Bit 6 - Ramp
extern UBYTE      t6522IRB_7;  // Input  register B - Bit 6 - Ramp
extern UBYTE      t6522IRB_7L;  // Input  register B - Bit 6 - Ramp

extern UBYTE      t6522ORA;    // Output register A
extern UBYTE      t6522IRA;    // Input register A
extern UBYTE      t6522IRAL;            // Input register A latch

extern UBYTE      t6522DDRB;            // Data Direction reg B     0x00
extern UBYTE      t6522DDRA;            // Data Direction reg A     0x01


extern UBYTE      t6522CA1;    // CA1 Pin
extern UBYTE      t6522CA2;    // CA2 Pin
extern UBYTE      t6522CB1;    // CB1 Pin
extern UBYTE      t6522CB2;    // CB2 Pin

extern TIMER      u6522Timer1;            // Timer structures
extern TIMER      u6522Timer2;
extern BOOL       bTimer2OneShot;

extern UBYTE      t6522AUX_T1CTRL;
extern UBYTE      t6522AUX_T2CTRL;
extern UBYTE      t6522AUX_SHFTCTRL;
extern UBYTE      t6522AUX_PA_LE;       // Aux register port A latch enable
extern UBYTE      t6522AUX_PB_LE;       // Aux register port B latch enable

extern UBYTE      t6522PCR_CB2CTRL;     // Peripheral ctrl reg CB2
extern UBYTE      t6522PCR_CB1CTRL;     // Peripheral ctrl reg CB1
extern UBYTE      t6522PCR_CA2CTRL;     // Peripheral ctrl reg CA2
extern UBYTE      t6522PCR_CA1CTRL;     // Peripheral ctrl reg CA1

extern UBYTE      t6522IFLG;
extern UBYTE      t6522IER;

extern UBYTE      t6522SHFTREG;
extern UBYTE      t6522SHFTREGACT;
extern SWORD iToggle;
extern long tcount;
extern int t1int;

#endif


// Required vectrex hardware signals from the 6522

#define    E6522IFLG_CA2    0x01
#define    E6522IFLG_CA1    0x02
#define    E6522IFLG_SHFT   0x04
#define    E6522IFLG_CB2    0x08
#define    E6522IFLG_CB1    0x10
#define    E6522IFLG_TIMER2 0x20
#define    E6522IFLG_TIMER1 0x40

#define    tSIGDEF_DACBUS      t6522ORA
#define    tSIGDEF_SAMPHOLD    t6522ORB_0
#define    tSIGDEF_AMUXSEL     (t6522ORB_1 |t6522ORB_2)
#define    tSIGDEF_ZEROB       t6522CA2
#define    tSIGDEF_RAMPB       t6522ORB_7
#define    tSIGDEF_COMPARE     t6522IRB_5
#define    tSIGDEF_BLANKB      t6522CB2
#define    tSIGDEF_AY3BDIR     t6522ORB_4
#define    tSIGDEF_AY3BC1      t6522ORB_3
#define    tSIGDEF_SW7         t6522CA1
#define    tSIGDEF_CARTRIDGE   t6522IRB_6

#define    PIA6522_ADDR00  0xd000
#define    PIA6522_ADDR01  0xd001
#define    PIA6522_ADDR02  0xd002
#define    PIA6522_ADDR03  0xd003
#define    PIA6522_ADDR04  0xd004
#define    PIA6522_ADDR05  0xd005
#define    PIA6522_ADDR06  0xd006
#define    PIA6522_ADDR07  0xd007
#define    PIA6522_ADDR08  0xd008
#define    PIA6522_ADDR09  0xd009
#define    PIA6522_ADDR0A  0xd00a
#define    PIA6522_ADDR0B  0xd00b
#define    PIA6522_ADDR0C  0xd00c
#define    PIA6522_ADDR0D  0xd00d
#define    PIA6522_ADDR0E  0xd00e
#define    PIA6522_ADDR0F  0xd00f

#define    MASK_RB_0           0xfe
#define    TMPL_RB_0           0x01

#define    MASK_RB_1           0xfd
#define    TMPL_RB_1           0x02

#define    MASK_RB_2           0xfb
#define    TMPL_RB_2           0x04

#define    MASK_RB_3           0xf7
#define    TMPL_RB_3           0x08

#define    MASK_RB_4           0xef
#define    TMPL_RB_4           0x10

#define    MASK_RB_5           0xdf
#define    TMPL_RB_5           0x20

#define    MASK_RB_6           0xbf
#define    TMPL_RB_6           0x40

#define    MASK_RB_7           0x7f
#define    TMPL_RB_7           0x80



/* Declare the public functions for emu6522.c */

BOOL       f6522Init(void);
void       f6522Reset(void);
void       f6522Tick(void);

UBYTE      f6522Addr00Read(UWORD tAddress);
UBYTE      f6522Addr00Write(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr01Read(UWORD tAddress);
UBYTE      f6522Addr01Write(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr02Read(UWORD tAddress);
UBYTE      f6522Addr02Write(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr03Read(UWORD tAddress);
UBYTE      f6522Addr03Write(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr04Read(UWORD tAddress);
UBYTE      f6522Addr04Write(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr05Read(UWORD tAddress);
UBYTE      f6522Addr05Write(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr06Read(UWORD tAddress);
UBYTE      f6522Addr06Write(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr07Read(UWORD tAddress);
UBYTE      f6522Addr07Write(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr08Read(UWORD tAddress);
UBYTE      f6522Addr08Write(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr09Read(UWORD tAddress);
UBYTE      f6522Addr09Write(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr0ARead(UWORD tAddress);
UBYTE      f6522Addr0AWrite(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr0BRead(UWORD tAddress);
UBYTE      f6522Addr0BWrite(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr0CRead(UWORD tAddress);
UBYTE      f6522Addr0CWrite(UWORD tAddress,UBYTE tData);

extern UBYTE      (* f6522Addr0DRead)(UWORD tAddress);
UBYTE      f6522Addr0DReadNoLightpen(UWORD tAddress);
UBYTE      f6522Addr0DReadLightpen(UWORD tAddress);
UBYTE      f6522Addr0DWrite(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr0ERead(UWORD tAddress);
UBYTE      f6522Addr0EWrite(UWORD tAddress,UBYTE tData);

UBYTE      f6522Addr0FRead(UWORD tAddress);
UBYTE      f6522Addr0FWrite(UWORD tAddress,UBYTE tData);


#endif

/* End of EMU6522.H */



