/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : 6809 Emulator module header file (prototype & declaration) */
/*                                                                           */
/*   File Name  : EMU6809.H                                                  */
/*                                                                           */
/*   Author     : Keith Wilkins / C.S.                                       */
/*                                                                           */
/*   Version    : 1.20                                                       */
/*                                                                           */
/*   Desciption : This file contains all of the function prototypes and      */
/*                definitions for the 6809 emulator module.                  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    14/03/93  K.W  Creation of empty file.                         */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*    1.20    20/10/96  C.S  DVE V1.20                                       */
/*                                                                           */
/*****************************************************************************/

#if !defined EMU6809_H
    #define EMU6809_H

/* Useful defines */

#define CPU_MEMORY_SIZE     65536
#define CPU_MAX_HANDLERS    20

#define CPU_UNMAPPED_FILL   0xff


//
// Define a static memory space for "straight" memory applications
//

#ifdef EMU6809_C
  UBYTE atSystemImage[CPU_MEMORY_SIZE];
#else
  extern UBYTE atSystemImage[CPU_MEMORY_SIZE];
#endif

// **********************************************
//
// 6809 CPU OPERATING MODES
//
// **********************************************
//
// ALL CHANGES MUST BE REFLECTED IN COD6809.ASM
//
// **********************************************

#define  CPU_FETCH         0x0000
#define  CPU_WAIT          0x00f7
#define  CPU_HALT          0x00f8
#define  CPU_DECODE        0x00f9
#define  CPU_CWAI          0x00fa
#define  CPU_SYNC          0x00fb
#define  CPU_EXCEPTION     0x00fc



// **********************************************
//
// DEFINE CC MASKS
//
// **********************************************

#define CC_C_FLAG   0x01        /* Carry       */
#define CC_V_FLAG   0x02        /* Overflow    */
#define CC_Z_FLAG   0x04        /* Zero        */
#define CC_N_FLAG   0x08        /* Negative    */
#define CC_I_FLAG   0x10        /* IRQ Mask    */
#define CC_H_FLAG   0x20        /* Half Carry  */
#define CC_F_FLAG   0x40        /* FIRQ Mask   */
#define CC_E_FLAG   0x80        /* Entire Flag */

#define CC_C_FLAGB  0xFE        /* Carry       */
#define CC_V_FLAGB  0xFD        /* Overflow    */
#define CC_Z_FLAGB  0xFB        /* Zero        */
#define CC_N_FLAGB  0xF7        /* Negative    */
#define CC_I_FLAGB  0xEF        /* IRQ Mask    */
#define CC_H_FLAGB  0xDF        /* Half Carry  */
#define CC_F_FLAGB  0xBF        /* FIRQ Mask   */
#define CC_E_FLAGB  0x7F        /* Entire Flag */


/* Define the structures used in CPU emulation */

// Old struct now unused

typedef struct tsCPUMEMORYHANDLER
{
    UBYTE  (*pfReadFunc)(UWORD);
    UBYTE  (*pfWriteFunc)(UWORD,UBYTE);
    ULONG  tReadCount;
    ULONG  tWriteCount;
} tsCPUMEMORYHANDLER;


// **********************************************
//
// CPU Register structure
//
// **********************************************
//
// ALL CHANGES MUST BE REFLECTED IN COD6809.ASM
//
//
// NEVER READ CC DIRECTLY AS IT MUST BE CONVERTED
// TO 6809 FORM (from 8086) BEFORE BEING USED
//
// **********************************************

typedef struct CPUREGISTERS
{
   UWORD       tX;         /* X  - index register           */
   UWORD       tPacking1;

   UWORD       tY;         /* Y  - index register           */
   UWORD       tPacking2;

   UWORD       tUSP;       /* U  - User Stack Pointer       */
   UWORD       tPacking3;

   UWORD       tHSP;       /* S  - Hardware Stack Pointer   */
   UWORD       tPacking4;

   UWORD       tPC;        /* PC - Program Counter          */
   UWORD       tPacking5;

   union
   {
       UWORD   tD;         /* D  - Register = A(MSB)+B(LSB) */

       struct
       {
           UBYTE   tB;     /* B  - Accumulator              */
           UBYTE   tA;     /* A  - Accumulator              */
       }sAB;
   }uABD;
   UWORD       tPacking6;

   UBYTE       tDP;        /* DP - Direct Page Register     */
   UBYTE       tPacking7;  /* Used for LWORD alignment      */
   UWORD       tPacking8;

   union
   {
      ULONG       tCC;     /* CC - Condition Code Register  */

      struct
      {
         ULONG bC  :1;       // 6809-C   CARRY
         ULONG bSP1:1;       //          SPARE1 - Always 1
         ULONG bPAR:1;       //          PARITY
         ULONG bSP2:1;       //          SPARE2 - Always 0
         ULONG bH  :1;       // 6809-H   AUX CARRY
         ULONG bSP3:1;       //          SPARE3 - Always 0
         ULONG bZ  :1;       // 6809-Z   ZERO FLAG
         ULONG bN  :1;       // 6809-N   SIGN FLAG
         ULONG bTRP:1;       //          TRAP FLAG
         ULONG bIEN:1;       //          INTERRUPT ENABLE
         ULONG bDIR:1;       //          DIRECTION FLAG
         ULONG bV  :1;       // 6809-V   OVERFLOW FLAG
         ULONG bIOP:2;       //          IO PRIVELIDGE FLAG
         ULONG bNT :1;       //          NESTED TASK FLAG
         ULONG bSP4:1;       //          SPARE4 - Always 0
         ULONG bSP5:16;      // Unused upper word
      }sCC;
   }uCC8086;

   union
   {
      UWORD       tCC;    /* CC - Code condition register */

      struct
      {
         UWORD bC  :1;
         UWORD bV  :1;
         UWORD bZ  :1;
         UWORD bN  :1;
         UWORD bI  :1;
         UWORD bH  :1;
         UWORD bF  :1;
         UWORD bE  :1;
      }sCC;
   }uCC;
   UWORD       tPacking9;

   BOOL           tNMIFlag;   /* Hardware NMI Line status */
   BOOL           tIRQFlag;   /* Hardware IRQ Line status */
   BOOL           tFIRQFlag;  /* Hardware FIRQ Line status */

}CPUREGISTERS;


/* Function prototypes */

BOOL          f6809Init(void);
void          f6809Reset(void);
void          f6809Tick(void);
//void          f6809MultiTick(void);
UWORD         f6809LastInsAddr(void);
UWORD         f6809StatusRead(void);
void          f6809StatusWrite(UWORD tStat);
void          f6809SetBreakpoint(UWORD tBreakpoint,UWORD tSkip);
void          f6809ClearBreakpoint(void);
void          f6809ReadBreakpoint(UWORD *tBreakpoint,UWORD *tSkip,UWORD *tEnable);
BOOL          f6809DeclareAddress(UWORD tAddrBase,UWORD tSize,UBYTE (*pfReadFunc)(UWORD),UBYTE (*pfWriteFunc)(UWORD,UBYTE));

#define       f6809FastAddressRead(addr)   (atSystemImage[addr])

UBYTE         f6809AddressRead(UWORD tAddr);
#pragma aux   f6809AddressRead parm [AX];
#pragma aux   f6809AddressRead value [DL];

UBYTE          f6809AddressWrite(UWORD tAddr,UBYTE tData);
#pragma aux   f6809AddressWrite parm [AX] [DL];
#pragma aux   f6809AddressWrite value [DL];

void          f6809RegistersRead(CPUREGISTERS *pts6809Registers);
void          f6809RegistersWrite(CPUREGISTERS *pts6809Registers);

// ASM Functions

void     f6809Init8086CC(void);

UWORD    f6809IRQ(void);
UWORD    f6809FIRQ(void);

extern UWORD  iInstruction;

#ifdef DEBUG_MON
extern BOOL   iBPTriggered;
#endif


extern SWORD  iCount;

extern UWORD i6809status;
extern UWORD iLastAddress;
extern UWORD iBreakpoint;
extern UWORD iBreakpointEnable;
extern UWORD iSkipCount;

extern CPUREGISTERS  tsCPURegisters;
extern const UWORD(*apfExecuteInstruction[0x300])(void);
extern const SWORD packed_cycle_count[];
extern SWORD cycle_count[];



#endif




/* End of EMU6809.H */

