/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : Vectrex emulator text/graphics I/O functions header file   */
/*                                                                           */
/*   File Name  : IO.C                                                       */
/*                                                                           */
/*   Author     : Keith Wilkins & Christopher Salomon                        */
/*                                                                           */
/*   Version    : 1.20                                                       */
/*                                                                           */
/*   Desciption : This file contains the function prototypes and definitions */
/*                for the vectrex input/output system.                       */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    20/03/93  K.W  Creation                                        */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*    1.20    03/10/96  C.S  Some changes, look at io.c                      */
/*                                                                           */
/*****************************************************************************/

#if !defined IO_H
    #define IO_H

/**************************************************************************/
/* needed for scitech svgalib, otherwise windows.h included, bad for BOOL */
/**************************************************************************/
#ifndef __MSDOS__
#define __MSDOS__
#include <svga.h>
#include <pmode.h>
#undef __MSDOS__
#else
#include <svga.h>
#include <pmode.h>
#endif


/************************/
/* Defines and typedefs */
/************************/

#define FONT_MIN_SIZE   1

#define FONT_SMALL      1
#define FONT_NORMAL     2
#define FONT_LARGE      3
#define FONT_ELARGE     4

#define FONT_MAX_SIZE   4

#define BASE_FONT_WIDTH  6
#define BASE_FONT_HEIGHT 8

#define BOX_FILLED  TRUE
#define BOX_OUTLINE FALSE

#define BRIGHT0     0
#define BRIGHT1     1
#define BRIGHT2     2
#define BRIGHT3     3
#define BRIGHT4     4
#define BRIGHT5     5
#define BRIGHT6     6
#define BRIGHT7     7
#define BRIGHT8     8
#define BRIGHT9     9
#define BRIGHT10    10
#define BRIGHT11    11
#define BRIGHT12    12
#define BRIGHT13    13
#define BRIGHT14    14
#define BRIGHT15    15

#define WHITE_BASE  0
#define RED_BASE    0x10
#define GREEN_BASE  0x20
#define BLUE_BASE   0x30

#define   IO_PRINT_BUF     1600

/******************************************************/
/* wrap around for negativ offsets (memory enough...) */
/******************************************************/
#define   MAX_SCREEN_LINES 65536

/*********************/
/* kind of VESA mode */
/*********************/
#define FB_NONE 0
#define FB_LINEAR 1
#define FB_BANKED 2

/****************************/
/* Used by VESA mode setups */
/****************************/
typedef struct PIXEL
{
   int  iXpos;
   int  iYpos;
   int  iColour;
}PIXEL;

typedef struct BITBLOCK
{
   ULONG   tBitblockMagicNumber1;  /* Only for check bloc validity */
   ULONG   tBitblockMagicNumber2; 
   SWORD   width;
   SWORD   height;
   UBYTE   *tBlockData;
}BITBLOCK,*BITBLOCKPTR;

/***********************************/
/* SuperVGA mode information block */
/***********************************/
typedef struct {
    short   ModeAttributes;         /* Mode attributes                  */
    char    WinAAttributes;         /* Window A attributes              */
    char    WinBAttributes;         /* Window B attributes              */
    short   WinGranularity;         /* Window granularity in k          */
    short   WinSize;                /* Window size in k                 */
    short   WinASegment;            /* Window A segment                 */
    short   WinBSegment;            /* Window B segment                 */
    short   WinFuncOffset;          /* Pointer to window function       */
    short   WinFuncSegment;         /* Pointer to window function       */
    short   BytesPerScanLine;       /* Bytes per scanline               */
    short   XResolution;            /* Horizontal resolution            */
    short   YResolution;            /* Vertical resolution              */
    char    XCharSize;              /* Character cell width             */
    char    YCharSize;              /* Character cell height            */
    char    NumberOfPlanes;         /* Number of memory planes          */
    char    BitsPerPixel;           /* Bits per pixel                   */
    char    NumberOfBanks;          /* Number of CGA style banks        */
    char    MemoryModel;            /* Memory model type                */
    char    BankSize;               /* Size of CGA style banks          */
    char    NumberOfImagePages;     /* Number of images pages           */
    char    res1;                   /* Reserved                         */
    char    RedMaskSize;            /* Size of direct color red mask    */
    char    RedFieldPosition;       /* Bit posn of lsb of red mask      */
    char    GreenMaskSize;          /* Size of direct color green mask  */
    char    GreenFieldPosition;     /* Bit posn of lsb of green mask    */
    char    BlueMaskSize;           /* Size of direct color blue mask   */
    char    BlueFieldPosition;      /* Bit posn of lsb of blue mask     */
    char    RsvdMaskSize;           /* Size of direct color res mask    */
    char    RsvdFieldPosition;      /* Bit posn of lsb of res mask      */
    char    DirectColorModeInfo;    /* Direct color mode attributes     */
    char    res2[216];              /* Pad to 256 byte block size       */
    } ModeInfoBlock;

typedef struct DOS4GW_RMREGS
{
        ULONG           edi;
        ULONG           esi;
        ULONG           ebp;
        ULONG           reserved;
        ULONG           ebx;
        ULONG           edx;
        ULONG           ecx;
        ULONG           eax;
        UWORD           flags;
        UWORD           es,ds,fs,gs,ip,cs,sp,ss;
} DOS4GW_RMREGS;

typedef struct RealPointer
{
        UWORD                   Segment;
        UWORD                   Selector;
} REALPTR;

/**************************/
/* Stuff for PCX handling */
/**************************/

#define BILD_UNBEKANNT 102
#define BILD_IST_256_FARBE_BILD 100
#define LOAD_BUFFER_SIZE 5000
#define FROM_FILE_END 2
#define FROM_FILE_HERE 1
#define FROM_FILE_START 0

 typedef struct {
                 UBYTE red;
                 UBYTE green;
                 UBYTE blue;
               } RGB;
 typedef struct {
                 RGB rgb_farben[256];
                 }FARB_TABELLE_256;
 typedef struct{
               UBYTE hersteller;
               UBYTE version;
               UBYTE kodierungsart;
               UBYTE bits_pro_pixel;
               UWORD xmin;
               UWORD ymin;
               UWORD xmax;
               UWORD ymax;
               UWORD aufloesung_x;
               UWORD aufloesung_y;
                      RGB rgb_farben[16];
               UBYTE reserviert;
               UBYTE anzahl_farbebenen;
               UWORD bytes_pro_zeile;
               UWORD farb_art;
               UBYTE fuell_werte[58];
               }PCX_HEADER;


/******************************/
/* Define function prototypes */
/******************************/
BOOL       fIOInit(void);
BOOL       fIORestore(void);

PIXEL      fIOOutChar(SWORD iXpos,SWORD iYpos,SWORD iColour,SWORD iBackground,SWORD FontSize,char cData);
PIXEL      fIOOutText(SWORD iXpos,SWORD iYpos,SWORD iColour,SWORD iBackground,SWORD FontSize,char *pcTextBuf);
PIXEL      fIOPrintf(SWORD iXpos,SWORD iYpos,SWORD iColour,SWORD iBackground,SWORD FontSize,char *pcTextFormat,...);
PIXEL      fIOGetLine(SWORD iXpos,SWORD iYpos,SWORD iColour,SWORD iBackground,SWORD iFontSize,char *pcTextBuf,UBYTE tMaxLength);
void       fIOSetClipping(SWORD iX1,SWORD iY1,SWORD iX2,SWORD iY2);
PIXEL      fIOSetCursorPos(SWORD iXpos,SWORD iYpos,SWORD iColour);
PIXEL      fIOGetScreenRes(void);
PIXEL      fIOGetCursorPos(void);
UBYTE*     fIOGetOverlayBuffer(void);
UBYTE*     fIOGetClipBuffer(void);
void       fIODrawBox(SWORD iX1,SWORD iY1,SWORD iX2,SWORD iY2,SWORD iColour,BOOL tSolid);
BOOL       fIOAreaMove(SWORD iXsrc1,SWORD iYsrc1,SWORD iXsrc2,SWORD iYsrc2,SWORD iXdest,SWORD iYdest);
BOOL       fIOAreaScroll(SWORD iXsrc1,SWORD iYsrc1,SWORD iXsrc2,SWORD iYsrc2,SWORD iXdest,SWORD iYdest,SWORD iBackgroundColour);
void       fIOAreaClear(SWORD iX1,SWORD iY1,SWORD iX2,SWORD iY2,SWORD iBackgroundColour);
void       fIOBitBlockRelease(BITBLOCKPTR tBlock);
void       fIOsetink(UWORD iInk,UBYTE ubRed,UBYTE ubGreen,UBYTE ubBlue);
void       fIOsetpalette(void);

void fIOUnSetClipBox(int iXpos,int iYpos,int iXsize,int iYsize);
void fIOSetClipBox(int iXpos,int iYpos,int iXsize,int iYsize);

UWORD      fIOload_pcx_overlay(char *filename);
UWORD      fIOpcx_write_256pic(void);
void vesa_mouse_update_bank(void);


/*****************************************************************/
/* Chris: Had to 'globalize' some Variables since I do 'sort of' */
/*        inline linedrawing...                                  */
/*****************************************************************/
extern SWORD iSWTmp;
extern UWORD rectengle_clipping_enable;
extern SV_devCtx *svdc;
extern int offset;
extern UBYTE* overlay_memory;
extern UBYTE* clip_memory;
extern ULONG linestartpos[MAX_SCREEN_LINES];
extern int framebuffer;

extern UWORD vesa_offset_compare;
extern UWORD vesa_segment_of_window[2];
extern UWORD vesa_address_of_window[2];
extern UWORD vesa_granularity;
extern UWORD vesa_granularity_potenz;
extern UWORD vesa_and_x;
extern ULONG vesa_write_seg;
extern ULONG vesa_read_seg;
extern UWORD vesa_read_window;
extern UWORD vesa_write_window;
extern UWORD WinFuncSegment,WinFuncOffset;
extern UWORD offset_x;
extern UWORD new_win_address;
extern void (*vesa_set_write_window)(void);
extern void (*vesa_set_read_window)(void);

extern volatile ULONG mouse_code;
extern volatile ULONG mouse_x;
extern volatile ULONG vmouse_y;
extern volatile ULONG vmouse_x;
extern volatile ULONG mouse_y;
extern volatile ULONG mouse_button;
extern volatile int mchanged;

#define fSwap(a,b)                 \
    {iSWTmp=*(a);                  \
    *(a)=*(b);                     \
    *(b)=iSWTmp;}

#define fSwapWord fSwap


extern void      (* fIOPlotPixel)(SWORD ,SWORD ,SWORD);
extern void      (* fIODrawMaskLine)(SWORD,SWORD,SWORD,SWORD,SWORD);
extern void      (* fIODrawLine)(SWORD,SWORD,SWORD,SWORD,SWORD);
extern void      (* fIOcopyvesascreen)(UBYTE *,SLONG,SLONG,SLONG);
extern BOOL      (* fIOAreaLoad)(SWORD,SWORD,BITBLOCK *);
extern BITBLOCK* (* fIOAreaSave)(SWORD,SWORD,SWORD,SWORD);
extern UWORD     (* fIOInitMouse)(void);

#endif
/* End of IO.H */



