/*****************************************************************************/
/*                                                                           */
/*   LIBRARY Functions - Scan code definitions        Copyright K.W 1996     */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : Keyboard - scan code to key code conversion                */
/*                                                                           */
/*   File Name  : KEYCONV.H                                                  */
/*                                                                           */
/*   Author     : Keith Wilkins                                              */
/*                                                                           */
/*   Version    : 1.00                                                       */
/*                                                                           */
/*   Desciption : This header file defines all of the scan code definitions  */
/*                                                                           */
/*   Functions  : None                                                       */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    13/11/94  K.W  Creation                                        */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*                                                                           */
/*****************************************************************************/

#ifndef KEYCONV_H
    #define KEYCONV_H


typedef struct sKeyConv
{
   SWORD iExtended;
   SWORD iKeyCode;
}KEYCONV;

// The +1 is for a terminating -1,-1 record

#define KEY_xxxxxxxxxxx    KEY_SPACE

const KEYCONV asScanToKeyConvert[(4*NO_OF_SCANMAPS)+1]=
{

   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_BASE        0x00 */
   FALSE, KEY_ESCAPE      ,  /* SCAN_ESC         0x01 */
   FALSE, KEY_1           ,  /* SCAN_1           0x02 */
   FALSE, KEY_2           ,  /* SCAN_2           0x03 */
   FALSE, KEY_3           ,  /* SCAN_3           0x04 */
   FALSE, KEY_4           ,  /* SCAN_4           0x05 */
   FALSE, KEY_5           ,  /* SCAN_5           0x06 */
   FALSE, KEY_6           ,  /* SCAN_6           0x07 */
   FALSE, KEY_7           ,  /* SCAN_7           0x08 */
   FALSE, KEY_8           ,  /* SCAN_8           0x09 */
   FALSE, KEY_9           ,  /* SCAN_9           0x0a */
   FALSE, KEY_0           ,  /* SCAN_0           0x0b */
   FALSE, KEY_MINUS       ,  /* SCAN_MINUS       0x0c */
   FALSE, KEY_EQUALS      ,  /* SCAN_EQUALS      0x0d */
   FALSE, KEY_BSPACE      ,  /* SCAN_BSPACE      0x0e */
   FALSE, KEY_TAB         ,  /* SCAN_TAB         0x0f */

   FALSE, KEY_q           ,  /* SCAN_Q           0x10 */
   FALSE, KEY_w           ,  /* SCAN_W           0x11 */
   FALSE, KEY_e           ,  /* SCAN_E           0x12 */
   FALSE, KEY_r           ,  /* SCAN_R           0x13 */
   FALSE, KEY_t           ,  /* SCAN_T           0x14 */
   FALSE, KEY_y           ,  /* SCAN_Y           0x15 */
   FALSE, KEY_u           ,  /* SCAN_U           0x16 */
   FALSE, KEY_i           ,  /* SCAN_I           0x17 */
   FALSE, KEY_o           ,  /* SCAN_O           0x18 */
   FALSE, KEY_p           ,  /* SCAN_P           0x19 */
   FALSE, KEY_LSQRB       ,  /* SCAN_LSQRB       0x1a */
   FALSE, KEY_RSQRB       ,  /* SCAN_RSQRB       0x1b */
   FALSE, KEY_ENTER       ,  /* SCAN_ENTER       0x1c */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_CTRL        0x1d */
   FALSE, KEY_a           ,  /* SCAN_A           0x1e */
   FALSE, KEY_s           ,  /* SCAN_S           0x1f */

   FALSE, KEY_d           ,  /* SCAN_D           0x20 */
   FALSE, KEY_f           ,  /* SCAN_F           0x21 */
   FALSE, KEY_g           ,  /* SCAN_G           0x22 */
   FALSE, KEY_h           ,  /* SCAN_H           0x23 */
   FALSE, KEY_j           ,  /* SCAN_J           0x24 */
   FALSE, KEY_k           ,  /* SCAN_K           0x25 */
   FALSE, KEY_l           ,  /* SCAN_L           0x26 */
   FALSE, KEY_SCOLON      ,  /* SCAN_SCOLON      0x27 */
   FALSE, KEY_APSTPY      ,  /* SCAN_APSTPY      0x28 Apostrophy ''''' */
   FALSE, KEY_TILDE       ,  /* SCAN_TILDE       0x29 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_LSHIFT      0x2a */
   FALSE, KEY_HASH        ,  /* SCAN_HASH        0x2b */
   FALSE, KEY_z           ,  /* SCAN_Z           0x2c */
   FALSE, KEY_x           ,  /* SCAN_X           0x2d */
   FALSE, KEY_c           ,  /* SCAN_C           0x2e */
   FALSE, KEY_v           ,  /* SCAN_V           0x2f */

   FALSE, KEY_b           ,  /* SCAN_B           0x30 */
   FALSE, KEY_n           ,  /* SCAN_N           0x31 */
   FALSE, KEY_m           ,  /* SCAN_M           0x32 */
   FALSE, KEY_COMMA       ,  /* SCAN_COMMA       0x33 */
   FALSE, KEY_STOP        ,  /* SCAN_STOP        0x34 */
   FALSE, KEY_FSLASH      ,  /* SCAN_FSLASH      0x35 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_RSHIFT      0x36 */
   FALSE, KEY_STAR        ,  /* SCAN_STAR        0x37 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_ALT         0x38 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x39 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_CAPS        0x3a */
   TRUE , KEY_F1          ,  /* SCAN_F1          0x3b */
   TRUE , KEY_F2          ,  /* SCAN_F2          0x3c */
   TRUE , KEY_F3          ,  /* SCAN_F3          0x3d */
   TRUE , KEY_F4          ,  /* SCAN_F4          0x3e */
   TRUE , KEY_F5          ,  /* SCAN_F5          0x3f */

   TRUE , KEY_F6          ,  /* SCAN_F6          0x40 */
   TRUE , KEY_F7          ,  /* SCAN_F7          0x41 */
   TRUE , KEY_F8          ,  /* SCAN_F8          0x42 */
   TRUE , KEY_F9          ,  /* SCAN_F9          0x43 */
   TRUE , KEY_F10         ,  /* SCAN_F10         0x44 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_NUMLCK      0x45 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_SCRLCK      0x46 */
   TRUE , KEY_HOME        ,  /* SCAN_HOME        0x47 */
   TRUE , KEY_UP          ,  /* SCAN_UP          0x48 */
   TRUE , KEY_PGUP        ,  /* SCAN_PGUP        0x49 */
   FALSE, KEY_MINUS       ,  /* SCAN_DASH        0x4a Number pad minus */
   TRUE , KEY_LEFT        ,  /* SCAN_LEFT        0x4b */
   FALSE, KEY_5           ,  /* SCAN_CENTRE      0x4c Number pad centre */
   TRUE , KEY_RIGHT       ,  /* SCAN_RIGHT       0x4d */
   FALSE, KEY_PLUS        ,  /* SCAN_PLUS        0x4e Number pad plus */
   TRUE , KEY_END         ,  /* SCAN_END         0x4f */

   TRUE , KEY_DOWN        ,  /* SCAN_DOWN        0x50 */
   TRUE , KEY_PGDN        ,  /* SCAN_PGDN        0x51 */
   TRUE , KEY_INS         ,  /* SCAN_INS         0x52 */
   TRUE , KEY_DEL         ,  /* SCAN_DEL         0x53 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x54 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x55 */
   FALSE, KEY_BSLASH      ,  /* SCAN_BSLASH      0x56 */
   TRUE , KEY_F11         ,  /* SCAN_F11         0x57 */
   TRUE , KEY_F12         ,  /* SCAN_F12         0x58 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x59 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5f */

   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x60 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x61 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x62 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x63 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x64 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x65 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x66 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x67 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x68 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x69 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6f */

   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x70 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x71 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x72 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x73 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x74 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x75 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x76 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x77 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x78 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x79 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7f */


//
// SHIFT Versions
//

   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_BASE        0x00 */
   FALSE, KEY_ESCAPE      ,  /* SCAN_ESC         0x01 */
   FALSE, KEY_PLING       ,  /* SCAN_1           0x02 */
   FALSE, KEY_QUOTE       ,  /* SCAN_2           0x03 */
   FALSE, KEY_HASH        ,  /* SCAN_3           0x04 */
   FALSE, KEY_HASH        ,  /* SCAN_4           0x05 */
   FALSE, KEY_DOLLAR      ,  /* SCAN_5           0x06 */
   FALSE, KEY_HAT         ,  /* SCAN_6           0x07 */
   FALSE, KEY_AMPSND      ,  /* SCAN_7           0x08 */
   FALSE, KEY_STAR        ,  /* SCAN_8           0x09 */
   FALSE, KEY_OPNBKT      ,  /* SCAN_9           0x0a */
   FALSE, KEY_CLSBKT      ,  /* SCAN_0           0x0b */
   FALSE, KEY_USCORE      ,  /* SCAN_MINUS       0x0c */
   FALSE, KEY_PLUS        ,  /* SCAN_EQUALS      0x0d */
   FALSE, KEY_BSPACE      ,  /* SCAN_BSPACE      0x0e */
   FALSE, KEY_SHIFT_TAB   ,  /* SCAN_TAB         0x0f */

   FALSE, KEY_Q           ,  /* SCAN_Q           0x10 */
   FALSE, KEY_W           ,  /* SCAN_W           0x11 */
   FALSE, KEY_E           ,  /* SCAN_E           0x12 */
   FALSE, KEY_R           ,  /* SCAN_R           0x13 */
   FALSE, KEY_T           ,  /* SCAN_T           0x14 */
   FALSE, KEY_Y           ,  /* SCAN_Y           0x15 */
   FALSE, KEY_U           ,  /* SCAN_U           0x16 */
   FALSE, KEY_I           ,  /* SCAN_I           0x17 */
   FALSE, KEY_O           ,  /* SCAN_O           0x18 */
   FALSE, KEY_P           ,  /* SCAN_P           0x19 */
   FALSE, KEY_LCURLY      ,  /* SCAN_LSQRB       0x1a */
   FALSE, KEY_RCURLY      ,  /* SCAN_RSQRB       0x1b */
   FALSE, KEY_ENTER       ,  /* SCAN_ENTER       0x1c */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_CTRL        0x1d */
   FALSE, KEY_A           ,  /* SCAN_A           0x1e */
   FALSE, KEY_S           ,  /* SCAN_S           0x1f */

   FALSE, KEY_D           ,  /* SCAN_D           0x20 */
   FALSE, KEY_F           ,  /* SCAN_F           0x21 */
   FALSE, KEY_G           ,  /* SCAN_G           0x22 */
   FALSE, KEY_H           ,  /* SCAN_H           0x23 */
   FALSE, KEY_J           ,  /* SCAN_J           0x24 */
   FALSE, KEY_K           ,  /* SCAN_K           0x25 */
   FALSE, KEY_L           ,  /* SCAN_L           0x26 */
   FALSE, KEY_COLON       ,  /* SCAN_SCOLON      0x27 */
   FALSE, KEY_AT          ,  /* SCAN_APSTPY      0x28 Apostrophy ''''' */
   FALSE, KEY_HASH        ,  /* SCAN_TILDE       0x29 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_LSHIFT      0x2a */
   FALSE, KEY_HASH        ,  /* SCAN_HASH        0x2b */
   FALSE, KEY_Z           ,  /* SCAN_Z           0x2c */
   FALSE, KEY_X           ,  /* SCAN_X           0x2d */
   FALSE, KEY_C           ,  /* SCAN_C           0x2e */
   FALSE, KEY_V           ,  /* SCAN_V           0x2f */

   FALSE, KEY_B           ,  /* SCAN_B           0x30 */
   FALSE, KEY_N           ,  /* SCAN_N           0x31 */
   FALSE, KEY_M           ,  /* SCAN_M           0x32 */
   FALSE, KEY_LTHAN       ,  /* SCAN_COMMA       0x33 */
   FALSE, KEY_GTHAN       ,  /* SCAN_STOP        0x34 */
   FALSE, KEY_QMARK       ,  /* SCAN_FSLASH      0x35 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_RSHIFT      0x36 */
   FALSE, KEY_STAR        ,  /* SCAN_STAR        0x37 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_ALT         0x38 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x39 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_CAPS        0x3a */
   TRUE , KEY_SHIFT_F1    ,  /* SCAN_F1          0x3b */
   TRUE , KEY_SHIFT_F2    ,  /* SCAN_F2          0x3c */
   TRUE , KEY_SHIFT_F3    ,  /* SCAN_F3          0x3d */
   TRUE , KEY_SHIFT_F4    ,  /* SCAN_F4          0x3e */
   TRUE , KEY_SHIFT_F5    ,  /* SCAN_F5          0x3f */

   TRUE , KEY_SHIFT_F6    ,  /* SCAN_F6          0x40 */
   TRUE , KEY_SHIFT_F7    ,  /* SCAN_F7          0x41 */
   TRUE , KEY_SHIFT_F8    ,  /* SCAN_F8          0x42 */
   TRUE , KEY_SHIFT_F9    ,  /* SCAN_F9          0x43 */
   TRUE , KEY_SHIFT_F10   ,  /* SCAN_F10         0x44 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_NUMLCK      0x45 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_SCRLCK      0x46 */
   TRUE , KEY_HOME        ,  /* SCAN_HOME        0x47 */
   TRUE , KEY_UP          ,  /* SCAN_UP          0x48 */
   TRUE , KEY_PGUP        ,  /* SCAN_PGUP        0x49 */
   FALSE, KEY_MINUS       ,  /* SCAN_DASH        0x4a Number pad minus */
   TRUE , KEY_LEFT        ,  /* SCAN_LEFT        0x4b */
   FALSE, KEY_5           ,  /* SCAN_CENTRE      0x4c Number pad centre */
   TRUE , KEY_RIGHT       ,  /* SCAN_RIGHT       0x4d */
   FALSE, KEY_PLUS        ,  /* SCAN_PLUS        0x4e Number pad plus */
   TRUE , KEY_END         ,  /* SCAN_END         0x4f */

   TRUE , KEY_DOWN        ,  /* SCAN_DOWN        0x50 */
   TRUE , KEY_PGDN        ,  /* SCAN_PGDN        0x51 */
   TRUE , KEY_INS         ,  /* SCAN_INS         0x52 */
   TRUE , KEY_DEL         ,  /* SCAN_DEL         0x53 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x54 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x55 */
   FALSE, KEY_BSLASH      ,  /* SCAN_BSLASH      0x56 */
   TRUE , KEY_SHIFT_F11   ,  /* SCAN_F11         0x57 */
   TRUE , KEY_SHIFT_F12   ,  /* SCAN_F12         0x58 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x59 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5f */

   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x60 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x61 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x62 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x63 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x64 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x65 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x66 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x67 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x68 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x69 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6f */

   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x70 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x71 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x72 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x73 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x74 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x75 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x76 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x77 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x78 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x79 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7f */




//
// CTRL Versions
//

   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_BASE        0x00 */
   FALSE, KEY_ESCAPE      ,  /* SCAN_ESC         0x01 */
   FALSE, KEY_1           ,  /* SCAN_1           0x02 */
   FALSE, KEY_2           ,  /* SCAN_2           0x03 */
   FALSE, KEY_3           ,  /* SCAN_3           0x04 */
   FALSE, KEY_4           ,  /* SCAN_4           0x05 */
   FALSE, KEY_5           ,  /* SCAN_5           0x06 */
   FALSE, KEY_6           ,  /* SCAN_6           0x07 */
   FALSE, KEY_7           ,  /* SCAN_7           0x08 */
   FALSE, KEY_8           ,  /* SCAN_8           0x09 */
   FALSE, KEY_9           ,  /* SCAN_9           0x0a */
   FALSE, KEY_0           ,  /* SCAN_0           0x0b */
   FALSE, KEY_MINUS       ,  /* SCAN_MINUS       0x0c */
   FALSE, KEY_EQUALS      ,  /* SCAN_EQUALS      0x0d */
   FALSE, KEY_BSPACE      ,  /* SCAN_BSPACE      0x0e */
   TRUE , KEY_CTRL_TAB    ,  /* SCAN_TAB         0x0f */

   FALSE, KEY_CTRL_Q      ,  /* SCAN_Q           0x10 */
   FALSE, KEY_CTRL_W      ,  /* SCAN_W           0x11 */
   FALSE, KEY_CTRL_E      ,  /* SCAN_E           0x12 */
   FALSE, KEY_CTRL_R      ,  /* SCAN_R           0x13 */
   FALSE, KEY_CTRL_T      ,  /* SCAN_T           0x14 */
   FALSE, KEY_CTRL_Y      ,  /* SCAN_Y           0x15 */
   FALSE, KEY_CTRL_U      ,  /* SCAN_U           0x16 */
   FALSE, KEY_CTRL_I      ,  /* SCAN_I           0x17 */
   FALSE, KEY_CTRL_O      ,  /* SCAN_O           0x18 */
   FALSE, KEY_CTRL_P      ,  /* SCAN_P           0x19 */
   FALSE, KEY_LSQRB       ,  /* SCAN_LSQRB       0x1a */
   FALSE, KEY_RSQRB       ,  /* SCAN_RSQRB       0x1b */
   FALSE, KEY_ENTER       ,  /* SCAN_ENTER       0x1c */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_CTRL        0x1d */
   FALSE, KEY_CTRL_A      ,  /* SCAN_A           0x1e */
   FALSE, KEY_CTRL_S      ,  /* SCAN_S           0x1f */

   FALSE, KEY_CTRL_D      ,  /* SCAN_D           0x20 */
   FALSE, KEY_CTRL_F      ,  /* SCAN_F           0x21 */
   FALSE, KEY_CTRL_G      ,  /* SCAN_G           0x22 */
   FALSE, KEY_CTRL_H      ,  /* SCAN_H           0x23 */
   FALSE, KEY_CTRL_J      ,  /* SCAN_J           0x24 */
   FALSE, KEY_CTRL_K      ,  /* SCAN_K           0x25 */
   FALSE, KEY_CTRL_L      ,  /* SCAN_L           0x26 */
   FALSE, KEY_SCOLON      ,  /* SCAN_SCOLON      0x27 */
   FALSE, KEY_APSTPY      ,  /* SCAN_APSTPY      0x28 Apostrophy ''''' */
   FALSE, KEY_TILDE       ,  /* SCAN_TILDE       0x29 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_LSHIFT      0x2a */
   FALSE, KEY_HASH        ,  /* SCAN_HASH        0x2b */
   FALSE, KEY_CTRL_Z      ,  /* SCAN_Z           0x2c */
   FALSE, KEY_CTRL_X      ,  /* SCAN_X           0x2d */
   FALSE, KEY_CTRL_C      ,  /* SCAN_C           0x2e */
   FALSE, KEY_CTRL_V      ,  /* SCAN_V           0x2f */

   FALSE, KEY_CTRL_B      ,  /* SCAN_B           0x30 */
   FALSE, KEY_CTRL_N      ,  /* SCAN_N           0x31 */
   FALSE, KEY_CTRL_M      ,  /* SCAN_M           0x32 */
   FALSE, KEY_COMMA       ,  /* SCAN_COMMA       0x33 */
   FALSE, KEY_STOP        ,  /* SCAN_STOP        0x34 */
   FALSE, KEY_FSLASH      ,  /* SCAN_FSLASH      0x35 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_RSHIFT      0x36 */
   FALSE, KEY_STAR        ,  /* SCAN_STAR        0x37 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_ALT         0x38 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x39 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_CAPS        0x3a */
   TRUE , KEY_CTRL_F1     ,  /* SCAN_F1          0x3b */
   TRUE , KEY_CTRL_F2     ,  /* SCAN_F2          0x3c */
   TRUE , KEY_CTRL_F3     ,  /* SCAN_F3          0x3d */
   TRUE , KEY_CTRL_F4     ,  /* SCAN_F4          0x3e */
   TRUE , KEY_CTRL_F5     ,  /* SCAN_F5          0x3f */

   TRUE , KEY_CTRL_F6     ,  /* SCAN_F6          0x40 */
   TRUE , KEY_CTRL_F7     ,  /* SCAN_F7          0x41 */
   TRUE , KEY_CTRL_F8     ,  /* SCAN_F8          0x42 */
   TRUE , KEY_CTRL_F9     ,  /* SCAN_F9          0x43 */
   TRUE , KEY_CTRL_F10    ,  /* SCAN_F10         0x44 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_NUMLCK      0x45 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_SCRLCK      0x46 */
   TRUE , KEY_CTRL_HOME   ,  /* SCAN_HOME        0x47 */
   TRUE , KEY_CTRL_UP     ,  /* SCAN_UP          0x48 */
   TRUE , KEY_CTRL_PGUP   ,  /* SCAN_PGUP        0x49 */
   FALSE, KEY_MINUS       ,  /* SCAN_DASH        0x4a Number pad minus */
   TRUE , KEY_CTRL_LEFT   ,  /* SCAN_LEFT        0x4b */
   FALSE, KEY_5           ,  /* SCAN_CENTRE      0x4c Number pad centre */
   TRUE , KEY_CTRL_RIGHT  ,  /* SCAN_RIGHT       0x4d */
   FALSE, KEY_PLUS        ,  /* SCAN_PLUS        0x4e Number pad plus */
   TRUE , KEY_CTRL_END    ,  /* SCAN_END         0x4f */

   TRUE , KEY_CTRL_DOWN   ,  /* SCAN_DOWN        0x50 */
   TRUE , KEY_CTRL_PGDN   ,  /* SCAN_PGDN        0x51 */
   TRUE , KEY_CTRL_INS    ,  /* SCAN_INS         0x52 */
   TRUE , KEY_CTRL_DEL    ,  /* SCAN_DEL         0x53 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x54 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x55 */
   FALSE, KEY_BSLASH      ,  /* SCAN_BSLASH      0x56 */
   TRUE , KEY_CTRL_F11    ,  /* SCAN_F11         0x57 */
   TRUE , KEY_CTRL_F12    ,  /* SCAN_F12         0x58 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x59 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5f */

   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x60 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x61 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x62 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x63 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x64 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x65 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x66 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x67 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x68 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x69 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6f */

   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x70 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x71 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x72 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x73 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x74 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x75 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x76 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x77 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x78 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x79 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7f */




//
// ALT Versions
//

   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_BASE        0x00 */
   TRUE , KEY_ALT_ESC     ,  /* SCAN_ESC         0x01 */
   TRUE , KEY_ALT_1       ,  /* SCAN_1           0x02 */
   TRUE , KEY_ALT_2       ,  /* SCAN_2           0x03 */
   TRUE , KEY_ALT_3       ,  /* SCAN_3           0x04 */
   TRUE , KEY_ALT_4       ,  /* SCAN_4           0x05 */
   TRUE , KEY_ALT_5       ,  /* SCAN_5           0x06 */
   TRUE , KEY_ALT_6       ,  /* SCAN_6           0x07 */
   TRUE , KEY_ALT_7       ,  /* SCAN_7           0x08 */
   TRUE , KEY_ALT_8       ,  /* SCAN_8           0x09 */
   TRUE , KEY_ALT_9       ,  /* SCAN_9           0x0a */
   TRUE , KEY_ALT_0       ,  /* SCAN_0           0x0b */
   FALSE, KEY_MINUS       ,  /* SCAN_MINUS       0x0c */
   FALSE, KEY_EQUALS      ,  /* SCAN_EQUALS      0x0d */
   FALSE, KEY_BSPACE      ,  /* SCAN_BSPACE      0x0e */
   TRUE , KEY_ALT_TAB     ,  /* SCAN_TAB         0x0f */

   TRUE , KEY_ALT_Q       ,  /* SCAN_Q           0x10 */
   TRUE , KEY_ALT_W       ,  /* SCAN_W           0x11 */
   TRUE , KEY_ALT_E       ,  /* SCAN_E           0x12 */
   TRUE , KEY_ALT_R       ,  /* SCAN_R           0x13 */
   TRUE , KEY_ALT_T       ,  /* SCAN_T           0x14 */
   TRUE , KEY_ALT_Y       ,  /* SCAN_Y           0x15 */
   TRUE , KEY_ALT_U       ,  /* SCAN_U           0x16 */
   TRUE , KEY_ALT_I       ,  /* SCAN_I           0x17 */
   TRUE , KEY_ALT_O       ,  /* SCAN_O           0x18 */
   TRUE , KEY_ALT_P       ,  /* SCAN_P           0x19 */
   FALSE, KEY_LSQRB       ,  /* SCAN_LSQRB       0x1a */
   FALSE, KEY_RSQRB       ,  /* SCAN_RSQRB       0x1b */
   FALSE, KEY_ENTER       ,  /* SCAN_ENTER       0x1c */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_CTRL        0x1d */
   TRUE , KEY_ALT_A       ,  /* SCAN_A           0x1e */
   TRUE , KEY_ALT_S       ,  /* SCAN_S           0x1f */

   TRUE , KEY_ALT_D       ,  /* SCAN_D           0x20 */
   TRUE , KEY_ALT_F       ,  /* SCAN_F           0x21 */
   TRUE , KEY_ALT_G       ,  /* SCAN_G           0x22 */
   TRUE , KEY_ALT_H       ,  /* SCAN_H           0x23 */
   TRUE , KEY_ALT_J       ,  /* SCAN_J           0x24 */
   TRUE , KEY_ALT_K       ,  /* SCAN_K           0x25 */
   TRUE , KEY_ALT_L       ,  /* SCAN_L           0x26 */
   FALSE, KEY_SCOLON      ,  /* SCAN_SCOLON      0x27 */
   FALSE, KEY_APSTPY      ,  /* SCAN_APSTPY      0x28 Apostrophy ''''' */
   FALSE, KEY_TILDE       ,  /* SCAN_TILDE       0x29 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_LSHIFT      0x2a */
   FALSE, KEY_HASH        ,  /* SCAN_HASH        0x2b */
   TRUE , KEY_ALT_Z       ,  /* SCAN_Z           0x2c */
   TRUE , KEY_ALT_X       ,  /* SCAN_X           0x2d */
   TRUE , KEY_ALT_C       ,  /* SCAN_C           0x2e */
   TRUE , KEY_ALT_V       ,  /* SCAN_V           0x2f */

   TRUE , KEY_ALT_B       ,  /* SCAN_B           0x30 */
   TRUE , KEY_ALT_N       ,  /* SCAN_N           0x31 */
   TRUE , KEY_ALT_M       ,  /* SCAN_M           0x32 */
   FALSE, KEY_COMMA       ,  /* SCAN_COMMA       0x33 */
   FALSE, KEY_STOP        ,  /* SCAN_STOP        0x34 */
   FALSE, KEY_FSLASH      ,  /* SCAN_FSLASH      0x35 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_RSHIFT      0x36 */
   FALSE, KEY_STAR        ,  /* SCAN_STAR        0x37 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_ALT         0x38 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x39 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_CAPS        0x3a */
   TRUE , KEY_ALT_F1      ,  /* SCAN_F1          0x3b */
   TRUE , KEY_ALT_F2      ,  /* SCAN_F2          0x3c */
   TRUE , KEY_ALT_F3      ,  /* SCAN_F3          0x3d */
   TRUE , KEY_ALT_F4      ,  /* SCAN_F4          0x3e */
   TRUE , KEY_ALT_F5      ,  /* SCAN_F5          0x3f */

   TRUE , KEY_ALT_F6      ,  /* SCAN_F6          0x40 */
   TRUE , KEY_ALT_F7      ,  /* SCAN_F7          0x41 */
   TRUE , KEY_ALT_F8      ,  /* SCAN_F8          0x42 */
   TRUE , KEY_ALT_F9      ,  /* SCAN_F9          0x43 */
   TRUE , KEY_ALT_F10     ,  /* SCAN_F10         0x44 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_NUMLCK      0x45 */
   FALSE, KEY_xxxxxxxxxxx ,  /* SCAN_SCRLCK      0x46 */
   TRUE , KEY_ALT_HOME    ,  /* SCAN_HOME        0x47 */
   TRUE , KEY_ALT_UP      ,  /* SCAN_UP          0x48 */
   TRUE , KEY_ALT_PGUP    ,  /* SCAN_PGUP        0x49 */
   FALSE, KEY_MINUS       ,  /* SCAN_DASH        0x4a Number pad minus */
   TRUE , KEY_ALT_LEFT    ,  /* SCAN_LEFT        0x4b */
   FALSE, KEY_5           ,  /* SCAN_CENTRE      0x4c Number pad centre */
   TRUE , KEY_ALT_RIGHT   ,  /* SCAN_RIGHT       0x4d */
   FALSE, KEY_PLUS        ,  /* SCAN_PLUS        0x4e Number pad plus */
   TRUE , KEY_ALT_END     ,  /* SCAN_END         0x4f */

   TRUE , KEY_ALT_DOWN    ,  /* SCAN_DOWN        0x50 */
   TRUE , KEY_ALT_PGDN    ,  /* SCAN_PGDN        0x51 */
   TRUE , KEY_ALT_INS     ,  /* SCAN_INS         0x52 */
   TRUE , KEY_ALT_DEL     ,  /* SCAN_DEL         0x53 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x54 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x55 */
   FALSE, KEY_BSLASH      ,  /* SCAN_BSLASH      0x56 */
   TRUE , KEY_ALT_F11     ,  /* SCAN_F11         0x57 */
   TRUE , KEY_ALT_F12     ,  /* SCAN_F12         0x58 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x59 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x5f */

   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x60 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x61 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x62 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x63 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x64 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x65 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x66 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x67 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x68 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x69 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x6f */

   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x70 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x71 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x72 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x73 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x74 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x75 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x76 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x77 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x78 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x79 */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7a */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7b */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7c */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7d */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7e */
   FALSE, KEY_xxxxxxxxxxx ,  /*                  0x7f */

   -1,-1,
};

#endif


/* END OF SCANDEF.H */
