/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : This file contains all of the Debug Monitor command funcs  */
/*                                                                           */
/*   File Name  : MON_CMDS.C                                                 */
/*                                                                           */
/*   Author     : Keith Wilkins                                              */
/*                                                                           */
/*   Version    : 1.00                                                       */
/*                                                                           */
/*   Desciption : The Vectrex debug monitor commands are all function driven */
/*                and this file conatins all of the command functions that   */
/*                are called by fDebugMonitor() in monitor.c                 */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    20/03/93  K.W  Creation                                        */
/*    0.02    15/04/95  K.W  File name change and addition of new commands   */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*                                                                           */
/*****************************************************************************/

#define MON_CMDS_C

#include "vectrex.h"
#include "mon_cmds.h"


#ifdef DEBUG_MON

extern tsCOMMAND    asCommandList[];
extern GWINDOWPTR   pwMonitorWin;
extern DEBUGWININFO asMonWin[];

void fSequencerReset(UWORD tKeyPress);        // Trigger sequencer reset
void fSequencerReboot(UWORD tKeyPress);       // System reset


/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdBreakSet()                                          */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Set CPU breakpoint                                        */
/*                                                                           */
/*   Accepts     : iArgC, apcArgV                                            */
/*                                                                           */
/*   Returns     : Nowt much,                                                */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdBreakSet(SWORD iArgC,UBYTE *apcArgV[])
{
   UWORD iBPAddr;
   UWORD iBPSkip=0;

   if(iArgC<2 || iArgC>3)
   {
      fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid argument.\nUsage: BSET <xxxx> <skip>");
      return(DBG_ERROR);
   }

   if(fStringToWord(apcArgV[1],&iBPAddr)==FALSE)
   {
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid breakpoint address.");
       return(DBG_ERROR);
   }
   if(iArgC==3 && fStringToWord(apcArgV[2],&iBPSkip)==FALSE)
   {
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid breakpoint address.");
       return(DBG_ERROR);
   }

   f6809SetBreakpoint(iBPAddr,iBPSkip);
	return(DBG_OK);
}



/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdBreakClear()                                        */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Clear CPU breakpoint                                      */
/*                                                                           */
/*   Accepts     : iArgC, apcArgV                                            */
/*                                                                           */
/*   Returns     : Nowt much,                                                */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdBreakClear(SWORD iArgC,UBYTE *apcArgV[])
{
   apcArgV[0]=NULL;
   if(iArgC!=1)
   {
      fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nNo arguments required");
      return(DBG_ERROR);
   }
   f6809ClearBreakpoint();
   return(DBG_OK);
}




/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdBreakShow()                                        */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Clear CPU breakpoint                                      */
/*                                                                           */
/*   Accepts     : iArgC, apcArgV                                            */
/*                                                                           */
/*   Returns     : Nowt much,                                                */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdBreakShow(SWORD iArgC,UBYTE *apcArgV[])
{
   UWORD iBPAddr;
   UWORD iSkip;
   UWORD iEnable;

   apcArgV[0]=NULL;
   if(iArgC!=1)
   {
      fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nNo arguments required");
      return(DBG_ERROR);
   }
   f6809ReadBreakpoint(&iBPAddr,&iSkip,&iEnable);
   fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\n Addr-0x%04x  Skip-0x%02x (%s)",iBPAddr,iSkip,iEnable?"enabled":"diasabled");
   return(DBG_OK);
}



/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdDebug()                                            */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Enable/Disable debug window                               */
/*                                                                           */
/*   Accepts     : int argc char ** argv                                     */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

#ifdef DEBUG_MSG

UBYTE fMonCmdDebug(SWORD iArgC,UBYTE *apcArgV[])
{
   if(iArgC==1)
   {
      fDebugToggle(0);
   }
   else if(iArgC==2)
   {
       if(strcmp(apcArgV[1],"LOW")==0)
       {
           fDebugLevelLow(0);
       }
       else if(strcmp(apcArgV[1],"MED")==0)
       {
           fDebugLevelMid(0);
       }
       else if(strcmp(apcArgV[1],"HIGH")==0)
       {
           fDebugLevelHigh(0);
       }
       else if(strcmp(apcArgV[1],"FILE")==0)
       {
           fDebugFileToggle(0);
       }
       else
       {
           fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid argument: LOW,MED,HIGH,FILE");
           return(DBG_ERROR);
       }
   }
   else
   {
      fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nNo arguments required");
      return(DBG_ERROR);
   }
   return(DBG_OK);
}


#endif

/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdDissasemble()                                       */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Dissasemble 6809 code to the screen.                      */
/*                                                                           */
/*   Accepts     : int argc char ** argv                                     */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdDissemble(SWORD iArgC,UBYTE *apcArgV[])
{
    static UWORD wAddress=0;
    UWORD        wStart,wEnd,wInchar,wByteCount,wLoop;
    SWORD        iWindowSize,iRowPos;
    UBYTE        acDisString[DISAS_MAX_STRING];

    /* Set defaults for now values */

    wStart=wAddress;
    wEnd=wStart+DISAS_DEFAULT_LENGTH;
    iWindowSize=(pwMonitorWin->iYSizeInternal/(BASE_FONT_HEIGHT*asMonWin[wGblDisplayMode].iWinFontsize))-2;

    /* Proces the input params */

    switch(iArgC)
    {
        case 3:
            if(fStringToWord(apcArgV[1],&wStart)==FALSE)
            {
                fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid Start Address.");
                return(DBG_ERROR);
            }

            if(fStringToWord(apcArgV[2],&wEnd)==FALSE)
            {
                fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid End Address.");
                return(DBG_ERROR);
            }
            break;

        case 2:
            if(fStringToWord(apcArgV[1],&wStart)==FALSE)
            {
                fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid Start Address.");
                return(DBG_ERROR);
            }
            wEnd=wStart+DISAS_DEFAULT_LENGTH;
            break;
                
        case 1:
            break;

        default:
            fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nToo many arguments.\nUsage: D <xxxx>,<yyyy>");
            return(DBG_ERROR);
    }


    /* Now do the Dump */

    iRowPos=0;
    do
    {
   		/* Do the disassemble and print out addr + bytes + string */
    
   		wAddress=f6809Disassemble(wStart,acDisString);
   		wByteCount=wAddress-wStart;
   		fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\n%04x  ",wStart);

		for(wLoop=0;wLoop<wByteCount;wLoop++)
		{
			fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"%02x ",f6809AddressRead(wStart+wLoop));
		}
		for(wLoop=wByteCount;wLoop<DISAS_BYTE_MAX;wLoop++)
		{
			fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"   ");
		}
   		                                           
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"  %s",acDisString);
 		wStart=wAddress;
    		
        if(++iRowPos>=iWindowSize)
        {
            fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\n-more-");
            wInchar=toupper(fKBDgetch());
            fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\b\b\b\b\b\b      \b\b\b\b\b\b");
                        
            if(wInchar==KEY_Q)
            {
                break;
            }       
            else
            {
                fWinClear(pwMonitorWin);
                iRowPos=0;
            }
        }
    }
    while(wAddress<=wEnd);
        
	return(DBG_OK);
}




/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdDump()                                             */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Dump memory in hex format to the screen.                  */
/*                                                                           */
/*   Accepts     : argc argv                                                 */
/*                                                                           */
/*   Returns     : FALSE as we dont want debug to exit                       */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdDump(SWORD iArgC,UBYTE *apcArgV[])
{
    static UWORD wAddress=0;
    UWORD        wStart,wEnd,tSave,wInchar;
    SWORD        iRowPos,iWindowSize,iCount=0,iLoop;

    /* Set defaults for now values */

    wStart=wAddress;
    wEnd=wStart+DUMP_DEFAULT_LENGTH;
    iWindowSize=(pwMonitorWin->iYSizeInternal/(BASE_FONT_HEIGHT*asMonWin[wGblDisplayMode].iWinFontsize))-2;

    /* Proces the input params */

    switch(iArgC)
    {
        case 3:
            if(fStringToWord(apcArgV[1],&wStart)==FALSE)
            {
                fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid Start Address.");
                return(DBG_ERROR);
            }

            if(fStringToWord(apcArgV[2],&wEnd)==FALSE)
            {
                fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid End Address.");
                return(DBG_ERROR);
            }
            break;

        case 2:
            if(fStringToWord(apcArgV[1],&wStart)==FALSE)
            {
                fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid Start Address.");
                return(DBG_ERROR);
            }
            wEnd=wStart+DUMP_DEFAULT_LENGTH;
            break;
                
        case 1:
            break;

        default:
            fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nToo many arguments.\nUsage: D <xxxx>,<yyyy>");
            return(DBG_ERROR);
    }


    /* Now do the Dump */

    wAddress=wStart;
    iCount=0;
    iRowPos=0;
    do
    {
        /* Print Address header when required */

        if(iCount==0)
        {
            tSave=wAddress;
            fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\n%04x ",wAddress);
        }

        /* Print a byte */

        fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"%02x ",f6809AddressRead(wAddress));

        /* Increment the count of line width */

        if(++iCount>=DUMP_LINE_WIDTH)
        {

            /* We are at the end of the line so print out the characters reperenting the line */
            /* from the SAVE position                                                         */

            fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize," ");
            for(iLoop=0;iLoop<DUMP_LINE_WIDTH;iLoop++) fWinOutChar(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,(UBYTE)f6809AddressRead(tSave++));
            iCount=0;

            /* Check if were overflowing the window and stop if so */

            if(++iRowPos>=iWindowSize)
            {
                fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\n-more-");
                wInchar=toupper(fKBDgetch());
                fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\b\b\b\b\b\b      \b\b\b\b\b\b");

                if(wInchar==KEY_Q || wInchar==KEY_N)
                {
                    break;
                }       
                else
                {
                    fWinClear(pwMonitorWin);
                    iRowPos=0;
                }
            }
        }
    }
    while(++wAddress!=wEnd+1);
        
	return(DBG_OK);
}




/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdDumpReg()                                           */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Dump CPU registers of 6809 processor to screen            */
/*                                                                           */
/*   Accepts     : argc argv                                                 */
/*                                                                           */
/*   Returns     : FALSE as we dont want debug to exit                       */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdRegDump(SWORD iArgC,UBYTE *apcArgV[])
{
   CPUREGISTERS   tsRegs;

   iArgC=0;
   apcArgV[0]=NULL;

   f6809RegistersRead(&tsRegs);

   fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nx   0x%04x    y   0x%04x    d   0x%04x (a=0x%02x  b=0x%02x)",tsRegs.tX,tsRegs.tY,tsRegs.uABD.tD,tsRegs.uABD.sAB.tA,tsRegs.uABD.sAB.tB);
   fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nusp 0x%04x    hsp 0x%04x    dp  0x%02x",tsRegs.tUSP,tsRegs.tHSP,tsRegs.tDP);
   fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\npc  0x%04x    cc  0x%02x ",tsRegs.tPC,tsRegs.uCC.tCC);
   fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"(c=%d v=%d z=%d n=%d)\n",tsRegs.uCC.sCC.bC,tsRegs.uCC.sCC.bV,tsRegs.uCC.sCC.bZ,tsRegs.uCC.sCC.bN);
//   fWinPrintf(pwMonitorWin,asMonWin[wGblDisiWinFontsize,"(C:%d V:%d Z:%d N:%d I:%d H:%d F:%d E:%d)\n",tsRegs.uCC.sCC.bC,tsRegs.uCC.sCC.bV,tsRegs.uCC.sCC.bZ,tsRegs.uCC.sCC.bN,tsRegs.uCC.sCC.bI,tsRegs.uCC.sCC.bH,tsRegs.uCC.sCC.bF,tsRegs.uCC.sCC.bE);

   return(DBG_OK);
}


/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdExit()                                              */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Exit from the debug monitor back to the sequencer         */
/*                                                                           */
/*   Accepts     : argc argv                                                 */
/*                                                                           */
/*   Returns     : DBG_QUIT always                                           */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdExit(SWORD iArgC,UBYTE *apcArgV[])
{
   iArgC=0;
   apcArgV[0]=NULL;
	return(DBG_QUIT);
}



/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdFind()                                              */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Find a hex string within CPU memory space                 */
/*                                                                           */
/*   Accepts     : argc argv                                                 */
/*                                                                           */
/*                 find <start> <end> xx xx xx xx                            */
/*                 find <start> <end> xxxxxxxx OR xxxx xxxx OR xxxxxx xx xx  */
/*                 find <start> <end> "string"                               */
/*                                                                           */
/*   Returns     : DBG_OK if executed OK else DBG_ERROR                      */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdFind(SWORD iArgC,UBYTE *apcArgV[])
{
	UBYTE   abFind[FIND_MAX_LENGTH];
	UWORD   wStart,wEnd,wAddress,wInchar,wFindCount,wHitCount;
	UBYTE   acTmpStr[3],acFakeArgs[3][6];
	UBYTE   *pcTextPtr,*apcFakeArgList[3];
	SWORD   iArgNo,iNibbleNo;

	/* Check argument count against max */
	
	if(iArgC<4 || iArgC > (FIND_MAX_LENGTH+3))
	{
        fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nIncorrect number of arguments.");
        return(DBG_ERROR);
   }
   
   /* Retrieve the start and end address */
                        
   iArgNo=1;
   
   if(fStringToWord(apcArgV[iArgNo++],&wStart)==FALSE)
   {
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid Start Address.");
       return(DBG_ERROR);
   }

   if(fStringToWord(apcArgV[iArgNo++],&wEnd)==FALSE)
   {
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid End Address.");
       return(DBG_ERROR);
   }
   
   /* Generate the find array from the arguments, test for string or hex */

   wFindCount=0;

	if(apcArgV[iArgNo][0]=='"')
	{
		/* Text string */
		
		if(strlen(apcArgV[iArgNo])>FIND_MAX_LENGTH+2)
		{
	        fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nString too long.");
	        return(DBG_ERROR);
	    }
	    
	    /* Copy the string across minus the quotes */
	        
	    pcTextPtr=&apcArgV[iArgNo][1];
	    while(*pcTextPtr!='"' && *pcTextPtr!='\0')
	    {
	    	abFind[wFindCount++]=(UBYTE)*(pcTextPtr++);
	    	
	    	if(wFindCount>FIND_MAX_LENGTH)
	    	{
		        fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nString too long.");
		        return(DBG_ERROR);
	    	}
	    }
	}
	else
	{
		/* Hex string, convert in byte pairs */
	        
	    do
    	{
    		pcTextPtr=apcArgV[iArgNo++];
    	
    		/* Process string */
    	
	    	iNibbleNo=0;
    		while(*pcTextPtr!='\0')
    		{
	    		if(iNibbleNo==0)
    			{
    				acTmpStr[0]=*pcTextPtr++;
    				iNibbleNo++;
	    		}           
    			else
	    		{
    				acTmpStr[1]=*pcTextPtr++;
    				acTmpStr[2]='\0';
    			
    				/* We've got the 2nd byte, now convert */
		    	
			    	if(fHexStringToByte(acTmpStr,&abFind[wFindCount])==FALSE)
			    	{
				        fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nArgument error, use HEX only.");
        				return(DBG_ERROR);
	        		}
        	
    	    		/* Converted O.K so increment the search count, and reset the nibble count */
        			wFindCount++;
        			iNibbleNo=0;
	    		}
    		}
	    }
	    while(iArgNo!=iArgC);
	}
    
    /* Now lets search */
    
    wHitCount=0;
    
	for(wAddress=wStart;wAddress!=wEnd;wAddress++)
	{
	
		/* Check for byte match */
	
		if(f6809AddressRead(wAddress)==abFind[wHitCount])
		{
			wHitCount++;
		}
		else
		{
			wHitCount=0;
		}
		
		/* Check for complete match, and if so dump 3 lines and pause */
		
		if(wHitCount==wFindCount)
		{
			wHitCount=0;
			strcpy(acFakeArgs[0],"DUMP");
			sprintf(acFakeArgs[1],"$%04x",wAddress-wFindCount-15);
			sprintf(acFakeArgs[2],"$%04x",wAddress-wFindCount+32);
			for(iNibbleNo=0;iNibbleNo<3;iNibbleNo++) apcFakeArgList[iNibbleNo]=acFakeArgs[iNibbleNo];
			
			fMonCmdDump(3,(UBYTE**)apcFakeArgList);

			/* Ask if user wants to search further */

            fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nContinue (y/n)");
            wInchar=toupper(fKBDgetch());
            fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\b\b\b\b\b\b\b\b\b\b\b\b\b\b              \b\b\b\b\b\b\b\b\b\b\b\b\b\b");
            if(wInchar==KEY_N || wInchar==KEY_Q) return(DBG_OK);
		}
	}
	
	/* Sucessful exit */
		
	return(DBG_OK);
}



/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdHelp()                                              */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Printout the debug command list help to the monitor       */
/*                 window.                                                   */
/*                                                                           */
/*   Accepts     : argc,argv                                                 */
/*                                                                           */
/*   Returns     : FALSE as we dont want debug to exit                       */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdHelp(SWORD iArgC,UBYTE *apcArgV[])
{
    SWORD     iIndex=0,iRowPos=0;
    SWORD     iWindowSize;
    UWORD     wInchar;

    iArgC=0;
    apcArgV[0]=NULL;

    iWindowSize=(pwMonitorWin->iYSizeInternal/(BASE_FONT_HEIGHT*asMonWin[wGblDisplayMode].iWinFontsize))-2;

    while(asCommandList[iIndex].pfCommandFunc!=NULL)
    {
        if(strcmp(asCommandList[iIndex].pcCommandHelp,"")!=0)
        {
            if(++iRowPos>=iWindowSize)
            {
                fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\n-more-");
                wInchar=toupper(fKBDgetch());
                fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\b\b\b\b\b\b      \b\b\b\b\b\b");

                if(wInchar==KEY_Q)
                {
                    break;
                }
                else
                {
                    fWinClear(pwMonitorWin);
                    iRowPos=0;
                }
            }
            fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\n%s",asCommandList[iIndex].pcCommandHelp);
        }
        iIndex++; 
    }
    return(DBG_OK);
}




/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdRun()                                               */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Run the emulator from a given position or from current    */
/*                 PC if no address is given.                                */
/*                                                                           */
/*   Accepts     : argc argv                                                 */
/*                                                                           */
/*   Returns     : FALSE as we exit debug by disabling debug mode            */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdRun(SWORD iArgC,UBYTE *apcArgV[])
{
   UWORD           tRunAddress;
   CPUREGISTERS    ts6809Regs;

   /* Check argc */

   if(iArgC>2)
   {
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nToo many arguments.\nUsage: R <xxxx>");
       return(DBG_ERROR);
   }

   /* Decode the address agument is it exists */

   if(iArgC==2)
   {
       if(fStringToWord(apcArgV[1],&tRunAddress)==FALSE)
       {
           fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid Run Address.");
           return(DBG_ERROR);
       }

       /* Set the new PC in the CPU registers */

       f6809RegistersRead(&ts6809Regs);

       ts6809Regs.tPC=tRunAddress;

       f6809RegistersWrite(&ts6809Regs);

       fDebug(HIGH,("fMonCmdRun() CPU program counter set to %04x",tRunAddress));
   }

   /* Exit command exec with no errors, and close debug */

   f6809StatusWrite(CPU_FETCH);
	return(DBG_QUIT);
}




/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdSequence()                                          */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Execute a certain number machine cycles of a particular   */
/*                 subsystem either T=System Cycles C=CPU A=Analog S=Sound   */
/*                 P=Persistance                                             */
/*                 If no subsysem is specified then default is T.            */
/*                                                                           */
/*                                                                           */
/*   Accepts     : argv,argc                                                 */
/*                                                                           */
/*   Returns     : TRUE if execution with no errors                          */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdSequence(SWORD iArgC,UBYTE *apcArgV[])
{
	ULONG   lwStepCount;
	SWORD   iArgNo;
	UBYTE   cStepType;
	
   if(iArgC>3)
   {
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nToo many arguments.");
       return(DBG_ERROR);
   }
   else if(iArgC<2)
   {
   	fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nNo arguments specified.");
   	return(DBG_ERROR);
   }

   /* Decode the address agument is it exists */

   if(iArgC==3)
   {
   	if(strlen(apcArgV[1])!=1)
   	{
   		fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nBad Step type : T,D,S are valid");
   		return(DBG_ERROR);
   	}
   	cStepType=toupper(apcArgV[1][0]);
   	iArgNo=2;
   }
   else
   {
   	iArgNo=1;
   	cStepType='T';
   }
   
   if(fStringToLongWord(apcArgV[iArgNo],&lwStepCount)==FALSE || lwStepCount==0)
   {
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nError: Step count missing or invalid");
       return(DBG_ERROR);
   }
   
   /* Now convert the StepCount into system Ticks by multiplying by reload values */
   
   switch(cStepType)
   {
   	case 'D':
   		lwStepCount=lwStepCount*lwGblDisplayTickReload;
   		break;
   		
   	case 'T':
   	default:
   		break;
   }                                                          
   
   /* Set when the breakpoint will occur */
               
   lwGblSystemTickBreakpoint=lwGblSystemTickTotal+lwStepCount;
   fDebug(HIGH,("fMonCmdSequence(): Tick=%ld, Breakpoint=%ld",lwGblSystemTickTotal,lwGblSystemTickBreakpoint));

   /* Now return debug to the sequencer */                                                           
    
   f6809StatusWrite(CPU_FETCH);
	return(DBG_SEQUENCE);
}



/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdStep()                                              */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Single step the CPU a certain number of instructions      */
/*                 is one cycle.                                             */
/*                                                                           */
/*   Accepts     : argv,argc                                                 */
/*                                                                           */
/*   Returns     : TRUE if execution with no errors                          */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdStep(SWORD iArgC,UBYTE *apcArgV[])
{
   UWORD           tRunAddress;
   CPUREGISTERS    ts6809Regs;

   if(iArgC>2)
   {
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nToo many arguments.\nUsage: R <xxxx>");
       return(DBG_ERROR);
   }

   /* Decode the address agument is it exists */

   if(iArgC==2)
   {
       if(fStringToWord(apcArgV[1],&tRunAddress)==FALSE)
       {
           fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid Run Address.");
           return(DBG_ERROR);
       }

       /* Set the new PC in the CPU registers */

       f6809RegistersRead(&ts6809Regs);

       ts6809Regs.tPC=tRunAddress;

       f6809RegistersWrite(&ts6809Regs);

       fDebug(HIGH,("fMonCmdStep() CPU program counter set to %04x",tRunAddress));
   }

   f6809StatusWrite(CPU_FETCH);
  	return(DBG_SEQUENCE);
}




/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdStepRun()                                           */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Single step the CPU a continuously                        */
/*                                                                           */
/*   Accepts     : argv,argc                                                 */
/*                                                                           */
/*   Returns     : TRUE if execution with no errors                          */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdStepRun(SWORD iArgC,UBYTE *apcArgV[])
{
   UWORD           tRunAddress;
   CPUREGISTERS    ts6809Regs;

   if(iArgC>2)
   {
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nToo many arguments.\nUsage: R <xxxx>");
       return(DBG_ERROR);
   }

   /* Decode the address agument is it exists */

   if(iArgC==2)
   {
       if(fStringToWord(apcArgV[1],&tRunAddress)==FALSE)
       {
           fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nInvalid Run Address.");
           return(DBG_ERROR);
       }

       /* Set the new PC in the CPU registers */

       f6809RegistersRead(&ts6809Regs);

       ts6809Regs.tPC=tRunAddress;

       f6809RegistersWrite(&ts6809Regs);

       fDebug(HIGH,("fMonCmdStep() CPU program counter set to %04x",tRunAddress));
   }

   f6809StatusWrite(CPU_FETCH);
  	return(DBG_STEPRUN);
}


/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdReset()                                             */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Reset some/all of the vectrex functions                   */
/*                                                                           */
/*   Accepts     : argv,argc                                                 */
/*                                                                           */
/*   Returns     : TRUE if execution with no errors                          */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdReset(SWORD iArgC,UBYTE *apcArgV[])
{
   if(iArgC>2)
   {
       fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nToo many arguments.");
       return(DBG_ERROR);
   }

   if(strcmp(apcArgV[1],"ALL")==0 || iArgC==1)
   {
      fSequencerReboot(0);
   }
   else if(strcmp(apcArgV[1],"CPU")==0)
   {
      fSequencerReset(0);
   }
   else
   {
      fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nBad reset argument.");
   }

  	return(DBG_OK);
}




/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdTicks()                                             */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Display system tick status counters                       */
/*                                                                           */
/*   Accepts     : argv,argc                                                 */
/*                                                                           */
/*   Returns     : TRUE if execution with no errors                          */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdTicks(SWORD iArgC,UBYTE *apcArgV[])
{
   iArgC=0;
   apcArgV[0]=NULL;

   fWinPrintf(pwMonitorWin,asMonWin[wGblDisplayMode].iWinFontsize,"\nTicks:  system=%08ld   display=%08ld",
                                              lwGblSystemTickTotal,
                                              lwGblSystemTickTotal/lwGblDisplayTickReload);
	return(DBG_OK);
}



/*****************************************************************************/
/*                                                                           */
/*   Function    : fMonCmdQuit()                                              */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Exit from the Vectrex emulator back to DOS                */
/*                                                                           */
/*   Accepts     : argc argv                                                 */
/*                                                                           */
/*   Returns     : TRUE/FALSE if command exec OK                             */
/*                                                                           */
/*****************************************************************************/

UBYTE fMonCmdQuit(SWORD iArgC,UBYTE *apcArgV[])
{
   iArgC=0;
   apcArgV[0]=NULL;

   bGblQuitFlag=TRUE;
	return(DBG_QUIT);
}


#endif

/* End of MON_CMDS.C */
