/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : This file contains function prototypes for debug commands  */
/*                                                                           */
/*   File Name  : MON_CMDS.H                                                 */
/*                                                                           */
/*   Author     : Keith Wilkins                                              */
/*                                                                           */
/*   Version    : 1.00                                                       */
/*                                                                           */
/*   Desciption : The Vectrex debug monitor command function prototypes      */
/*                and necessary definitions                                  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    20/03/93  K.W  Creation                                        */
/*    0.02    15/03/93  K.W  File name change from commands.h and new        */
/*                           commands added to the prototype list            */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*                                                                           */
/*****************************************************************************/

#if !defined MON_CMDS_H
    #define MON_CMDS_H

/* Defined for various commands */

#define DUMP_LINE_WIDTH			16		/* Number of dump bytes on a line */
#define DUMP_DEFAULT_LENGTH		(64-1)	/* Default number of bytes to dump */
#define DISAS_DEFAULT_LENGTH	24		/* Number of bytes to disassemble as a default */
#define DISAS_MAX_STRING		30		/* Length of string array passed to disassembler */
#define DISAS_BYTE_MAX			4		/* Max number of bytes printed in disassembly output */
#define FIND_MAX_LENGTH         12      /* Maximum search string length in bytes */

/* Define debug return Values */

#define  DBG_ERROR      0
#define  DBG_QUIT       1
#define  DBG_OK         2
#define  DBG_SEQUENCE   3
#define  DBG_STEPRUN    4

/* Function prototypes */

UBYTE  fMonCmdBreakSet(SWORD,UBYTE**);
UBYTE  fMonCmdBreakClear(SWORD,UBYTE**);
UBYTE  fMonCmdBreakShow(SWORD,UBYTE**);
#ifdef DEBUG_MSG
UBYTE  fMonCmdDebug(SWORD,UBYTE**);
#endif
UBYTE  fMonCmdDump(SWORD,UBYTE**);
UBYTE  fMonCmdRegDump(SWORD,UBYTE**);
UBYTE	fMonCmdDissemble(SWORD,UBYTE**);
UBYTE  fMonCmdReset(SWORD,UBYTE**);
UBYTE  fMonCmdExit(SWORD,UBYTE**);
UBYTE  fMonCmdFind(SWORD,UBYTE**);
UBYTE  fMonCmdHelp(SWORD,UBYTE**);
UBYTE  fMonCmdRun(SWORD,UBYTE**);
UBYTE  fMonCmdStepRun(SWORD,UBYTE**);
UBYTE  fMonCmdSequence(SWORD,UBYTE**);
UBYTE  fMonCmdStep(SWORD,UBYTE**);
UBYTE  fMonCmdTicks(SWORD,UBYTE**);
UBYTE  fMonCmdQuit(SWORD,UBYTE**);


#endif

/* End of MON_CMDS.H */



