/*****************************************************************************/
/*                                                                           */
/*   Function    : f6522Tick()                                               */
/*                                                                           */
/*   Author      : K.W / C.S.                                                */
/*                                                                           */
/*   Description : Perform a single tick of the 6522 counters                */
/*                 Sometimes inline...                                       */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/
{
#ifdef PIA_TIMER1
 /*********************/
 /* TIMER 1 EMULATION */
 /*********************/
 if ((u6522Timer1.uCount.tW--)==0xffff)
 {
#ifdef DEBUG_MSG
  tcount++;
  fDebug(MID,("f6522Tick() - Timer 1 = %04x (%i)",u6522Timer1.uCount.tW,tcount));
#endif
//  u6522Timer1.uCount.tW=u6522Timer1.uLatch.tW; // moved to switch clause
  t6522IFLG|=E6522IFLG_TIMER1;
  if ((t6522IER&E6522IFLG_TIMER1) && !tsCPURegisters.uCC.sCC.bI)
  {
   /**********************************************************************/
   /* The only rom I know of which actually does a T1 interrupt is SPIKE */
   /**********************************************************************/
   fDebug(HIGH,("<T1<--  f6522Tick() - IRQ asserted by 6522 Timer 1 at PC=<%04x>",tsCPURegisters.tPC));
   fDebug(HIGH,("u6522Timer1.uLatch.tW:%04x",u6522Timer1.uLatch.tW));
   fDebug(HIGH,("IRQ lwGblSystemTickTotal:%i -->T1>",(ULONG)lwGblSystemTickTotal));
   tsCPURegisters.tIRQFlag=TRUE;
   t1int=TRUE; /* YUK! */
  }
  /************************/
  /* Switch on timer mode */
  /************************/
  switch (t6522AUX_T1CTRL)
  {
   case 2: // ONE SHOT MODE ON PB7
    fDebug(LOW,("f6522Tick() - Timer 1 expiry (PB7 was ==%d) PB7=1",(t6522ORB_7)?1:0));
    t6522ORB_7=TMPL_RB_7;
    fAnalogTick();
    break;
   case 3: // FREE RUN MODE
    u6522Timer1.uCount.tW=u6522Timer1.uLatch.tW;
    fDebug(LOW,("f6522Tick() - Timer 1 expiry PB7 Toggle"));
    t6522ORB_7^=TMPL_RB_7;
    fAnalogTick();
    break;
   case 0:
   case 1:
    break;
   default:
    fDebug(VERROR,("f6522Tick() Illegal TIMER1 mode (%d)",t6522AUX_T2CTRL));
    break;
  }
 }
#endif
#ifdef PIA_TIMER2
 /*********************/
 /* TIMER 2 EMULATION */
 /*********************/
 if (!u6522Timer2.uCount.tW--)
 {
  if (bTimer2OneShot)
  {
   fDebug(MED,("f6522Tick() - 6522 Timer 2 at PC=<%04x>",tsCPURegisters.tPC));
   bTimer2OneShot=FALSE;
   t6522IFLG|=E6522IFLG_TIMER2;
   if ((t6522IER&E6522IFLG_TIMER2) && !tsCPURegisters.uCC.sCC.bI )
   {
    fDebug(HIGH,("f6522Tick() - IRQ asserted by 6522 Timer 2 at PC=<%04x>",tsCPURegisters.tPC));
    tsCPURegisters.tIRQFlag=TRUE;
   }
  }
  else
  {
   fDebug(HIGH,("f6522Tick() - Timer 2 expired, no IRQ"));
  }
 }
#endif
#ifdef PIA_SHIFT
 /***************************/
 /* SHIFT REGISTER EMULATON */
 /***************************/
 /******************************/
 /* CB1 emulation not required */
 /******************************/
 if (t6522SHFTREGACT)
 {
  fDebug(MED,("f6522Tick() - Shifting %02x -> Out %02x",t6522SHFTREG,t6522CB2));
  if (iToggle)
  {
   switch(t6522AUX_SHFTCTRL)
   {
    case 6: // Shift out using main clk
     t6522CB2=t6522SHFTREG&0x80;
     fDebug(MED,("f6522Tick() - Shifting %02x -> Out %02x",t6522SHFTREG,t6522CB2));
     t6522SHFTREG=t6522SHFTREG<<1;
     t6522SHFTREGACT--;
     fAnalogTick();
     break;
    default:
     fDebug(VERROR,("f6522Tick() - Shift register mode not supported"));
     break;
   }
   if (!t6522SHFTREGACT)
   {
    t6522IFLG|=E6522IFLG_SHFT;
    if (t6522IER&0x04)
    {
     fDebug(HIGH,("f6522Tick() - IRQ asserted by 6522 Shift Reg at PC=<%04x>",tsCPURegisters.tPC));
     tsCPURegisters.tIRQFlag=TRUE;
    }
   }
   iToggle=0;
  }
  else
  {
   iToggle=1;
  }
 }
#endif
}
