/*****************************************************************************/
/*                                                                           */
/*   Function    : fVectrexEmulate()                                         */
/*                                                                           */
/*   Author      : K.W / C.S.                                                */
/*                                                                           */
/*   Description : This function is the HEART of the emluator. It schedules  */
/*                 and executes all of the functions that make up the        */
/*                 Vectrex emulator. The system is run on an infinite loop   */
/*                 which is only broken if the tQuitFlag==TRUE, this is a    */
/*                 global variable and can be set by any module of the user  */
/*                 wants to quit or the error is serious enough. For each    */
/*                 loop the gGblSystemTick variable is incremented. Also     */
/*                 for each loop the tGbl????SystemTickCounters are decreme- */
/*                 nted and when one reaches zero tGbl????SystemTickTotal    */
/*                 is incremented and the f????Tick function is executed.    */
/*                                                                           */
/*                 The debug flags can disable the execution of any of the   */
/*                 tick functions in an individual basis. When the DEBUG     */
/*                 mode is activated the fDoDebug() is called on every       */
/*                 time tGblSystemTick in incrmented.                        */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : TRUE or FALSE depending on if an error occured            */
/*                                                                           */
/*****************************************************************************/

{
#ifndef DEBUG_MSG
 SWORD tTicker=10;
#endif
#ifdef CLAY_ONLY
 SLONG grabticker=0;
#endif


#ifdef DEBUG_MSG
 int sync=0;
  int cwai=0;
 int ex=0;
 int wait=0;
 int halt=0;
#endif

 f6809Reset();
 fKBDbuffering(FALSE);

 /************************/
 /* Install control keys */
 /************************/

 fKBDfuncreg(KEY_ALT_X,fSequencerAbort);
 fKBDfuncreg(KEY_ALT_R,fSequencerReset);
 fKBDfuncreg(KEY_ALT_B,fSequencerReboot);
 fKBDfuncreg(KEY_ALT_P,fSequencerPause);
 fKBDfuncreg(KEY_ALT_G,fScreenshot);

 fKBDfuncreg(KEY_ALT_L,fLoadState);
 fKBDfuncreg(KEY_1,fLoadState);
 fKBDfuncreg(KEY_2,fLoadState);
 fKBDfuncreg(KEY_3,fLoadState);
 fKBDfuncreg(KEY_4,fLoadState);
 fKBDfuncreg(KEY_5,fLoadState);
 fKBDfuncreg(KEY_6,fLoadState);
 fKBDfuncreg(KEY_7,fLoadState);
 fKBDfuncreg(KEY_8,fLoadState);
 fKBDfuncreg(KEY_9,fLoadState);

 fKBDfuncreg(KEY_ALT_W,fSaveState);
 fKBDfuncreg(KEY_ALT_1,fSaveState);
 fKBDfuncreg(KEY_ALT_2,fSaveState);
 fKBDfuncreg(KEY_ALT_3,fSaveState);
 fKBDfuncreg(KEY_ALT_4,fSaveState);
 fKBDfuncreg(KEY_ALT_5,fSaveState);
 fKBDfuncreg(KEY_ALT_6,fSaveState);
 fKBDfuncreg(KEY_ALT_7,fSaveState);
 fKBDfuncreg(KEY_ALT_8,fSaveState);
 fKBDfuncreg(KEY_ALT_9,fSaveState);

#ifdef CLAY_ONLY
 fKBDfuncreg(KEY_ALT_C,fCScreenshot);
#endif

 fKBDfuncreg(KEY_ALT_F1 ,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F2 ,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F3 ,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F4 ,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F5 ,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F6 ,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F7 ,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F8 ,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F9 ,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F10,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F11,fCarouselSelect);
 fKBDfuncreg(KEY_ALT_F12,fCarouselSelect);

#ifdef DEBUG_MON
 if(bGblDebugMonitorEnable==TRUE)
  fDebugMonitor();
#endif

#ifdef SEQUENCER_LINEAR
 #ifdef SEQUENCER_LIGHTPEN
  fDebug(HIGH,("sequencer() - Linear + Lightpen used"));
 #else
  #ifdef SEQUENCER_IMAGER
   fDebug(HIGH,("sequencer() - Linear - Imager used"));
  #else
   fDebug(HIGH,("sequencer() - Linear used"));
  #endif
 #endif
#else
 #ifdef SEQUENCER_LIGHTPEN
  fDebug(HIGH,("sequencer() - Banked + Lightpen used"));
 #else
  #ifdef SEQUENCER_IMAGER
   fDebug(HIGH,("sequencer() - Banked - Imager used"));
  #else
   fDebug(HIGH,("sequencer() - Banked used"));
  #endif
 #endif
#endif




 /****************************/
 /* Start the sequencer loop */
 /****************************/
 /* perhaps one should define limit as the length of the next instruction? */
 #define limit ((UWORD)(20))
 if (bGblRestoreOnStart==TRUE)
 {
  fLoadState(0);
 }

 while(!bGblQuitFlag)
 {
#ifdef DEBUG_MSG
  if (i6809status==CPU_HALT)
   halt++;
  if (i6809status==CPU_EXCEPTION)
   ex++;
  if (i6809status==CPU_CWAI)
   cwai++;
  if (i6809status==CPU_SYNC)
   sync++;
  if (i6809status==CPU_WAIT)
   wait++;
#endif

#if defined SEQUENCER_LIGHTPEN
  /***********************************************************/
  /* Mouse shit... can't do it in the interrupthandler, why? */
  /***********************************************************/
  if (mchanged==TRUE)
  {
   vesa_mouse_update_bank();
   mchanged=FALSE;
  }
  /**********************************************/
  /* if lightpen is active, do a FULL emulation */
  /**********************************************/
  if ((t6522IER&0x02)&&(mouse_button))
  {
   fAnalogTick();
   #define PIA_TIMER1
   #define PIA_TIMER2
   #define PIA_SHIFT

   /******************************************************************/
   /* one could optimize this to multi ticks as well... but it's not */
   /* worth the trouble                                              */
   /******************************************************************/
   #include "pia.cc"

   #undef PIA_TIMER1
   #undef PIA_TIMER2
   #undef PIA_SHIFT

   #include "singlcpu.cc"
   lwGblSystemTickTotal++;
  }
  else
#endif

#if defined CLAY_ONLY && !defined SEQUENCER_LIGHTPEN
  if (cscreenshots==TRUE)
  {
   fAnalogTick();
   #define PIA_TIMER1
   #define PIA_TIMER2
   #define PIA_SHIFT
   #include "pia.cc"

   #undef PIA_TIMER1
   #undef PIA_TIMER2
   #undef PIA_SHIFT

   #include "singlcpu.cc"
   lwGblSystemTickTotal++;
   grabticker--;
  }
  else
#endif
  if (bGblForceSingleProcessor==TRUE)
  {
   fAnalogTick();
   #define PIA_TIMER1
   #define PIA_TIMER2
   #define PIA_SHIFT
   #include "pia.cc"

   #undef PIA_TIMER1
   #undef PIA_TIMER2
   #undef PIA_SHIFT

   #include "singlcpu.cc"
   lwGblSystemTickTotal++;
  }
  else
  /*****************************/
  /* look at the readme.txt... */
  /*****************************/
  if (((UWORD)((UWORD)(u6522Timer1.uCount.tW)+(UWORD)1)>(UWORD)limit+1)
  && ((u6522Timer2.uCount.tW)>limit+1)
#ifdef SEQUENCER_IMAGER
&&
(/*(bGblGoggleGame==TRUE)&&*/
(((ULONG)(counter_since_change-lwGblSystemTickTotal))>(ULONG)limit+1)&&
(((ULONG)(new_ticks-lwGblSystemTickTotal))>(ULONG)limit+1))
&&(!tsCPURegisters.tIRQFlag)

#endif
  )
  {
   if ( (!t6522SHFTREGACT) )
   {
    if (i6809status!=CPU_FETCH)
    {
    /*******************/
    /* NO PIA ACTIVITY */
    /*******************/
    #include "multicpu.cc"
    }
    else
    {
     #include "jfctgcpu.cc"
    }
   }
   else
   {
    /******************************/
    /* PIA JUST SHIFTING ACTIVITY */
    /******************************/
    #include "mshftcpu.cc"
   }
  }
  else
  {
   /*********************/
   /* PIA FULL ACTIVITY */
   /*********************/
   do
   {
    #include "msnglcpu.cc"
   }
   while (t1int==TRUE);
  }
#ifdef SEQUENCER_IMAGER
  if (lwGblSystemTickTotal>new_ticks)
  {
   new_ticks=lwGblSystemTickTotal+lwGblWheelTicks;
   state=(state+1)%2;
   if (state==0)
   {
    t6522IFLG|=E6522IFLG_CA1;
    t6522IRA&=127;
    if (t6522IER&0x02)
    {
     fDebug(HIGH,("f6522Tick() - IRQ asserted by 6522 CA1 Reg at PC=<%04x>",tsCPURegisters.tPC));
     fDebug(HIGH,("Imager Syncronizinig"));
     tsCPURegisters.tIRQFlag=TRUE; /* keyboard 2 normaler IRQ */
    }
   }
  }
  if (lwGblSystemTickTotal>counter_since_change)
  {
   switch (wGblImagerMode)
   {
//# 0 links einfarbig  \ eigene overlays moeglich
//# 1 rechts einfarbig /
//# 2 rot blau
//# 3 links farbig
//# 4 rechts farbig
//# 5 farbig (chaos)
//# 6 links links, rechts rechts farbig
    case 0: // einfarbig rechts
    {
     counter_since_change=lwGblSystemTickTotal+lwGblWheelTicks;
     if (state==1)
     {
      ImagerColorNow=IMAGER_DRAW_OVERLAY;
     }
     else
     {
      ImagerColorNow=IMAGER_DRAW_NOTHING;
     }
     break;
    }
    case 1: // einfarbig links
    {
     counter_since_change=lwGblSystemTickTotal+lwGblWheelTicks;
     if (state==1)
     {
      ImagerColorNow=IMAGER_DRAW_NOTHING;
     }
     else
     {
      ImagerColorNow=IMAGER_DRAW_OVERLAY;
     }
     break;
    }
    case 2: // rot blau
    {
     counter_since_change=lwGblSystemTickTotal+lwGblWheelTicks;
     if (state==1)
     {
      ImagerColorNow=IMAGER_DRAW_COLOR_16;
     }
     else
     {
      ImagerColorNow=IMAGER_DRAW_COLOR_15;
     }
     break;
    }
    case 3: //mehrfarbig rechts
    {
     if (state==0)
     {
      if (wheel_color_rotator==0)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_1;
       counter_since_change=lwGblSystemTickTotal+wGblColor1Degrees;
      }
      else if (wheel_color_rotator==1)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_2;
       counter_since_change=lwGblSystemTickTotal+wGblColor2Degrees;
      }
      else
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_3;
       counter_since_change=lwGblSystemTickTotal+wGblColor3Degrees;
       new_ticks=lwGblSystemTickTotal+wGblColor3Degrees; // for syncronization
      }
      wheel_color_rotator=(wheel_color_rotator+1)%3;
     }
     else
     {
      if (wheel_color_rotator==0)
      {
       ImagerColorNow=IMAGER_DRAW_NOTHING;
       counter_since_change=lwGblSystemTickTotal+wGblColor1Degrees;
      }
      else if (wheel_color_rotator==1)
      {
       ImagerColorNow=IMAGER_DRAW_NOTHING;
       counter_since_change=lwGblSystemTickTotal+wGblColor2Degrees;
      }
      else
      {
       ImagerColorNow=IMAGER_DRAW_NOTHING;
       counter_since_change=lwGblSystemTickTotal+wGblColor3Degrees;
       new_ticks=lwGblSystemTickTotal+wGblColor3Degrees; // for syncronization
      }
      wheel_color_rotator=(wheel_color_rotator+1)%3;
     }
     break;
    }
    case 4: //mehrfarbig links
    {
     if (state==1)
     {
      if (wheel_color_rotator==0)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_1;
       counter_since_change=lwGblSystemTickTotal+wGblColor1Degrees;
      }
      else if (wheel_color_rotator==1)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_2;
       counter_since_change=lwGblSystemTickTotal+wGblColor2Degrees;
      }
      else
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_3;
       counter_since_change=lwGblSystemTickTotal+wGblColor3Degrees;
       new_ticks=lwGblSystemTickTotal+wGblColor3Degrees; // for syncronization
      }
      wheel_color_rotator=(wheel_color_rotator+1)%3;
     }
     else
     {
      if (wheel_color_rotator==0)
      {
       ImagerColorNow=IMAGER_DRAW_NOTHING;
       counter_since_change=lwGblSystemTickTotal+wGblColor1Degrees;
      }
      else if (wheel_color_rotator==1)
      {
       ImagerColorNow=IMAGER_DRAW_NOTHING;
       counter_since_change=lwGblSystemTickTotal+wGblColor2Degrees;
      }
      else
      {
       ImagerColorNow=IMAGER_DRAW_NOTHING;
       counter_since_change=lwGblSystemTickTotal+wGblColor3Degrees;
       new_ticks=lwGblSystemTickTotal+wGblColor3Degrees; // for syncronization
      }
      wheel_color_rotator=(wheel_color_rotator+1)%3;
     }
     break;
    }
    case 5: //mehrfarbig chaos
    {
     if (state==0)
     {
      if (wheel_color_rotator==0)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_1;
       counter_since_change=lwGblSystemTickTotal+wGblColor1Degrees;
      }
      else if (wheel_color_rotator==1)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_2;
       counter_since_change=lwGblSystemTickTotal+wGblColor2Degrees;
      }
      else
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_3;
       counter_since_change=lwGblSystemTickTotal+wGblColor3Degrees;
       new_ticks=lwGblSystemTickTotal+wGblColor3Degrees; // for syncronization
      }
      wheel_color_rotator=(wheel_color_rotator+1)%3;
     }
     else
     {
      if (wheel_color_rotator==0)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_1;
       counter_since_change=lwGblSystemTickTotal+wGblColor1Degrees;
      }
      else if (wheel_color_rotator==1)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_2;
       counter_since_change=lwGblSystemTickTotal+wGblColor2Degrees;
      }
      else
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_3;
       counter_since_change=lwGblSystemTickTotal+wGblColor3Degrees;
       new_ticks=lwGblSystemTickTotal+wGblColor3Degrees; // for syncronization
      }
      wheel_color_rotator=(wheel_color_rotator+1)%3;
     }
     break;
    }
    case 6: //mehrfarbig links rechts
    {
     if (state==0)
     {
      wGblImagerXOffset=wGblImagerXOffset1;
      if (wheel_color_rotator==0)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_1;
       counter_since_change=lwGblSystemTickTotal+wGblColor1Degrees;
      }
      else if (wheel_color_rotator==1)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_2;
       counter_since_change=lwGblSystemTickTotal+wGblColor2Degrees;
      }
      else
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_3;
       counter_since_change=lwGblSystemTickTotal+wGblColor3Degrees;
       new_ticks=lwGblSystemTickTotal+wGblColor3Degrees; // for syncronization
      }
      wheel_color_rotator=(wheel_color_rotator+1)%3;
     }
     else
     {
      wGblImagerXOffset=wGblImagerXOffset2;
      if (wheel_color_rotator==0)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_1;
       counter_since_change=lwGblSystemTickTotal+wGblColor1Degrees;
      }
      else if (wheel_color_rotator==1)
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_2;
       counter_since_change=lwGblSystemTickTotal+wGblColor2Degrees;
      }
      else
      {
       ImagerColorNow=IMAGER_DRAW_WHEEL_COLOR_3;
       counter_since_change=lwGblSystemTickTotal+wGblColor3Degrees;
       new_ticks=lwGblSystemTickTotal+wGblColor3Degrees; // for syncronization
      }
      wheel_color_rotator=(wheel_color_rotator+1)%3;
     }
     break;
    }





   }
  }
#endif
  /***********************************/
  /* Emulate the monitor persistance */
  /***********************************/
  if (lwGblSystemTickTotal>lwGblDisplayTickCounter)
  {
   lwGblDisplayTickCounter=lwGblSystemTickTotal+lwGblDisplayTickReload;
   if (bGblDisplayEnable)
   {
//    fDisplayTick();
    #ifdef SEQUENCER_IMAGER
    #define DISPLAY_TICK_IMAGER
    #endif

    #ifdef SEQUENCER_LINEAR
    #define DISPLAY_TICK_LINEAR
    #include "dtick.cc"
    #undef DISPLAY_TICK_LINEAR
    #endif

    #ifdef SEQUENCER_BANKED
    #define DISPLAY_TICK_BANKED
    #include "dtick.cc"
    #undef DISPLAY_TICK_BANKED
    #endif

    #ifdef SEQUENCER_IMAGER
    #undef DISPLAY_TICK_IMAGER
    #endif

   }
  }
  /****************************************/
  /* Use ... for speed regulator checking */
  /****************************************/
  if (lwGblSystemTickTotal>tick_timeout)
  {
#ifndef DEBUG_MSG
   if (!tTicker--)
   {
    tTicker=3;
    if(bKBDKeyPressed)
    {
      fKBDdokey();
    }
   }
#endif
   while (last_timer_tick==timer_tick)
   {
    /****************************************************/
    /* Pause until timer rollover, check for timeout ?? */
    /****************************************************/
    ;
   }
   last_timer_tick=timer_tick;
   tick_timeout=lwGblSystemTickTotal+lwGblMaxTicks;
  }
#ifdef CLAY_ONLY
  /****************************************/
  /* Use ... for speed regulator checking */
  /****************************************/
  if (cscreenshots==TRUE)
  if (grabticker<=0)
  {
   grabticker=lwGblTicksToGrab;
   fIOPrintf(0,0,31,0,1,"      ");
   fIOPrintf(0,20,31,0,1,"        ");
   fIOpcx_write_256pic();
   fIOPrintf(0,0,31,0,1,"Grabs:");
   fIOPrintf(0,20,31,0,1,"%08i",cgrabsdone);
   cgrabsdone++;
  }
#endif




  /*****************************************************************/
  /* Because the interrupt driver keyboard handler doesnt yet      */
  /* exist we must use fIODoKey to poll the keyboard once per loop */
  /* Only do this every 65536 sequence loops as this is a very     */
  /* slow process until I write my own driver                      */
  /*****************************************************************/
#ifdef DEBUG_MSG
  if (bKBDKeyPressed)
   fKBDdokey();
#endif
  /****************************************/
  /* Special debugging code for DP sanity */
  /****************************************/
//     if(tsCPURegisters.tDP!=0 && tsCPURegisters.tDP!=0xc8 && tsCPURegisters.tDP!=0xd0)
//     {
//         fFatal("fVectrexEmulate() - Last<%04x> Now<%04x>",iLastAddress,tsCPURegisters.tPC);
//         bGblDebugMonitorEnable=TRUE;
//     }

  /*****************************************/
  /* Execute the Debug function if enabled */
  /*****************************************/
#ifdef DEBUG_MON
  if(bGblDebugMonitorEnable)
  {
   bGblStatsValid=FALSE;
   fIOSetClipping(-1,-1,-1,-1);
   fSoundMute();
   fKBDbuffering(TRUE);
   fDebugMonitor();
   fKBDbuffering(FALSE);
   fSoundResume();
  }
#endif
 }

 if (bGblSaveOnExit==TRUE)
 {
  fSaveState(0);
 }
 /******************************************/
 /* For now always return Emulator success */
 /******************************************/
 fKBDbuffering(TRUE);

 #ifdef DEBUG_MSG
 printf("sync=%i\n",sync);
 printf("cwai=%i\n",cwai);
 printf("ex=%i\n",ex);
 printf("wait=%i\n",wait);
 printf("halt=%i\n",halt);
 #endif
 return(TRUE);
}






