/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : AY-3-8192 Sound Chip emulation module                      */
/*                                                                           */
/*   File Name  : SOUNDEMU.C                                                 */
/*                                                                           */
/*   Author     : Keith Wilkins  (ADLIB Sound code by Marcel De Kogel)       */
/*                                                                           */
/*   Version    : 1.00                                                       */
/*                                                                           */
/*   Desciption : This file contains the functions that emulate the AY-3-8192*/
/*                Chip. Any emulation functions calling the read write       */
/*                functions will cause an immediate update to the chip and   */
/*                and all associated variables.                              */
/*                                                                           */
/*   Functions  : fSoundInit()      - Init the AY-3-8192 Sound chip          */
/*                fSoundTick()      - Emulate one time tick of the chip      */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    14/03/93  K.W  Creation of empty file and descriptions         */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*                                                                           */
/*****************************************************************************/

#define SOUNDEMU_C

#include "vectrex.h"


// Sound variables


#define NUM_CHANNELS            18
#define PSG_VOICE               0
#define SCC_VOICE               9
#define PSG_VOICE2              14


 unsigned char tOLD_BC1=0;
 unsigned char tOLD_BDIR=0;
 UWORD tAddr=0;

// static BOOL    got_sb=FALSE;
static UWORD   PSG_Reg[14];

// static unsigned SCC_Reg[32];

static unsigned char   kbf[NUM_CHANNELS];                 /* KEY_BLOCK_F-NUMBER */
static UWORD   ChannelFreq[NUM_CHANNELS];
static unsigned char   tone_on[NUM_CHANNELS];
static unsigned char   key_on[NUM_CHANNELS];
static UWORD   FMPoort=0x388;
static BOOL    sound_on=TRUE;
static BOOL    sound_is_on=TRUE;
static unsigned char   vol_is_nul[NUM_CHANNELS];
static unsigned char   *channel_voice[NUM_CHANNELS];
static BOOL    OPL3=TRUE;
// static BOOL    Do_SCC=FALSE;

static UWORD OpAd[]=
{ 0x000,0x001,0x002,0x008,0x009,0x00A,0x010,0x011,0x012,
  0x100,0x101,0x102,0x108,0x109,0x10A,0x110,0x111,0x112 };

static UWORD ChanAd[]=
{ 0x000,0x001,0x002,0x003,0x004,0x005,0x006,0x007,0x008,
  0x100,0x101,0x102,0x103,0x104,0x105,0x106,0x107,0x108 };

// static unsigned char SCC_Melodic[11]=
// {
//   0x21,0xA1,    /* AM_VIB_EG-TYP_KSR_MULTI */
//   0x18,0x00,    /* KSL_TL                  */
//   0xFF,0xFF,    /* AR_DR                   */
//   0x08,0x08,    /* SL_RR                   */
//   0x02,0x00,    /* WS                      */
//   0x06          /* FB_FM                   */
// };

static unsigned char PSG_Melodic1[11]=
{
  0x23,0x21,    /* AM_VIB_EG-TYP_KSR_MULTI */
  0x1E,0x08,    /* KSL_TL                  */
  0xFF,0xFF,    /* AR_DR                   */
  0x08,0x08,    /* SL_RR                   */
  0x00,0x00,    /* WS                      */
  0x0E          /* FB_FM                   */
};

static unsigned char PSG_Melodic2[11]=
{
  0x21,0x21,    /* AM_VIB_EG-TYP_KSR_MULTI */
  0x1E,0x0A,    /* KSL_TL                  */
  0xFF,0xFF,    /* AR_DR                   */
  0x08,0x08,    /* SL_RR                   */
  0x00,0x00,    /* WS                      */
  0x0F          /* FB_FM                   */
};

static unsigned char PSG_Noise[11]=
{
  0x2E,0x2E,    /* AM_VIB_EG-TYP_KSR_MULTI */
  0x40,0x00,    /* KSL_TL                  */
  0xF0,0xF0,    /* AR_DR                   */
  0xFA,0xFA,    /* SL_RR                   */
  0x00,0x03,    /* WS                      */
  0x0E          /* FB_FM                   */
};

static UWORD PSG_NoiseFreqs[32]=
{
 0,930,930,930,930,930,930,930,930,930,930,930,930,930,930,930,
 930,930,930,930,930,930,930,930,930,930,930,930,930,930,930,930,
};

static UWORD FMFreqs[] =
{
#include "FMFreqs.h"
};

/* Sound blaster stuff */

// PORT ADDRESSES
#define MIXER_ADDRESS           (sb_port+0x04)
#define MIXER_DATA              (sb_port+0x05)
#define DSP_RESET               (sb_port+0x06)
#define DSP_READ_DATA           (sb_port+0x0a)
#define DSP_WRITE_DATA          (sb_port+0x0c)
#define DSP_WRITE_STATUS        (sb_port+0x0c)
#define DSP_DATA_AVAIL          (sb_port+0x0e)

// DSP COMMANDS
#define SPEAKER_ON       (0xd1)
#define SPEAKER_OFF      (0xd3)
#define DIRECT_DAC       (0x10)

// DO A DSP COMMAND
#define writedac(x)                        \
{                                          \
 while(inp(DSP_WRITE_STATUS) & 0x80);      \
 outp(DSP_WRITE_DATA,(x));                 \
}
int digi_sound_possible=FALSE;
unsigned short int sb_port;          // sb base port
unsigned short int sb_ver;           // DSP version number

// Give access to CPU registers for IRQ flags
extern CPUREGISTERS  tsCPURegisters;


// Private functions

static void    WriteFM_OPL2(unsigned char value,unsigned char index);
static void    WriteFM_OPL3_1(unsigned char value,unsigned char index);
static void    WriteFM_OPL3_2(unsigned char value,unsigned char index);
static void    WriteFM(unsigned char value,UWORD index);
static unsigned char   ReadFM(void);
static void    Adlib_ChannelOff(unsigned char channel);
static void    Adlib_ChannelOn(unsigned char channel);
static void    WriteVolume(unsigned char vol,unsigned char reg,UWORD index);
static void    Adlib_SetVolume(unsigned char channel,unsigned char vol);
static void    Adlib_SetVibrato(unsigned char channel);
static void    Adlib_ResetVibrato(unsigned char channel);
static void    Adlib_SetFreq(unsigned char channel,UWORD freq);
static void    Adlib_SetVoice(unsigned char *voice,SWORD channel);
static void    Adlib_SetVoices(void);
static BOOL    Reset_Adlib(void);
static void    Adlib_ToneOn(unsigned char channel,unsigned char on);
static UWORD   GetPSGVolume(unsigned char v);

int SB_WaitDSPWrite(void);
int SB_WaitDSPRead(void);
int SB_WriteDSP(unsigned char data);
unsigned short int SB_ReadDSP(void);
int SB_Ping(void);
unsigned short int SB_GetDSPVersion(void);
void SB_ResetDSP(void);
int SB_IsThere(void);
int SB_Init(void);
void SB_Exit(void);

/*****************************************************************************/
/*                                                                           */
/*   Function    : fSoundInit()                                              */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Initialise the sound emulation functions                  */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

BOOL fSoundInit(void)
{
  fSoundReset();
  return(TRUE);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : fSoundReset()                                             */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Do a hardware reset of the sound chip                     */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

void fSoundReset(void)
{
 /* If Adlib is not to be used, don't initialize */
 if (!bGblSoundEnable)
  return;
 if (SB_Init()==TRUE)
 {
  digi_sound_possible=TRUE;
  writedac(SPEAKER_ON);
  if (bGblDisplayVerbose==TRUE)
   printf("\nSoundblaster init OK, digisound possible...");
 }
 else
 {
  if (bGblDisplayVerbose==TRUE)
   printf("\nSoundblaster init Failed, no digisound...");
 }
 if(!Reset_Adlib())
 {
  bGblSoundEnable=FALSE;
  if (bGblDisplayVerbose==TRUE)
   printf("\nAdlib sound init failed...");
  return;
 }
 if (bGblDisplayVerbose==TRUE)
  printf("\nAdlib sound init OK...");
 if (OPL3)
 {
  if((ReadFM()&6)==0)
  {
   WriteFM(1,0x105);
   WriteFM(0,0x104);
  }
  else
  {
   OPL3=0;
  }
 }
 memset(vol_is_nul,0,sizeof(vol_is_nul));
 memset(tone_on,1,sizeof(tone_on));
 WriteFM(0x00,0xBD);
 WriteFM(0,8);
 Adlib_SetVoices();
 return;
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : fSoundRestore()                                           */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Shutdown the sound chip before exit.                      */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

void fSoundRestore(void)
{
 SWORD i;
 if(!bGblSoundEnable)
  return;
 for(i=PSG_VOICE+0;i<PSG_VOICE+6;++i)
 {
  Adlib_ChannelOff(i);
  Adlib_SetVolume(i,0);
 }
 if(OPL3)
 {
  for(i=PSG_VOICE2+0;i<PSG_VOICE2+3;++i)
  {
   Adlib_ChannelOff(i);
   Adlib_SetVolume(i,0);
  }
 }
 if (OPL3)
  WriteFM(0,0x105);
 OPL3=0;
 Reset_Adlib();
 WriteFM(0x00,0xBD);
 WriteFM(0,8);
 if (digi_sound_possible==TRUE)
  SB_Exit();
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : fSoundTick()                                              */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : Do one unit of sound chip emulation.                      */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : nothing                                                   */
/*                                                                           */
/*****************************************************************************/

void fSoundTick(void)
{
  unsigned char        tTmp;
  unsigned char        tKeyWord;
  static int last_was_digi_sound=FALSE;
#ifdef DIGI_SOUND_HACK
  static int spike192=FALSE;
  if (bGblSoundEnable)
  if (digi_sound_possible==TRUE)
  if (digisound==true)
  {
   unsigned char tData=(unsigned char)((((signed char)(t6522ORA))+(signed char)(128)));
   tData=(tData*wGblDigitalVolume)/100;
   if (last_was_digi_sound==FALSE)
   {
    last_was_digi_sound=TRUE;

/* this is truely ugly, can anybody tell me why spike
  initiates digiatal sound output (like I see it) all
  thru the game? resulting without that patch in
  a horrible background noise...
  Now all samples beginning with 192 are not played yuk!
*/
    if (t6522ORA==192)
     spike192=TRUE;
    else
     spike192=FALSE;
   }
   if (spike192==FALSE)
   {
    writedac(DIRECT_DAC);
    writedac(tData);
   }
   if (tOLD_BC1!=tSIGDEF_AY3BC1 || tOLD_BDIR!=tSIGDEF_AY3BDIR)
   {
    tTmp=tOLD_BDIR|tOLD_BC1;
    fDebug(MED,("fSoundTick() - %02x",tTmp));
    if ((tTmp==0x08)&&(tAddr==0x0e))
    {
     // Build the key char
     tKeyWord =(fKBDisdown(wGblPly2Button4))?0:1<<7;
     tKeyWord|=(fKBDisdown(wGblPly2Button3))?0:1<<6;
     tKeyWord|=(fKBDisdown(wGblPly2Button2))?0:1<<5;
     tKeyWord|=(fKBDisdown(wGblPly2Button1))?0:1<<4;
     tKeyWord|=(fKBDisdown(wGblPly1Button4))?0:1<<3;
     tKeyWord|=(fKBDisdown(wGblPly1Button3))?0:1<<2;
     tKeyWord|=(fKBDisdown(wGblPly1Button2))?0:1<<1;
     tKeyWord|=(fKBDisdown(wGblPly1Button1))?0:1<<0;
     if (fKBDisdown(wGblPly1Button4))
     {
      // fixme, also IRQ for player2 button 4
      tsCPURegisters.tFIRQFlag=TRUE;
     }
     if ((t6522IER&0x02)&&(fKBDisdown(wGblPly2Button4)))
     {
      fDebug(HIGH,("f6522Tick() - IRQ asserted by 6522 CA1 Reg at PC=<%04x>",tsCPURegisters.tPC));
      tsCPURegisters.tIRQFlag=TRUE; /* CA1 -> keyboard 2 normaler IRQ ??? */
     }
     fDebug(MED,("fSoundTick() - Joypad button returned %02x",tKeyWord));
     t6522IRA=tKeyWord;
    }
   }
   tOLD_BC1=tSIGDEF_AY3BC1;
   tOLD_BDIR=tSIGDEF_AY3BDIR;
   return;
  }
#endif
  if (tOLD_BC1!=tSIGDEF_AY3BC1 || tOLD_BDIR!=tSIGDEF_AY3BDIR)
  {
    if (last_was_digi_sound==TRUE)
    {
     last_was_digi_sound=FALSE;
    }
    tTmp=tOLD_BDIR|tOLD_BC1;
    fDebug(MED,("fSoundTick() - %02x",tTmp));
    switch(tTmp)
    {
       case 0x00:   // Back to the inactive state
           break;

       case 0x08:   // Read from AY-3-8192

           switch(tAddr)
           {
               case 0x0e:

               // Build the key char

                   tKeyWord =(fKBDisdown(wGblPly2Button4))?0:1<<7;
                   tKeyWord|=(fKBDisdown(wGblPly2Button3))?0:1<<6;
                   tKeyWord|=(fKBDisdown(wGblPly2Button2))?0:1<<5;
                   tKeyWord|=(fKBDisdown(wGblPly2Button1))?0:1<<4;
                   tKeyWord|=(fKBDisdown(wGblPly1Button4))?0:1<<3;
                   tKeyWord|=(fKBDisdown(wGblPly1Button3))?0:1<<2;
                   tKeyWord|=(fKBDisdown(wGblPly1Button2))?0:1<<1;
                   tKeyWord|=(fKBDisdown(wGblPly1Button1))?0:1<<0;

                   if (fKBDisdown(wGblPly1Button4))
                    tsCPURegisters.tFIRQFlag=TRUE;

                   if ((t6522IER&0x02)&&(fKBDisdown(wGblPly2Button4)))
                   {
                    fDebug(HIGH,("f6522Tick() - IRQ asserted by 6522 CA1 Reg at PC=<%04x>",tsCPURegisters.tPC));
                    tsCPURegisters.tIRQFlag=TRUE; /* CA1 -> keyboard 2 normaler IRQ ??? */
                   }
                   fDebug(MED,("fSoundTick() - Joypad button returned %02x",tKeyWord));

                   t6522IRA=tKeyWord;

                   break;

               default:
                   fDebug(MED,("fSoundTick() - Bad register select"));
                   break;

           }
           break;

       case 0x10:   // Write to AY-3-8192

           {
               unsigned char tData=t6522ORA;
               UWORD tmp;
               if (!bGblSoundEnable) break;
               if (tAddr>13) break;

               switch(tAddr)
               {
                   case 0:
                   case 2:
                   case 4:
                       Adlib_SetFreq((tAddr>>1)+PSG_VOICE,tData+(PSG_Reg[tAddr+1]<<8));
                       if(OPL3) Adlib_SetFreq((tAddr>>1)+PSG_VOICE2,tData+(PSG_Reg[tAddr+1]<<8));
                       break;

                   case 1:
                   case 3:
                   case 5:
                       tData&=0x0F;
                       Adlib_SetFreq((tAddr>>1)+PSG_VOICE,(tData<<8)+PSG_Reg[tAddr-1]);
                       if(OPL3) Adlib_SetFreq((tAddr>>1)+PSG_VOICE2,(tData<<8)+PSG_Reg[tAddr-1]);
                       break;

                   case 6:
                       tData&=0x1F;
                       Adlib_SetFreq(PSG_VOICE+3,PSG_NoiseFreqs[tData]);
                       Adlib_SetFreq(PSG_VOICE+4,PSG_NoiseFreqs[tData]);
                       Adlib_SetFreq(PSG_VOICE+5,PSG_NoiseFreqs[tData]);
                       break;

                   case 7:
                       Adlib_ToneOn(0+PSG_VOICE,tData&0x01);
                       Adlib_ToneOn(1+PSG_VOICE,tData&0x02);
                       Adlib_ToneOn(2+PSG_VOICE,tData&0x04);
                       if(OPL3)
                       {
                           Adlib_ToneOn(0+PSG_VOICE2,tData&0x01);
                           Adlib_ToneOn(1+PSG_VOICE2,tData&0x02);
                           Adlib_ToneOn(2+PSG_VOICE2,tData&0x04);
                       }
                       Adlib_ToneOn(3+PSG_VOICE,tData&0x08);
                       Adlib_ToneOn(4+PSG_VOICE,tData&0x10);
                       Adlib_ToneOn(5+PSG_VOICE,tData&0x20);
                       break;

                   case 8:
                   case 9:
                   case 10:
                       tData&=0x1F;
                       tmp=GetPSGVolume(tData);
                       if (tmp<256)
                       {
                           Adlib_ResetVibrato(tAddr-8+PSG_VOICE);
                           Adlib_ResetVibrato(tAddr-8+3+PSG_VOICE);
                           Adlib_SetVolume(tAddr-8+PSG_VOICE,tmp);
                           Adlib_SetVolume(tAddr-8+3+PSG_VOICE,tmp);
                           if (OPL3)
                           {
                               Adlib_ResetVibrato(tAddr-8+PSG_VOICE2);
                               Adlib_SetVolume(tAddr-8+PSG_VOICE2,tmp);
                           }
                       }
                       else
                       {
                           Adlib_SetVolume(tAddr-8+PSG_VOICE,12);
                           if (OPL3) Adlib_SetVolume(tAddr-8+PSG_VOICE2,12);
                           Adlib_SetVolume(tAddr-8+3+PSG_VOICE,12);

                           switch (tmp&0x0f)
                           {
                               case 0x00:
                               case 0x01:
                               case 0x02:
                               case 0x03:
                               case 0x09:
                               case 0x04:
                               case 0x05:
                               case 0x06:
                               case 0x07:
                               case 0x0F:
                                   Adlib_ResetVibrato(tAddr-8+PSG_VOICE);
                                   if(OPL3) Adlib_ResetVibrato(tAddr-8+PSG_VOICE);
                                   Adlib_ResetVibrato(tAddr-8+3+PSG_VOICE);
                                   if(tone_on[tAddr-8+PSG_VOICE])
                                   {
                                       if(key_on[tAddr-8+PSG_VOICE]==0)
                                       {
                                           Adlib_ChannelOn(tAddr-8+PSG_VOICE);
                                           if(OPL3) Adlib_ChannelOn(tAddr-8+PSG_VOICE2);
                                       }
                                       Adlib_SetVolume(tAddr-8+PSG_VOICE,0);
                                       if(OPL3) Adlib_SetVolume(tAddr-8+PSG_VOICE2,0);
                                   }
                                   if(tone_on[tAddr-8+3+PSG_VOICE])
                                   {
                                       if(key_on[tAddr-8+3+PSG_VOICE]==0) Adlib_ChannelOn(tAddr-8+3+PSG_VOICE);
                                       Adlib_SetVolume(tAddr-8+3+PSG_VOICE,0);
                                   }
                                   break;
                               case 0x08:
                               case 0x0A:
                               case 0x0C:
                               case 0x0E:
                                   Adlib_SetVibrato(tAddr-8+PSG_VOICE);
                                   if(OPL3) Adlib_SetVibrato (tAddr-8+PSG_VOICE2);
                                   Adlib_SetVibrato(tAddr-8+3+PSG_VOICE);
                                   break;
                               default:
                                   Adlib_ResetVibrato(tAddr-8+PSG_VOICE);
                                   if(OPL3) Adlib_ResetVibrato (tAddr-8+PSG_VOICE2);
                                   Adlib_ResetVibrato (tAddr-8+3+PSG_VOICE);
                                   break;
                           }
                       }
                       break;

                   case 11:              /* bits 7-0 vd snelheid vh volumeverloop */
                       break;

                   case 12:              /* bits 15-8 */
                       break;

                   case 13:              /* soort volumeverloop */
                       tData&=0x0F;
                       break;
                }
                PSG_Reg[tAddr]=tData;
           }
           fDebug(MED,("fSoundTick() - Write"));
           break;

       case 0x18:   // Latch address to AY-3-8192

           fDebug(MED,("fSoundTick() - Latch Address"));
           tAddr=t6522ORA&0x0f;
           break;

       default:
           break;

    }
  }

  tOLD_BC1=tSIGDEF_AY3BC1;
  tOLD_BDIR=tSIGDEF_AY3BDIR;
}

void fSoundMute(void)
{
 SWORD i;
 if (!bGblSoundEnable)
  return;
 sound_is_on=FALSE;
 for(i=PSG_VOICE+0;i<PSG_VOICE+6;++i)
  Adlib_ChannelOff (i);
 if(OPL3) for(i=PSG_VOICE2+0;i<PSG_VOICE2+3;++i)
  Adlib_ChannelOff (i);
}

void fSoundResume(void)
{
 SWORD i;
 if (!bGblSoundEnable)
  return;
 if (!sound_on)
  return;
 sound_is_on=TRUE;
 for (i=PSG_VOICE+0;i<PSG_VOICE+6;++i)
  if (tone_on[i])
   Adlib_ChannelOn(i);
 if (OPL3)
  for (i=PSG_VOICE2+0;i<PSG_VOICE2+3;++i)
   if (tone_on[i])
    Adlib_ChannelOn(i);
}
//
//
// PRIVATE FUNCTIONS
//
//
static void WriteFM_OPL2(unsigned char value,unsigned char index)
{
 SWORD i;
 outp(FMPoort,index);
 for(i=0;i<6;++i)
  inp(FMPoort);
 outp(FMPoort+1,value);
 for(i=0;i<35;++i)
  inp(FMPoort);
}

static void WriteFM_OPL3_1(unsigned char value,unsigned char index)
{
 outp(FMPoort,index);
 inp(FMPoort);
 outp(FMPoort+1,value);
 inp(FMPoort);
 inp(FMPoort);
}

static void WriteFM_OPL3_2(unsigned char value,unsigned char index)
{
 outp(FMPoort+2,index);
 inp(FMPoort);
 outp(FMPoort+3,value);
 inp(FMPoort);
 inp(FMPoort);
}

static void WriteFM(unsigned char value,UWORD index)
{
 if(OPL3)
 {
  if (index&0x100)
   WriteFM_OPL3_2 (value,(unsigned char)index);
  else
   WriteFM_OPL3_1 (value,(unsigned char)index);
 }
 else
 {
 if ((index&0x100)==0)
  WriteFM_OPL2(value,(unsigned char)index);
 }
}

static unsigned char ReadFM(void)
{
 return inp(FMPoort);
}

static void Adlib_ChannelOff(unsigned char channel)
{
 WriteFM(kbf[channel],0xB0+ChanAd[channel]);
 key_on[channel]=0;
}

static void Adlib_ChannelOn(unsigned char channel)
{
 if (!sound_on)
  return;
 if (vol_is_nul[channel])
  return;
 WriteFM(kbf[channel]|32,0xB0+ChanAd[channel]);
 key_on[channel]=1;
}

static void WriteVolume(unsigned char vol,unsigned char reg,UWORD index)
{
 UWORD tl;
 tl=63-(reg&63);
 tl*=(64-2*(15-(vol&15)));
 tl/=64;
 if(tl>=64) tl=63;
 tl=63-tl;
 WriteFM((reg&0xC0)|tl,index);
}

static void Adlib_SetVolume(unsigned char channel,unsigned char vol)
{
 if (vol==0)
 {
  vol_is_nul[channel]=1;
  if (tone_on[channel])
   Adlib_ChannelOff(channel);
  return;
 }
 if (vol_is_nul[channel])
 {
  vol_is_nul[channel]=0;
  if (tone_on[channel])
   Adlib_ChannelOn (channel);
 }
 if (*(channel_voice[channel]+10)&1)  /* additive synthesis */
  WriteVolume(vol,*(channel_voice[channel]+2),0x40+OpAd[channel]);
 WriteVolume(vol,*(channel_voice[channel]+3),0x43+OpAd[channel]);
}

static void Adlib_SetVibrato(unsigned char channel)
{
 if (*(channel_voice[channel]+10)&1)
  WriteFM (*(channel_voice[channel]+0)|0x80,0x20+OpAd[channel]);
 WriteFM (*(channel_voice[channel]+1)|0x80,0x23+OpAd[channel]);
}

static void Adlib_ResetVibrato(unsigned char channel)
{
 if (*(channel_voice[channel]+10)&1)
  WriteFM (*(channel_voice[channel]+0),0x20+OpAd[channel]);
 WriteFM (*(channel_voice[channel]+1),0x23+OpAd[channel]);
}

static void Adlib_SetFreq(unsigned char channel,UWORD freq)
{
 SWORD block,f;
 ChannelFreq[channel]=freq;
 f=FMFreqs[freq*2+1];
 block=FMFreqs[freq*2];
 WriteFM(f&0xFF,0xA0+ChanAd[channel]);
 kbf[channel]=(f>>8)+(block<<2);
 if (tone_on[channel])
 {
  if (f)
   Adlib_ChannelOn(channel);
  else
   Adlib_ChannelOff(channel);
 }
}

static void Adlib_SetVoice(unsigned char *voice,SWORD channel)
{
 WriteFM(voice[0],0x20+OpAd[channel]);
 WriteFM(voice[1],0x23+OpAd[channel]);
 WriteFM(voice[2],0x40+OpAd[channel]);
 WriteFM(voice[3],0x43+OpAd[channel]);
 WriteFM(voice[4],0x60+OpAd[channel]);
 WriteFM(voice[5],0x63+OpAd[channel]);
 WriteFM(voice[6],0x80+OpAd[channel]);
 WriteFM(voice[7],0x83+OpAd[channel]);
 WriteFM(voice[8],0xE0+OpAd[channel]);
 WriteFM(voice[9],0xE3+OpAd[channel]);
 if (OPL3)
  WriteFM(voice[10]|0x30,0xC0+ChanAd[channel]);
 else
  WriteFM(voice[10],0xC0+ChanAd[channel]);
 channel_voice[channel]=voice;
}

static void Adlib_SetVoices(void)
{
 SWORD i;
 for (i=PSG_VOICE;i<PSG_VOICE+3;++i)
 {
  Adlib_SetVoice(PSG_Melodic1,i);
  Adlib_SetVolume(i,0);
 }
 if (OPL3)
 {
  for (i=PSG_VOICE2;i<PSG_VOICE2+3;++i)
  {
   Adlib_SetVoice(PSG_Melodic2,i);
   Adlib_SetVolume(i,15);
  }
 }
 for (i=PSG_VOICE+3;i<PSG_VOICE+6;++i)
 {
  Adlib_SetVoice(PSG_Noise,i);
  Adlib_SetVolume(i,15);
 }
}

static BOOL Reset_Adlib(void)
{
 unsigned char tmp1,tmp2;
 SWORD i;

 WriteFM(0,1);
 WriteFM(0x60,4);
 WriteFM(0x80,4);
 tmp1=ReadFM();
 WriteFM(0xFF,2);
 WriteFM(0x21,4);
 for (i=0;i<300;++i) ReadFM();
 tmp2=ReadFM();
 WriteFM(0x60,4);
 WriteFM(0x80,4);
 if ((tmp1&0xE0)!=0)
  return FALSE;
 if ((tmp2&0xE0)!=0xC0)
  return FALSE;
 WriteFM(0,1);
 return TRUE;
}

static void Adlib_ToneOn(unsigned char channel,unsigned char on)
{
 on=(on)?0:1;
 if (tone_on[channel]!=on)
 {
  tone_on[channel]=on;
  if (tone_on[channel])
   Adlib_ChannelOn(channel);
  else
   Adlib_ChannelOff(channel);
 }
}

static UWORD GetPSGVolume(unsigned char v)
{
 UWORD retval;
 retval=v&0x0F;
 if (v&0x10)
  retval+=0x100;
 return retval;
}

int SB_WaitDSPWrite(void)
/*
 Waits until the DSP is ready to be written to.

 returns FALSE on timeout
*/
{
 unsigned short int timeout=32767;
 while(timeout--)
 {
  if (!(inp(DSP_WRITE_STATUS)&0x80))
   return TRUE;
 }
 return FALSE;
}

int SB_WaitDSPRead(void)
/*
 Waits until the DSP is ready to read from.

 returns FALSE on timeout
*/
{
 unsigned short int timeout=32767;
 while (timeout--)
 {
  if (inp(DSP_DATA_AVAIL)&0x80)
   return TRUE;
 }
 return FALSE;
}

int SB_WriteDSP(unsigned char data)
/*
 Writes char 'data' to the DSP.

 returns FALSE on timeout.
*/
{
 if (!SB_WaitDSPWrite())
  return FALSE;
 outp(DSP_WRITE_DATA,data);
 return TRUE;
}

unsigned short int SB_ReadDSP(void)
/*
 Reads a char from the DSP.

 returns 0xffff on timeout.
*/
{
 if (!SB_WaitDSPRead())
  return 0xffff;
 return (inp(DSP_READ_DATA));
}

int SB_Ping(void)
/*
 Checks if a SB is present at the current baseport by
 resetting the DSP and checking if it returned the value 0xaa.

 returns: TRUE   => SB is present
          FALSE  => No SB detected
*/
{
 SB_ResetDSP();
 if (SB_ReadDSP()==0xaa)
  return TRUE;
 return FALSE;
}

unsigned short int SB_GetDSPVersion(void)
/*
 Gets SB-dsp version. returns 0xffff if dsp didn't respond.
*/
{
 unsigned short int hi,lo;
 if (!SB_WriteDSP(0xe1))
  return 0xffff;

 hi=SB_ReadDSP();
 lo=SB_ReadDSP();
 return ((hi<<8)|lo);
}

void SB_ResetDSP(void)
/*
 Resets the DSP.
*/
{
 // reset the DSP by sending 1, (delay), then 0
 outp(DSP_RESET,1);
 inp(DSP_RESET);
 inp(DSP_RESET);
 inp(DSP_RESET);
 inp(DSP_RESET);
 inp(DSP_RESET);
 inp(DSP_RESET);
 inp(DSP_RESET);
 inp(DSP_RESET);
 outp(DSP_RESET,0);
}

int SB_IsThere(void)
{
 char *envptr,*endptr,c;

 sb_port =0xffff;

 if ((envptr=getenv("BLASTER"))==NULL)
  return FALSE;

 while(1)
 {
  // skip whitespace
  do
   c=*(envptr++);
  while(c==' ' || c=='\t');

  // reached end of string? -> exit
  if (c==0)
   break;
  switch(c)
  {
   case 'a':
   case 'A':
    sb_port=strtol(envptr,&endptr,16);
    break;
   default:
    strtol(envptr,&endptr,16);
    break;
  }
  envptr=endptr;
 }
 if (sb_port==0xffff)
  return FALSE;
 if (!SB_Ping())
  return FALSE;
 // get dsp version.
 if ((sb_ver=SB_GetDSPVersion())==0xffff)
  return FALSE;
 return TRUE;
}

int SB_Init(void)
{
 if (!SB_IsThere())
 {
  // fixme
  printf("\nNo such hardware detected, check your 'BLASTER' env. variable");
  return FALSE;
 }
 SB_ResetDSP();
 return TRUE;
}

void SB_Exit(void)
{
 SB_ResetDSP();
}

/* End of SOUNDEMU.C */






