;*****************************************************************************
;*                                                                           *
;*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  *
;*                                                                           *
;*****************************************************************************
;*                                                                           *
;*   Title      : 6809 Emulator code TFR instruction code                    *
;*                                                                           *
;*   File Name  : TFR6809.ASM                                                *
;*                                                                           *
;*   Author     : Keith Wilkins                                              *
;*                                                                           *
;*   Version    : 1.00                                                       *
;*                                                                           *
;*   Desciption : This file contains the code to emulate the tfr instruction *
;*                a jump table is used to decode the register combinations   *
;*                                                                           *
;*****************************************************************************
;*                                                                           *
;*   Revision History:                                                       *
;*                                                                           *
;*   Version    Date    Who  Description of changes                          *
;*   -------    ----    ---  ----------------------                          *
;*                                                                           *
;*    0.01    22/11/95  K.W  Creation of empty stub file, description only   *
;*    1.00    06/08/96  K.W  Public release of DVE 1.0                       *
;*                                                                           *
;*****************************************************************************

extern f6809AddressRead_:near
extern f6809AddressWrite_:near
extern _Idx6809JumpTable:near
extern f6809cctodl_:near
extern f6809dltocc_:near

include "reg6809.h"
include "mac6809.h"


_TFR6809_TEXT segment dword public 'CODE'
              assume  CS:_TFR6809_TEXT

;
; CPU Exception
;
tfr6809_EX    PROC    near

              add     sp,4               ; Remove return address
              mov     ax,CPU_EXCEPTION
              m_exit
              ret

tfr6809_EX    ENDP



;
; 00 - D,D
;
tfr6809_00     PROC    near

               mov     ax,CPU_FETCH
               ret

tfr6809_00     ENDP


;
; 01 - D,X
;
tfr6809_01     PROC    near

               mov     dx,[R6809_D]
               mov     [R6809_X],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_01     ENDP


;
; 02 - D,Y
;
tfr6809_02     PROC    near

               mov     dx,[R6809_D]
               mov     [R6809_Y],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_02     ENDP


;
; 03 - D,U
;
tfr6809_03     PROC    near

               mov     dx,[R6809_D]
               mov     [R6809_USP],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_03     ENDP


;
; 04 - D,S
;
tfr6809_04     PROC    near

               mov     dx,[R6809_D]
               mov     [R6809_HSP],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_04     ENDP


;
; 05 - D,PC
;
tfr6809_05     PROC    near

               mov     dx,[R6809_D]
               mov     [R6809_PC],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_05     ENDP


;
; 10 - X,D
;
tfr6809_10     PROC    near

               mov     dx,[R6809_X]
               mov     [R6809_D],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_10     ENDP



;
; 11 - X,X
;
tfr6809_11     PROC    near

               mov     ax,CPU_FETCH
               ret

tfr6809_11     ENDP



;
; 12 - X,Y
;
tfr6809_12     PROC    near

               mov     dx,[R6809_X]
               mov     [R6809_Y],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_12     ENDP



;
; 13 - X,U
;
tfr6809_13     PROC    near

               mov     dx,[R6809_X]
               mov     [R6809_USP],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_13     ENDP


;
; 14 - X,S
;
tfr6809_14     PROC    near

               mov     dx,[R6809_X]
               mov     [R6809_HSP],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_14     ENDP



;
; 15 - X,PC
;
tfr6809_15     PROC    near

               mov     dx,[R6809_X]
               mov     [R6809_PC],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_15     ENDP



;
; 20 - Y,D
;
tfr6809_20     PROC    near

               mov     dx,[R6809_Y]
               mov     [R6809_D],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_20     ENDP



;
; 21 - Y,X
;
tfr6809_21     PROC    near

               mov     dx,[R6809_Y]
               mov     [R6809_X],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_21     ENDP



;
; 22 - Y,Y
;
tfr6809_22     PROC    near

               mov     ax,CPU_FETCH
               ret

tfr6809_22     ENDP



;
; 23 - Y,U
;
tfr6809_23     PROC    near

               mov     dx,[R6809_Y]
               mov     [R6809_USP],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_23     ENDP


;
; 24 - Y,S
;
tfr6809_24     PROC    near

               mov     dx,[R6809_Y]
               mov     [R6809_HSP],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_24     ENDP



;
; 25 - Y,PC
;
tfr6809_25     PROC    near

               mov     dx,[R6809_Y]
               mov     [R6809_PC],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_25     ENDP



;
; 30 - U,D
;
tfr6809_30     PROC    near

               mov     dx,[R6809_USP]
               mov     [R6809_D],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_30     ENDP



;
; 31 - U,X
;
tfr6809_31     PROC    near

               mov     dx,[R6809_USP]
               mov     [R6809_X],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_31     ENDP



;
; 32 - U,Y
;
tfr6809_32     PROC    near

               mov     dx,[R6809_USP]
               mov     [R6809_Y],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_32     ENDP



;
; 33 - U,U
;
tfr6809_33     PROC    near

               mov     ax,CPU_FETCH
               ret

tfr6809_33     ENDP



;
; 34 - U,S
;
tfr6809_34     PROC    near

               mov     dx,[R6809_USP]
               mov     [R6809_HSP],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_34     ENDP



;
; 35 - U,PC
;
tfr6809_35     PROC    near

               mov     dx,[R6809_USP]
               mov     [R6809_PC],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_35     ENDP



;
; 40 - S,D
;
tfr6809_40     PROC    near

               mov     dx,[R6809_HSP]
               mov     [R6809_D],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_40     ENDP



;
; 41 - S,X
;
tfr6809_41     PROC    near

               mov     dx,[R6809_HSP]
               mov     [R6809_X],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_41     ENDP



;
; 42 - S,Y
;
tfr6809_42     PROC    near

               mov     dx,[R6809_HSP]
               mov     [R6809_Y],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_42     ENDP



;
; 43 - S,U
;
tfr6809_43     PROC    near

               mov     dx,[R6809_HSP]
               mov     [R6809_USP],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_43     ENDP



;
; 44 - S,S
;
tfr6809_44     PROC    near

               mov     ax,CPU_FETCH
               ret

tfr6809_44     ENDP



;
; 45 - S,PC
;
tfr6809_45     PROC    near

               mov     dx,[R6809_HSP]
               mov     [R6809_PC],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_45     ENDP



;
; 50 - PC,D
;
tfr6809_50     PROC    near

               mov     dx,[R6809_PC]
               mov     [R6809_D],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_50     ENDP



;
; 51 - PC,X
;
tfr6809_51     PROC    near

               mov     dx,[R6809_PC]
               mov     [R6809_X],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_51     ENDP



;
; 52 - PC,Y
;
tfr6809_52     PROC    near

               mov     dx,[R6809_PC]
               mov     [R6809_Y],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_52     ENDP



;
; 53 - PC,U
;
tfr6809_53     PROC    near

               mov     dx,[R6809_PC]
               mov     [R6809_USP],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_53     ENDP



;
; 54 - PC,S
;
tfr6809_54     PROC    near

               mov     dx,[R6809_PC]
               mov     [R6809_HSP],dx
               mov     ax,CPU_FETCH
               ret

tfr6809_54     ENDP



;
; 55 - PC,PC
;
tfr6809_55     PROC    near

               mov     ax,CPU_FETCH
               ret

tfr6809_55     ENDP



;
; 88 - A,A
;
tfr6809_88     PROC    near

               mov     ax,CPU_FETCH
               ret

tfr6809_88     ENDP



;
; 89 - A,B
;
tfr6809_89     PROC    near

               mov     dl,[R6809_A]
               mov     [R6809_B],dl
               mov     ax,CPU_FETCH
               ret

tfr6809_89     ENDP



;
; 8A - A,CC
;
tfr6809_8A     PROC    near

               mov     dl,[R6809_A]
               call    f6809dltocc_
               mov     ax,CPU_FETCH
               ret

tfr6809_8A     ENDP



;
; 8b - A,DP
;
tfr6809_8B     PROC    near

               mov     dl,[R6809_A]
               mov     [R6809_DP],dl
               mov     ax,CPU_FETCH
               ret

tfr6809_8B     ENDP


;
; 98 - B,A
;
tfr6809_98     PROC    near

               mov     dl,[R6809_B]
               mov     [R6809_A],dl
               mov     ax,CPU_FETCH
               ret

tfr6809_98     ENDP



;
; 99 - B,B
;
tfr6809_99     PROC    near

               mov     ax,CPU_FETCH
               ret

tfr6809_99     ENDP



;
; 9A - B,CC
;
tfr6809_9A     PROC    near

               mov     dl,[R6809_B]
               call    f6809dltocc_
               mov     ax,CPU_FETCH
               ret

tfr6809_9A     ENDP



;
; 9B - B,DP
;
tfr6809_9B     PROC    near

               mov     dl,[R6809_B]
               mov     [R6809_DP],dl
               mov     ax,CPU_FETCH
               ret

tfr6809_9B     ENDP



;
; A8 - CC,A
;
tfr6809_A8     PROC    near

               call    f6809cctodl_
               mov     [R6809_A],dl
               mov     ax,CPU_FETCH
               ret

tfr6809_A8     ENDP




;
; A9 - CC,B
;
tfr6809_A9     PROC    near

               call    f6809cctodl_
               mov     [R6809_B],dl
               mov     ax,CPU_FETCH
               ret

tfr6809_A9     ENDP




;
; AA - CC,CC
;
tfr6809_AA     PROC    near

               mov     ax,CPU_FETCH
               ret

tfr6809_AA     ENDP




;
; AB - CC,DP
;
tfr6809_AB     PROC    near

               call    f6809cctodl_
               mov     [R6809_DP],dl
               mov     ax,CPU_FETCH
               ret

tfr6809_AB     ENDP



;
; B8 - DP,A
;
tfr6809_B8     PROC    near

               mov     dl,[R6809_DP]
               mov     [R6809_A],dl
               mov     ax,CPU_FETCH
               ret

tfr6809_B8     ENDP




;
; B9 - DP,B
;
tfr6809_B9     PROC    near

               mov     dl,[R6809_DP]
               mov     [R6809_B],dl
               mov     ax,CPU_FETCH
               ret

tfr6809_B9     ENDP




;
; BA - DP,CC
;
tfr6809_BA     PROC    near

               mov     dl,[R6809_DP]
               call    f6809dltocc_
               mov     ax,CPU_FETCH
               ret

tfr6809_BA     ENDP




;
; BB - DP,DP
;
tfr6809_BB     PROC    near

               mov     ax,CPU_FETCH
               ret

tfr6809_BB     ENDP








_TFR6809_TEXT ends


_TFR6809_DATA segment byte public 'DATA'
              assume  DS:_TFR6809_DATA

_Tfr6809JumpTable dd  tfr6809_00
                  dd  tfr6809_01
                  dd  tfr6809_02
                  dd  tfr6809_03
                  dd  tfr6809_04
                  dd  tfr6809_05
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;10               
                  dd  tfr6809_10
                  dd  tfr6809_11
                  dd  tfr6809_12
                  dd  tfr6809_13
                  dd  tfr6809_14
                  dd  tfr6809_15
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;20
                  dd  tfr6809_20
                  dd  tfr6809_21
                  dd  tfr6809_22
                  dd  tfr6809_23
                  dd  tfr6809_24
                  dd  tfr6809_25
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;30
                  dd  tfr6809_30
                  dd  tfr6809_31
                  dd  tfr6809_32
                  dd  tfr6809_33
                  dd  tfr6809_34
                  dd  tfr6809_35
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;40
                  dd  tfr6809_40
                  dd  tfr6809_41
                  dd  tfr6809_42
                  dd  tfr6809_43
                  dd  tfr6809_44
                  dd  tfr6809_45
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;50
                  dd  tfr6809_50
                  dd  tfr6809_51
                  dd  tfr6809_52
                  dd  tfr6809_53
                  dd  tfr6809_54
                  dd  tfr6809_55
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;60
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;70
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;80
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_88
                  dd  tfr6809_89
                  dd  tfr6809_8A
                  dd  tfr6809_8B
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;90
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_98
                  dd  tfr6809_99
                  dd  tfr6809_9A
                  dd  tfr6809_9B
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;A0
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_A8
                  dd  tfr6809_A9
                  dd  tfr6809_AA
                  dd  tfr6809_AB
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;B0
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_B8
                  dd  tfr6809_B9
                  dd  tfr6809_BA
                  dd  tfr6809_BB
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;C0
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;D0
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;E0
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
;F0
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX
                  dd  tfr6809_EX

public _Tfr6809JumpTable



_TFR6809_DATA ends



end

