#ifdef DEBUG_MSG
/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : Vectrex emulator debugging functions                       */
/*                                                                           */
/*   File Name  : VDEBUG.C   (name change)                                   */
/*                                                                           */
/*   Author     : Keith Wilkins                                              */
/*                                                                           */
/*   Version    : 1.00                                                       */
/*                                                                           */
/*   Desciption : This file contains the functions for the Vectrex debugging */
/*                functions                                                  */
/*                                                                           */
/*   Functions  : fDebugInit()          - Debug initialisation               */
/*                fDebug()              - Debug message printing             */
/*                fDebugToggle()        - Toggle the Debug window ON/OFF     */
/*                fDebugMonitorToggle() - Toggle the monitor ON/OFF          */
/*                fDebugMonitor()       - The runtime debugger               */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    15/03/93  K.W  Creation of empty file and descriptions         */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*                                                                           */
/*****************************************************************************/

#define DEBUG_C

#include "vectrex.h"

GWINDOWPTR   pwDebugWin=NULL;

#define	VARG_BUFF_SIZE	500

/* Define local function prototypes */

#ifdef DEBUG_MON
  void fDebugMonitorToggle(UWORD tKeyPress);


// Define window size/position array for debug & monitor windows

DEBUGWININFO asDbgWin[RESOLUTION_COUNT]=
{
// Xpos Ypos Xsiz Ysiz FGcl                 BGcl                 BDcl                 Font        FGTitle              BGTitle

     5,   5, 600,  50, WHITE_BASE+BRIGHT15,    RED_BASE+BRIGHT7,   RED_BASE+BRIGHT15, FONT_SMALL,    RED_BASE+BRIGHT7, WHITE_BASE+BRIGHT15,
     5,   5, 600,  80, WHITE_BASE+BRIGHT15,    RED_BASE+BRIGHT7,   RED_BASE+BRIGHT15, FONT_SMALL,    RED_BASE+BRIGHT7, WHITE_BASE+BRIGHT15,
     5,   5, 600,  100, WHITE_BASE+BRIGHT15,    RED_BASE+BRIGHT7,   RED_BASE+BRIGHT15, FONT_SMALL,    RED_BASE+BRIGHT7, WHITE_BASE+BRIGHT15,
     5,   5, 600,  200, WHITE_BASE+BRIGHT15,    RED_BASE+BRIGHT7,   RED_BASE+BRIGHT15, FONT_SMALL,    RED_BASE+BRIGHT7, WHITE_BASE+BRIGHT15,
     5,   5, 600,  300, WHITE_BASE+BRIGHT15,    RED_BASE+BRIGHT7,   RED_BASE+BRIGHT15, FONT_SMALL,    RED_BASE+BRIGHT7, WHITE_BASE+BRIGHT15,
};


DEBUGWININFO asMonWin[RESOLUTION_COUNT]=
{
// Xpos Ypos Xsiz Ysiz FGcl                 BGcl                 BDcl                 Font        FGTitle              BGTitle

     5, 300, 600, 150, WHITE_BASE+BRIGHT15,  GREEN_BASE+BRIGHT7, GREEN_BASE+BRIGHT15, FONT_SMALL,  GREEN_BASE+BRIGHT7, WHITE_BASE+BRIGHT15,
     5, 370, 600, 200, WHITE_BASE+BRIGHT15,  GREEN_BASE+BRIGHT7, GREEN_BASE+BRIGHT15, FONT_SMALL,  GREEN_BASE+BRIGHT7, WHITE_BASE+BRIGHT15,
     5, 530, 600, 200, WHITE_BASE+BRIGHT15,  GREEN_BASE+BRIGHT7, GREEN_BASE+BRIGHT15, FONT_SMALL,  GREEN_BASE+BRIGHT7, WHITE_BASE+BRIGHT15,
     5, 370, 600, 200, WHITE_BASE+BRIGHT15,  GREEN_BASE+BRIGHT7, GREEN_BASE+BRIGHT15, FONT_SMALL,  GREEN_BASE+BRIGHT7, WHITE_BASE+BRIGHT15,
     5, 530, 600, 200, WHITE_BASE+BRIGHT15,  GREEN_BASE+BRIGHT7, GREEN_BASE+BRIGHT15, FONT_SMALL,  GREEN_BASE+BRIGHT7, WHITE_BASE+BRIGHT15,
};

// 640x480  All screen
// 800x600  All screen
// 1024x768 All screen

#endif

#ifdef DEBUG_MSG
  void fDebugToggle(UWORD tKeyPress);
  void fDebugFileToggle(UWORD tKeyPress);
  void fDebugLevelLow(UWORD tKeyPress);
  void fDebugLevelMid(UWORD tKeyPress);
  void fDebugLevelHigh(UWORD tKeyPress);
#endif


/*****************************************************************************/
/*                                                                           */
/*   Function    : fDebugInit()                                              */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : This function initialises the debug functionality by      */
/*                 registering the debug KEYS to toggle the debug window     */
/*                 and Monitor window ON/OFF.                                */
/*                                                                           */
/*   Accepts     : nothing                                                   */
/*                                                                           */
/*   Returns     : BOOLEAN TRUE/FALSE                                        */
/*                                                                           */
/*****************************************************************************/

BOOL fDebugInit(void)
{

#ifdef DEBUG_MON
  fKBDfuncreg(KEY_ALT_M,fDebugMonitorToggle);
#endif

#ifdef DEBUG_MSG
  fKBDfuncreg(KEY_ALT_D,fDebugToggle);
  fKBDfuncreg(KEY_ALT_F,fDebugFileToggle);
  fKBDfuncreg(KEY_ALT_1,fDebugLevelLow);
  fKBDfuncreg(KEY_ALT_2,fDebugLevelMid);
  fKBDfuncreg(KEY_ALT_3,fDebugLevelHigh);
#endif    
                                                      
    return(TRUE);
}


/* Local Functions to toggle ON/OFF functions */

#ifdef DEBUG_MON
  void fDebugMonitorToggle(UWORD tKeyPress)
  {
    tKeyPress=0;
    
    /* Make sure we enter the monitor even if we're tracing or waiting */
    
    bGblDebugMonitorEnable=TRUE;
    lwGblSystemTickBreakpoint=0;
  }
#endif


#ifdef DEBUG_MSG
  void fDebugToggle(UWORD tKeyPress)
  {
    tKeyPress=0;
    if(bGblDebugEnable==FALSE)
    {
      bGblDebugEnable=TRUE;
      fDebug(HIGH,("fDebugToggle(): Debug switched ON"));
    }
    else
    {
      fDebug(HIGH,("fDebugToggle(): Debug switched OFF"));
      bGblDebugEnable=FALSE;
      fWinClose(pwDebugWin);
      pwDebugWin=NULL;
    }
  }
#endif


#ifdef DEBUG_MSG
  void fDebugFileToggle(UWORD tKeyPress)
  {
    tKeyPress=0;
    if(bGblDebugFileEnable==FALSE)
    {
      bGblDebugFileEnable=TRUE;
      fDebug(HIGH,("fDebugFileToggle() - FILE OUTPUT ENABLED"));
    }
    else
    {
      bGblDebugFileEnable=FALSE;
      fDebug(HIGH,("fDebugFileToggle() - FILE OUTPUT DISABLED"));
    }
  }   
#endif


#ifdef DEBUG_MSG
  void fDebugLevelLow(UWORD tKeyPress)
  {
    tKeyPress=0;
    tGblDebugLevel=LOW;
    fDebug(HIGH,("fDebugLevel() - ALL MESSAGES"));
  }
#endif


#ifdef DEBUG_MSG
  void fDebugLevelMid(UWORD tKeyPress)
  {
    tKeyPress=0;
    tGblDebugLevel=MID;
    fDebug(HIGH,("fDebugLevel() - MED & HIGH MESSAGES"));
  }
#endif


#ifdef DEBUG_MSG
  void fDebugLevelHigh(UWORD tKeyPress)
  {
    tKeyPress=0;
    tGblDebugLevel=HIGH;
    fDebug(HIGH,("fDebugLevel() - HIGH MESSAGES"));
  }
#endif



#ifdef DEBUG_MSG

/*****************************************************************************/
/*                                                                           */
/*   Function    : fDebug()                                                  */
/*                                                                           */
/*   Author      : K.W                                                       */
/*                                                                           */
/*   Description : This prints a message to the screen and to the debug file */
/*                 if open. Printing or file output is only done if the      */
/*                 tLevel >= tGblDebugLevel. All screen output is done via   */
/*                 the functions in io.c fOutText().                         */
/*                                                                           */
/*   Accepts     : BYTE tLevel, char *Text with formatting                   */
/*                                                                           */
/*   Returns     : Nothing                                                   */
/*                                                                           */
/*****************************************************************************/

SWORD fDebugXX(UBYTE *pcDebugString,...)
{
    static FILE  *pfDebugErrorFile=NULL;
    static FILE  *pfDebugFile=NULL;
    static UBYTE acTextBuffer[VARG_BUFF_SIZE];
    va_list      tGlop;


    /* Expand the varable argument list into the text buffer */

    va_start(tGlop,pcDebugString);                /* Initialise the variable array */

    if(vsprintf(acTextBuffer,pcDebugString,tGlop)==-1)
	 {
            fFatal("fDebugXX - Vararg Buffer Overflow");
            fKBDgetch();
    }

    va_end(tGlop);

    /* Test the tLevel */

    if(tGblDebugMsgLevel==VERROR)
    {
        /* If the error file isnt open then open it */

        if(pfDebugErrorFile==NULL)
        {
        	if((pfDebugErrorFile=fopen("vectrex.err","w"))==NULL)
        	{
               fFatal("fDebugXX() - Couldn't open error log file <vectrex.err>");
	            return(0);
    	    }
    	}
        fprintf(pfDebugErrorFile,"%s\n",acTextBuffer);
    }

    /* Now print to the window only if its been enabled, we dont need to */
    /* check the window handle but we will just in case                  */

    if(bGblDebugEnable==TRUE && tGblDebugMsgLevel>=tGblDebugLevel)
    {
       if(pwDebugWin==NULL)
       {
           if((pwDebugWin=fWinOpen(asDbgWin[wGblDisplayMode].iWinXpos    ,asDbgWin[wGblDisplayMode].iWinYpos,
                                   asDbgWin[wGblDisplayMode].iWinXsize   ,asDbgWin[wGblDisplayMode].iWinYsize,
                                   asDbgWin[wGblDisplayMode].iWinFGcolour,asDbgWin[wGblDisplayMode].iWinBGcolour,
                                   WIN_DEFAULT_BORDER_WIDTH,asDbgWin[wGblDisplayMode].iWinBDcolour))==NULL)
           {
               fIOPrintf(0,0,BRIGHT15,BRIGHT0,FONT_ELARGE,"fDebug() Window opening error");
           }
           else
           {
               fWinTitle(pwDebugWin,"Debug Messages",FONT_SMALL,(BASE_FONT_HEIGHT*FONT_SMALL)+2,
                         asDbgWin[wGblDisplayMode].iTitleFGcolour,asDbgWin[wGblDisplayMode].iTitleBGcolour);
           }
       }
       fWinPrintf(pwDebugWin,asDbgWin[wGblDisplayMode].iWinFontsize,"\n<%010ld> %s",lwGblSystemTickTotal,acTextBuffer);

       /* Output debug to file only if enabled */

	    if(bGblDebugFileEnable==TRUE)
       {
     	    if(pfDebugFile==NULL)
     	    {
     		    if((pfDebugFile=fopen("vectrex.dbg","w"))==NULL)
	     	    {
	 	            fFatal("fDebugXX() - Couldnt open debug log file <vectrex.dbg>");
	 	            return(0);
	     	    }
	        }
               fprintf(pfDebugFile,"<%010ld> %s\n",lwGblSystemTickTotal,acTextBuffer);
	    }
   }
   return(0);
}

#endif


/* End of DEBUG.C */
#endif
