/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : Vectrex HP3000 Main Source Code Module                     */
/*                                                                           */
/*   File Name  : VECTREX.C                                                  */
/*                                                                           */
/*   Author     : Keith Wilkins / C.S.                                       */
/*                                                                           */
/*   Version    : 1.20                                                       */
/*                                                                           */
/*   Desciption : This fie contains the main() function for the vectrex      */
/*                emulator software. This module does all the system init    */
/*                functions and contains the sequencer function to run the   */
/*                emulator itself.                                           */
/*                                                                           */
/*   Functions  : main()      - Initialisation, start and exit code          */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    13/03/93  K.W  File created                                    */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*                                                                           */
/*****************************************************************************/

#define VECTREX_C

#include "vectrex.h"


/* Local function prototypes */
BOOL fGlobalVariableLoad(void);


/**************************************************************************/
/* Define all of the Global externals, these are ALL defined in VECTREX.C */
/* by the inclusion of this file so do real declarations for vectrex.c &  */
/* therefore only externals for non VECTREX.C modules                     */
/**************************************************************************/

/*******************/
/* Profiling array */
/*******************/
#ifdef DEBUG_PROFILE
ULONG       alGblCPUProfile[0x300];
#endif

UWORD       wGblDigitalVolume=80;           /* Digital Volume Level */
BOOL        bGblForceVesa12=FALSE;
BOOL        bGblForceSingleProcessor=FALSE;
BOOL        bGblGoggleGame=FALSE;
UWORD       ImagerColorNow=255;

ULONG       lwGblWheelTicks=(1500000/25)/2; // every 25 ms is default

UWORD       wGblColor1Degrees=60;
UWORD       wGblColor2Degrees=60;
UWORD       wGblColor3Degrees=60;
UWORD       wGblImagerMode=0;

SWORD       wGblImagerXOffset1=0;
SWORD       wGblImagerXOffset2=0;
SWORD       wGblImagerXOffset=0;

/*******************/
/* Debug Variables */
/*******************/
BOOL        bGblDebugEnable=TRUE;           /* Debug prSWORDing screen on/off */
BOOL        bGblDebugFileEnable=TRUE;       /* Switch the debug file save on/off */
SWORD       tGblDebugLevel=HIGH;            /* Debug window display level */
SWORD       tGblDebugMsgLevel=HIGH;         /* Debug level of the individial messages */
BOOL        bGblDebugMonitorEnable=TRUE;    /* Switch the Monitor window on/off*/
/***********************/
/* Sequencer variables */
/***********************/

BOOL        bGblExtendedRam=FALSE;          /* Animaction Ram 2K */
BOOL        bGblNoShades=FALSE;
BOOL        bGblLoadColorsExact=FALSE;

BOOL        bGblQuitFlag=FALSE;             /* The exit flag */

BOOL        bGblStatsValid=TRUE;            // Allow stats printing

ULONG       lwGblSystemTickTotal=1;         /* Total number of SystemTicks since start */
ULONG       lwGblSystemTickBreakpoint=0;    /* BreakpoSWORD counter */

BOOL        bGblDisplayEnable=TRUE;         /* Monitor Persistance enable */
BOOL        bGblDisplayAutoRefresh=TRUE;
BOOL        bGblDisplayEnhance=FALSE;       // Enable vector repairing on redraw
BOOL        bGblDisplayVerbose=FALSE;              /* verbosity of output */
BOOL        bGblDisplayExactScaling=FALSE;         /* scaling by SHIFT or DIV */
BOOL        bGblDisplayFullRefresh=FALSE; /* set in *.vol refresh everything even if not changed */


SLONG       lwGblDisplayTickReload=30000;    /* Number of System ticks for one Peristance tick */
ULONG       lwGblDisplayTickMax=60000;
ULONG       lwGblDisplayTickMin=7500;
SLONG       lwGblDisplayTickCounter=1;
UWORD       wGblDisplayLinePeriod=1;        // How many display ticks a line lives for
UWORD       wGblDisplayBaseColour=1;

BOOL        bGblOverlayEnable=FALSE;
BOOL        bGblLightpenEnable=FALSE;       // read in overlay file *.vol

SLONG       lwGblClipX=17000;               // Clipping window top right
SLONG       lwGblClipY=18000;

BOOL        bGblSoundEnable=TRUE;           /* Enable the sound system */

SLONG       lwGblMaxTicks=100;              /* Vectrex speed limiter = 100%, converted to ticks later */

/**************************************/
/* Emulation type variables Variables */
/**************************************/
BOOL        bGblCartridgeROMWriteEnable=FALSE;
BOOL        bGblSystemROMWriteEnable=FALSE;

/**********************/
/* Setup default keys */
/**********************/
UWORD       wGblPly1Button1=SCAN_A;
UWORD       wGblPly1Button2=SCAN_S;
UWORD       wGblPly1Button3=SCAN_D;
UWORD       wGblPly1Button4=SCAN_F;

UWORD       wGblPly1JoystkUp=SCAN_UP;
UWORD       wGblPly1JoystkDn=SCAN_DOWN;
UWORD       wGblPly1JoystkLf=SCAN_LEFT;
UWORD       wGblPly1JoystkRt=SCAN_RIGHT;

UWORD       wGblPly2Button1=0x70;
UWORD       wGblPly2Button2=0x70;
UWORD       wGblPly2Button3=0x70;
UWORD       wGblPly2Button4=0x70;

UWORD       wGblPly2JoystkUp=0x70;
UWORD       wGblPly2JoystkDn=0x70;
UWORD       wGblPly2JoystkLf=0x70;
UWORD       wGblPly2JoystkRt=0x70;

BOOL        bGblDigitalJoystick=TRUE;
BOOL        bGblJoystickAutoCentre=TRUE;
SBYTE       tGblJoystickSensitivity=2;

ULONG        lwGblTicksToGrab=1500000/1000*33; // every 33 ms

BOOL        bGblRestoreOnStart=TRUE;
BOOL        bGblSaveOnExit=TRUE;


/*********************/
/* Define Video mode */
/*********************/
UWORD wGblDisplayMode=0;

/********************/
/* Define filenames */
/********************/
UBYTE   acGblROMImageFileName[STRING_LENGTH]="system.img";
UBYTE   acGblRAMImageFileName[STRING_LENGTH]="";
UBYTE   acGblDefaultCartFileName[STRING_LENGTH]="";
UBYTE   acGblCartCarousel[12][STRING_LENGTH]={"","","","","","","","","","","",""};

/*****************************************************************************/
/*                                                                           */
/*   Function   : main()                                                     */
/*                                                                           */
/*   Author     : K.W                                                        */
/*                                                                           */
/*   Desciption : Main initialisation, starup and exit code                  */
/*                                                                           */
/*   Accepts    : argc,argv values for command line args                     */
/*                                                                           */
/*   Returns    : nothing                                                    */
/*                                                                           */
/*****************************************************************************/

void main(SWORD argc,UBYTE *argv[])
{
 BOOL     bArgumentFail=FALSE;
 double   dCycSec;
 double   dEfficiency;
 time_t   tExecTime;

 printf("DVE version:%s, compiled %s\n",DVE_VERSION,__DATE__);

 /**********************************************************/
 /* First set all defaults, command line can then override */
 /**********************************************************/
 if(fGlobalVariableLoad()==FALSE)
  bArgumentFail=TRUE;
 /*******************************/
 /* Decode the arguments passed */
 /*******************************/


 if (argc==2)
 {
  strcpy(acGblDefaultCartFileName,argv[1]);
 }

 /******************************/
 /* Sanity check on video mode */
 /******************************/
 if (wGblDisplayMode>=RESOLUTION_COUNT)
 {
  fFatal("Invalid display mode has been set");
 }
 /*************************************/
 /* Call the initialisation functions */
 /*************************************/
 if (bArgumentFail==TRUE)
 {
  fprintf(stderr,"Command line argument decode failed, aborting emulator.\n");
 }
 else
 {
  if (fTimerInit()==FALSE)
  {
   fprintf(stderr,"Timer Initialisation failed\n");
  }
  else if(fKBDinit()==FALSE)
  {
   fprintf(stderr,"KBD Initialisation failed\n");
  }
  else if(fIOInit()==FALSE)
  {
   fprintf(stderr,"I/O Initialisation failed\n");
  }
#ifdef DEBUG_MSG
  else if(fDebugInit()==FALSE)
  {
   fprintf(stderr,"Debug Initialisation failed\n");
  }
#endif
  else
  {
   if (fSystemInit()==FALSE)
   {
    fDebug(VERROR,("Initialisation sequence failed, aborting emulator."));
   }
   else
   {
    /**************/
    /* Here we go */
    /**************/
//  fAlertBox(RED_BASE+BRIGHT8,"\n Welcome to DVE \n");

    /*************************************************/
    /* Convert lwGblMaxTicks to tick count from %age */
    /*************************************************/
    if (lwGblMaxTicks>1000)
     lwGblMaxTicks=1000;  // Ha ha ha ha
    lwGblMaxTicks*=(SLONG)(1500000.0/18.2);
    lwGblMaxTicks/=100;  // Remove %age
//  fAlertBox(RED_BASE+BRIGHT8,"\n Max ticks = %d \n",lwGblMaxTicks);
    tExecTime=clock();
    if (!fVectrexEmulate())
    {
     fDebug(VERROR,("Emulation exited with error"));
    }
    else
    {
     fDebug(HIGH,("Emulation completed succesfully"));
    }
    tExecTime=clock()-tExecTime;
   }
   /*******************/
   /* Prepare to quit */
   /*******************/

   /******************************/
   /* Dump profiling information */
   /******************************/
#ifdef DEBUG_PROFILE
   {
    FILE  *pfProfile=NULL;
    UWORD index,tag;
    ULONG max,total;

    /***********************************/
    /* Try and open the RAM image file */
    /***********************************/
    if ((pfProfile=fopen("vectrex.prf","wt"))==NULL)
    {
     fFatal("main() - Couldnt open profile dump file <vectrex.prf>");
    }
    total=0;
    for (index=0;index<300;index++)
     total+=alGblCPUProfile[index];
    do
    {
     max=0;
     for (index=0;index<300;index++)
     {
      if (alGblCPUProfile[index]>max)
      {
       tag=index;
       max=alGblCPUProfile[index];
      }
     }
     if (max)
     {
      fprintf(pfProfile,"Opcode <%03x> ",tag);
      fprintf(pfProfile," %6s ",as6809Dis[tag].acInstruction);
      fprintf(pfProfile,"- %12s ",ac6809Modes[as6809Dis[tag].bAddressMode]);
      fprintf(pfProfile," - Score %10ld (%02.02f%%)\n",max,((double)max/(double)total)*100.0);
      alGblCPUProfile[tag]=0;
     }
    }
    while(max);
   }
#endif
   /*************************************************/
   /* Return the screen and io handlers back to DOS */
   /*************************************************/
   fTimerRestore();
   fSoundRestore();
   fIORestore();
   fKBDrestore();
  }
 }
 /*******************/
 /* Now lets finish */
 /*******************/
 fcloseall();

 /******************************/
 /* Dump execution speed stats */
 /******************************/
 if (bGblStatsValid)
 {
  if (tExecTime/CLOCKS_PER_SEC)
  {
   dCycSec=(lwGblSystemTickTotal/(tExecTime/CLOCKS_PER_SEC));
   dEfficiency=(dCycSec/1500000.0)*100;
   printf("\nSpeed rating: %.0f Cycles/Sec",dCycSec);
   printf("\n              %3.1f%%\n",dEfficiency);
  }
  else
  {
   printf("\nDVE did not run for long enough to perform speed profile\n");
  }
 }
 exit(0);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : fSystemInit()                                             */
/*                                                                           */
/*   Author      : K.W / C.S.                                                */
/*                                                                           */
/*   Description : Perform complete machine initialisation                   */
/*                                                                           */
/*   Accepts     : Nothing                                                   */
/*                                                                           */
/*   Returns     : BOOL    TRUE=Init OK, FALSE=Init Fail                     */
/*                                                                           */
/*****************************************************************************/
BOOL fSystemInit(void)
{
 BOOL  bInitStatus=TRUE;
 UBYTE volfile[STRING_LENGTH];

 fDebug(HIGH,("fSystemReset() - VECTREX HP3000 Emulator Initialisation Sequence"));

 f6522Addr0DRead=f6522Addr0DReadNoLightpen;

 /********************************************/
 /* first set all specific functions         */
 /* mixture between: SHIFT/DIV/LINEAR/BANKED */
 /* neccessary because of 'inline' functions */
 /********************************************/
 if (bGblDigitalJoystick==TRUE)
   fAnalogJoystickUpdate=fAnalogJoystickUpdateDigital;
 else
   fAnalogJoystickUpdate=fAnalogJoystickUpdateAnalog;

 if (framebuffer==FB_LINEAR) /* linear framebuffer */
 {
  fVectrexEmulate=fVectrexEmulate_linear_no_lightpen;
  if (bGblDisplayExactScaling==FALSE) /* SHIFT */
  {
   fAnalogTick=fAnalogTick_linear_shift;
  }
  else /* DIV */
  {
   fAnalogTick=fAnalogTick_linear_div;
  }
 }
 else /* banked framebuffer */
 {
  fVectrexEmulate=fVectrexEmulate_banked_no_lightpen;
  if (bGblDisplayExactScaling==FALSE) /* SHIFT */
  {
   fAnalogTick=fAnalogTick_bank_shift;
  }
  else /* DIV */
  {
   fAnalogTick=fAnalogTick_bank_div;
  }
 }

 /***********/
 /* Display */
 /***********/
 /**********************************/
 /* Munge the name of the VOL file */
 /**********************************/
 strcpy(volfile,acGblDefaultCartFileName);
 if (strrchr(volfile,'.')!=NULL)
  *(strrchr(volfile,'.'))='\0';
 strcat(volfile,".vol");
 if ((bInitStatus&=fDisplayInit(volfile))==FALSE)
 {
  fFatal("fSystemInit() - Vector Display : Initialisation Failed.");
 }
 else
 {
  fDebug(HIGH ,("Vector Display       : O.K."));
 }


 /************/
 /* 6809 CPU */
 /************/
 if ((bInitStatus=f6809Init())==FALSE)
 {
  fFatal("fSystemInit() - 68A09 CPU : Initialisation Failed.");
 }
 else
 {
  fDebug(HIGH ,("68A09 CPU            : O.K."));
 }
#ifdef DEBUG_PROFILE
 {
  int loop;
  for(loop=0;loop<0x300;loop++) alGblCPUProfile[loop]=0;
 }
#endif



 /*****************/
 /* 2114 1Kx8 RAM */
 /*****************/
 if ((bInitStatus&=fSystemRAMLoad(acGblRAMImageFileName))==FALSE)
 {
  fFatal("fSystemInit() - 2114 RAM : Initialisation Failed.");
 }
 else
 {
  fDebug(HIGH ,("2114 RAM             : O.K."));
 }

 /*****************/
 /* 2363 8Kx8 ROM */
 /*****************/
 if ((bInitStatus&=fSystemROMLoad(acGblROMImageFileName))==FALSE)
 {
  fFatal("fSystemInit() - 2363 System ROM Load : Initialisation Failed.");
 }
 else
 {
  fDebug(HIGH ,("2363 System ROM Load : O.K."));
 }

 /*****************/
 /* Cartridge ROM */
 /*****************/
 if ((bInitStatus&=fCartridgeROMLoad(acGblDefaultCartFileName))==FALSE)
 {
  fFatal("fSystemInit() - Cartridge ROM Load : Initialisation Failed.");
 }
 else
 {
  fDebug(HIGH ,("Cartridge ROM Load   : O.K."));
 }



 /*************/
 /* 6522A PIA */
 /*************/
 if ((bInitStatus&=f6522Init())==FALSE)
 {
  fFatal("fSystemInit() - 6522A PIA : Initialisation Failed.");
 }
 else
 {
  fDebug(HIGH ,("6522A PIA            : O.K."));
 }

 /********************/
 /* AY-3-8192A Sound */
 /********************/
 if ((bInitStatus&=fSoundInit())==FALSE)
 {
  fFatal("fSystemInit() - AY-3-8192 Chip : Initialisation Failed.");
 }
 else
 {
  fDebug(HIGH ,("AY-3-8192 Chip       : O.K."));
 }

 /*********************/
 /*  Analog functions */
 /*********************/
 if ((bInitStatus&=fAnalogInit())==FALSE)
 {
  fFatal("fSystemInit() - Analog Block : Initialisation Failed.");
 }
 else
 {
  fDebug(HIGH ,("Analog Block         : O.K."));
 }

 return(bInitStatus);
}

/*****************************************************************************/
/*                                                                           */
/*   Function    : fGlobalVariableLoad()                                     */
/*                                                                           */
/*   Author      : K.W / C.S.                                                */
/*                                                                           */
/*   Description : Load/Initialise ALL of the global variables.              */
/*                                                                           */
/*   Accepts     : Char* pointer to file to load from                        */
/*                                                                           */
/*   Returns     : BOOL    TRUE=Init OK, FALSE=Init Fail                     */
/*                                                                           */
/*****************************************************************************/

BOOL    fGlobalVariableLoad(void)
{
 /***************************************************/
 /* Define the token array for parsing the INI file */
 /***************************************************/
 /**********************************************************/
 /* Chris: I had to move them from global to auto state... */
 /*        Dunno why, but at some stage it took 10% of     */
 /*        Emulator performance if they were global.       */
 /*        Is there some hidden speed secret about Watcom  */
 /*        global Variable overall size?                   */
 /**********************************************************/
 INIFILEENTRY asIniFileTable[]=
 {
  TYPE_UWORD    ,"digital_volume"        ,&wGblDigitalVolume,
  TYPE_BOOL     ,"force_vesa12"          ,&bGblForceVesa12,
  TYPE_BOOL     ,"force_single_processor",&bGblForceSingleProcessor,

  TYPE_BOOL     ,"debug_enable"          ,&bGblDebugEnable,
  TYPE_BOOL     ,"debug_monitor"         ,&bGblDebugMonitorEnable,

  TYPE_BOOL     ,"restore_on_start"      ,&bGblRestoreOnStart,
  TYPE_BOOL     ,"save_on_exit"          ,&bGblSaveOnExit,

  TYPE_BOOL     ,"display_verbose"       ,&bGblDisplayVerbose,
  TYPE_BOOL     ,"display_exact_scaling" ,&bGblDisplayExactScaling,

  TYPE_BOOL     ,"display_enable"        ,&bGblDisplayEnable,
  TYPE_BOOL     ,"display_auto_refresh"  ,&bGblDisplayAutoRefresh,
  TYPE_BOOL     ,"display_enhance"       ,&bGblDisplayEnhance,
  TYPE_SLONG    ,"display_update_period" ,&lwGblDisplayTickReload,
  TYPE_ULONG    ,"display_update_max"    ,&lwGblDisplayTickMax,
  TYPE_ULONG    ,"display_update_min"    ,&lwGblDisplayTickMin,
  TYPE_UWORD    ,"display_line_period"   ,&wGblDisplayLinePeriod,
  TYPE_UWORD    ,"display_mode"          ,&wGblDisplayMode,
  TYPE_UWORD    ,"display_colour"        ,&wGblDisplayBaseColour,
  TYPE_SLONG    ,"display_clip_x"        ,&lwGblClipX,
  TYPE_SLONG    ,"display_clip_y"        ,&lwGblClipY,

  TYPE_UWORD    ,"display_0div_x"        ,&(divider_x[0]),// SVGA640X480        0
  TYPE_UWORD    ,"display_0div_y"        ,&(divider_y[0]),
  TYPE_UWORD    ,"display_1div_x"        ,&(divider_x[1]),// SVGA800X600        1
  TYPE_UWORD    ,"display_1div_y"        ,&(divider_y[1]),
  TYPE_UWORD    ,"display_2div_x"        ,&(divider_x[2]),// SVGA1024X768       2
  TYPE_UWORD    ,"display_2div_y"        ,&(divider_y[2]),
  TYPE_UWORD    ,"display_3div_x"        ,&(divider_x[3]),// SVGA1280X1024      3
  TYPE_UWORD    ,"display_3div_y"        ,&(divider_y[3]),
  TYPE_UWORD    ,"display_4div_x"        ,&(divider_x[4]),// SVGA1600X1200      4
  TYPE_UWORD    ,"display_4div_y"        ,&(divider_y[4]),

  TYPE_UWORD    ,"display_0shifta_x"     ,&(display_infox[0].iDivShiftA),// SVGA640X480        0
  TYPE_UWORD    ,"display_0shiftb_x"     ,&(display_infox[0].iDivShiftB),
  TYPE_UWORD    ,"display_0shifta_y"     ,&(display_infoy[0].iDivShiftA),
  TYPE_UWORD    ,"display_0shiftb_y"     ,&(display_infoy[0].iDivShiftB),
  TYPE_UWORD    ,"display_1shifta_x"     ,&(display_infox[1].iDivShiftA),// SVGA800X600        1
  TYPE_UWORD    ,"display_1shiftb_x"     ,&(display_infox[1].iDivShiftB),
  TYPE_UWORD    ,"display_1shifta_y"     ,&(display_infoy[1].iDivShiftA),
  TYPE_UWORD    ,"display_1shiftb_y"     ,&(display_infoy[1].iDivShiftB),
  TYPE_UWORD    ,"display_2shifta_x"     ,&(display_infox[2].iDivShiftA),// SVGA1024X768       2
  TYPE_UWORD    ,"display_2shiftb_x"     ,&(display_infox[2].iDivShiftB),
  TYPE_UWORD    ,"display_2shifta_y"     ,&(display_infoy[2].iDivShiftA),
  TYPE_UWORD    ,"display_2shiftb_y"     ,&(display_infoy[2].iDivShiftB),
  TYPE_UWORD    ,"display_3shifta_x"     ,&(display_infox[3].iDivShiftA),// SVGA1280X1024      3
  TYPE_UWORD    ,"display_3shiftb_x"     ,&(display_infox[3].iDivShiftB),
  TYPE_UWORD    ,"display_3shifta_y"     ,&(display_infoy[3].iDivShiftA),
  TYPE_UWORD    ,"display_3shiftb_y"     ,&(display_infoy[3].iDivShiftB),
  TYPE_UWORD    ,"display_4shifta_x"     ,&(display_infox[4].iDivShiftA),// SVGA1600X1200      4
  TYPE_UWORD    ,"display_4shiftb_x"     ,&(display_infox[4].iDivShiftB),
  TYPE_UWORD    ,"display_4shifta_y"     ,&(display_infoy[4].iDivShiftA),
  TYPE_UWORD    ,"display_4shiftb_y"     ,&(display_infoy[4].iDivShiftB),

  TYPE_BOOL     ,"overlay_enable"        ,&bGblOverlayEnable,

  TYPE_BOOL     ,"sound_enable"          ,&bGblSoundEnable,

  TYPE_ULONG    ,"max_speed"             ,&lwGblMaxTicks,

  TYPE_STRING   ,"rom_image"             ,acGblROMImageFileName,
  TYPE_BOOL     ,"rom_write_enable"      ,&bGblSystemROMWriteEnable,
  TYPE_STRING   ,"ram_image"             ,acGblRAMImageFileName,
  TYPE_STRING   ,"default_cart"          ,acGblDefaultCartFileName,
  TYPE_BOOL     ,"cart_write_enable"     ,&bGblCartridgeROMWriteEnable,

  TYPE_BOOL     ,"digital_joystick"      ,&bGblDigitalJoystick,
  TYPE_BOOL     ,"joystick_auto_centre"  ,&bGblJoystickAutoCentre,
  TYPE_SBYTE    ,"joystick_sensitivity"  ,&tGblJoystickSensitivity,

  TYPE_STRING   ,"cartridge01"           ,acGblCartCarousel[0],
  TYPE_STRING   ,"cartridge02"           ,acGblCartCarousel[1],
  TYPE_STRING   ,"cartridge03"           ,acGblCartCarousel[2],
  TYPE_STRING   ,"cartridge04"           ,acGblCartCarousel[3],
  TYPE_STRING   ,"cartridge05"           ,acGblCartCarousel[4],
  TYPE_STRING   ,"cartridge06"           ,acGblCartCarousel[5],
  TYPE_STRING   ,"cartridge07"           ,acGblCartCarousel[6],
  TYPE_STRING   ,"cartridge08"           ,acGblCartCarousel[7],
  TYPE_STRING   ,"cartridge09"           ,acGblCartCarousel[8],
  TYPE_STRING   ,"cartridge10"           ,acGblCartCarousel[9],
  TYPE_STRING   ,"cartridge11"           ,acGblCartCarousel[10],
  TYPE_STRING   ,"cartridge12"           ,acGblCartCarousel[11],

  TYPE_UWORD    ,"player1button1"        ,&wGblPly1Button1,
  TYPE_UWORD    ,"player1button2"        ,&wGblPly1Button2,
  TYPE_UWORD    ,"player1button3"        ,&wGblPly1Button3,
  TYPE_UWORD    ,"player1button4"        ,&wGblPly1Button4,

  TYPE_UWORD    ,"player1up"             ,&wGblPly1JoystkUp,
  TYPE_UWORD    ,"player1down"           ,&wGblPly1JoystkDn,
  TYPE_UWORD    ,"player1left"           ,&wGblPly1JoystkLf,
  TYPE_UWORD    ,"player1right"          ,&wGblPly1JoystkRt,

  TYPE_UWORD    ,"player2button1"        ,&wGblPly2Button1,
  TYPE_UWORD    ,"player2button2"        ,&wGblPly2Button2,
  TYPE_UWORD    ,"player2button3"        ,&wGblPly2Button3,
  TYPE_UWORD    ,"player2button4"        ,&wGblPly2Button4,

  TYPE_UWORD    ,"player2up"             ,&wGblPly2JoystkUp,
  TYPE_UWORD    ,"player2down"           ,&wGblPly2JoystkDn,
  TYPE_UWORD    ,"player2left"           ,&wGblPly2JoystkLf,
  TYPE_UWORD    ,"player2right"          ,&wGblPly2JoystkRt,

  TYPE_ULONG    ,"tickstograb"           ,&lwGblTicksToGrab,
  -1            ,"END"                   ,NULL,
 };
 FILE    *pfINIfile;
 UBYTE   acBuffer[1024];
 UBYTE   *pcTokenName;
 UBYTE   *pcTokenValue;
 SWORD   iTmp;

 if (bGblDisplayVerbose==TRUE)
  printf("\nIni File loading...");
 /***********************************************/
 /* Now use the INI file to overload the values */
 /***********************************************/
 /***********************************/
 /* If not NULL then attempt a load */
 /***********************************/
 if ((pfINIfile=fopen("vectrex.ini","rt"))!=NULL)
 {
  if (bGblDisplayVerbose==TRUE)
   printf ("\nIni found...");
  while (fgets(acBuffer,1024,pfINIfile)!=NULL)
  {
   if (acBuffer[0]!='#')
   {
    if ((pcTokenName=strtok(acBuffer," ="))!=NULL && (pcTokenValue=strtok(NULL," =\n"))!=NULL)
    {
     strlwr(pcTokenName);
// printf("Token=\"%s\" Value=\"%s\"\n",pcTokenName,pcTokenValue);
     iTmp=0;
     while (asIniFileTable[iTmp].tType!=-1)
     {
      if (strcmp(asIniFileTable[iTmp].acNameTag,pcTokenName)==0)
      {
       switch(asIniFileTable[iTmp].tType)
       {
        case TYPE_BOOL:
        {
         SWORD iVal;
         iVal=atoi(pcTokenValue);
         *((BOOL *)asIniFileTable[iTmp].pvLocation)=(BOOL)(iVal)?TRUE:FALSE;
         break;
        }
        case TYPE_UBYTE:
        {
         SWORD iVal;
         iVal=(atoi(pcTokenValue));
         *((UBYTE*)asIniFileTable[iTmp].pvLocation)=(UBYTE)iVal;
         break;
        }
        break;
        case TYPE_SBYTE:
        {
         SWORD iVal;
         iVal=(atoi(pcTokenValue));
         *((SBYTE*)asIniFileTable[iTmp].pvLocation)=(SBYTE)iVal;
         break;
        }
        break;
        case TYPE_UWORD:
        {
         UWORD iVal;
         iVal=(atoi(pcTokenValue)&0xffff);
         *((UWORD*)asIniFileTable[iTmp].pvLocation)=(UWORD)iVal;
         break;
        }
        break;
        case TYPE_SWORD:
        {
         SWORD iVal;
         iVal=(atoi(pcTokenValue));
         *((SWORD*)asIniFileTable[iTmp].pvLocation)=(SWORD)iVal;
         break;
        }
        break;
        case TYPE_ULONG:
        {
         ULONG iVal;
         iVal=(atol(pcTokenValue));
         *((ULONG*)asIniFileTable[iTmp].pvLocation)=(ULONG)iVal;
         break;
        }
        break;
        case TYPE_SLONG:
        {
         SLONG iVal;
         iVal=(atol(pcTokenValue));
         *((SLONG*)asIniFileTable[iTmp].pvLocation)=(SLONG)iVal;
         break;
        }
        break;
        case TYPE_STRING:
        {
         strcpy((UBYTE*)asIniFileTable[iTmp].pvLocation,pcTokenValue);
         break;
        }
        default:
        {
         fFatal("Ini file parse error - Should never happen");
         break;
        }
       }
       break;
      }
      iTmp++;
     }
     if (asIniFileTable[iTmp].tType==-1)
     {
      fFatal("Unrecognized token in VECTREX.INI (%s)",pcTokenName);
     }
    }
   }
  }
  fclose(pfINIfile);
 }
 return(TRUE);
}

/* End of VECTREX.C */
