/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : Vectrex HP3000 Emulator Main Header file                   */
/*                                                                           */
/*   File Name  : VECTREX.H                                                  */
/*                                                                           */
/*   Author     : Keith Wilkins / C.S.                                       */
/*                                                                           */
/*   Version    : 1.20                                                       */
/*                                                                           */
/*   Desciption : This is the main header file for the vectrex emulator it   */
/*                inlcudes all of the headers that any vectrex emulator      */
/*                module could possibly require in the way of function       */
/*                prototypes and externals.                                  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    13/03/93  K.W  Creation                                        */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*    1.20    20/10/96  C.S  DVE V1.2                                        */
/*                                                                           */
/*****************************************************************************/

#if !defined VECTREX_H
    #define VECTREX_H

#include <ctype.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <graph.h>
#include <malloc.h>
#include <process.h>
#include <dos.h>
#include <math.h>
#include <time.h>
#include <i86.h>


/*********************/
/* Base declarations */
/*********************/
typedef                short BOOL;
typedef unsigned       char  UBYTE;
typedef signed         char  SBYTE;
typedef unsigned       short UWORD;
typedef                short SWORD;
typedef unsigned       long  ULONG;
typedef                long  SLONG;

#define TRUE           1
#define FALSE          0
#define STRING_LENGTH  128


/*******************************/
/* Define INI file token types */
/*******************************/
#define    TYPE_BOOL       0
#define    TYPE_UBYTE      1
#define    TYPE_SBYTE      2
#define    TYPE_UWORD      3
#define    TYPE_SWORD      4
#define    TYPE_ULONG      5
#define    TYPE_SLONG      6
#define    TYPE_STRING     7

typedef struct inientry
{
   SWORD   tType;
   char    *acNameTag;
   void    *pvLocation;
}INIFILEENTRY;

/**********************************/
/* Define screen resolution stuff */
/**********************************/
#define RESOLUTION_COUNT   5

#define SVGA640X480        0
#define SVGA800X600        1
#define SVGA1024X768       2
#define SVGA1280X1024      3
#define SVGA1600X1200      4

/**********************************************/
/* Inlcude the other module declaration files */
/**********************************************/
#include "io.h"
#include "keyboard.h"
#include "window.h"
#include "util.h"
#include "error.h"
#include "vdebug.h"
#include "monitor.h"
#include "emu6809.h"
#include "dis6809.h"
#include "sequence.h"
#include "sys_rom.h"
#include "sys_ram.h"
#include "cart_rom.h"
#include "emu6522.h"
#include "soundemu.h"
#include "analog.h"
#include "display.h"
#include "timer.h"


/***********************/
/* Function prototypes */
/***********************/
BOOL   fSystemReset(void);
BOOL   fSystemInit(void);


#ifdef DEBUG_PROFILE
    extern ULONG        alGblCPUProfile[0x300];
#endif


#define IMAGER_DRAW_OVERLAY 256
#define IMAGER_DRAW_NOTHING 257
#define IMAGER_DRAW_COLOR_16 240
#define IMAGER_DRAW_COLOR_15 225

#define IMAGER_DRAW_WHEEL_COLOR_1 240
#define IMAGER_DRAW_WHEEL_COLOR_2 210
#define IMAGER_DRAW_WHEEL_COLOR_3 225

extern UWORD        ImagerColorNow;
extern ULONG        lwGblWheelTicks;
extern SWORD        wGblImagerXOffset1;
extern SWORD        wGblImagerXOffset2;
extern SWORD        wGblImagerXOffset;

extern UWORD        wGblColor1Degrees;
extern UWORD        wGblColor2Degrees;
extern UWORD        wGblColor3Degrees;
extern UWORD        wGblImagerMode;
extern BOOL         bGblGoggleGame;
extern BOOL         bGblForceSingleProcessor;
extern BOOL         bGblForceVesa12;
extern BOOL         bGblDebugEnable;
extern BOOL         bGblDebugFileEnable;
extern SWORD        tGblDebugLevel;
extern SWORD        tGblDebugMsgLevel;
extern BOOL         bGblDebugMonitorEnable;
extern BOOL         bGblRestoreOnStart;
extern BOOL         bGblSaveOnExit;

extern BOOL         bGblExtendedRam;
extern BOOL         bGblNoShades;
extern BOOL         bGblLoadColorsExact;


extern BOOL         bGblQuitFlag;
extern BOOL         bGblStatsValid;
extern ULONG        lwGblSystemTickTotal;
extern ULONG        lwGblSystemTickBreakpoint;

extern BOOL         bGblDisplayEnable;
extern BOOL         bGblDisplayAutoRefresh;
extern BOOL         bGblDisplayEnhance;

extern BOOL         bGblDisplayFullRefresh;

extern BOOL         bGblDisplayVerbose;
extern BOOL         bGblDisplayExactScaling;
extern SLONG        lwGblDisplayTickReload;
extern ULONG        lwGblDisplayTickMax;
extern ULONG        lwGblDisplayTickMin;
extern SLONG        lwGblDisplayTickCounter;
extern UWORD        wGblDisplayLinePeriod;
extern UWORD        wGblDisplayBaseColour;

extern BOOL         bGblOverlayEnable;
extern BOOL         bGblLightpenEnable; // read in *.vol file, default: false

extern SLONG        lwGblClipX;
extern SLONG        lwGblClipY;

extern BOOL         bGblSoundEnable;
extern UWORD        wGblDigitalVolume;

extern SLONG        lwGblMaxTicks;

extern BOOL         bGblCartridgeROMWriteEnable;
extern BOOL         bGblSystemROMWriteEnable;

extern UWORD        wGblPly1Button1;
extern UWORD        wGblPly1Button2;
extern UWORD        wGblPly1Button3;
extern UWORD        wGblPly1Button4;

extern UWORD        wGblPly1JoystkUp;
extern UWORD        wGblPly1JoystkDn;
extern UWORD        wGblPly1JoystkLf;
extern UWORD        wGblPly1JoystkRt;

extern UWORD        wGblPly2Button1;
extern UWORD        wGblPly2Button2;
extern UWORD        wGblPly2Button3;
extern UWORD        wGblPly2Button4;

extern UWORD        wGblPly2JoystkUp;
extern UWORD        wGblPly2JoystkDn;
extern UWORD        wGblPly2JoystkLf;
extern UWORD        wGblPly2JoystkRt;

extern BOOL         bGblDigitalJoystick;
extern SBYTE        tGblJoystickSensitivity;
extern BOOL         bGblJoystickAutoCentre;

extern UWORD        wGblDisplayMode;

extern UBYTE        acGblROMImageFileName[STRING_LENGTH];
extern UBYTE        acGblRAMImageFileName[STRING_LENGTH];
extern UBYTE        acGblDefaultCartFileName[STRING_LENGTH];

extern UBYTE        acGblCartCarousel[12][STRING_LENGTH];

extern ULONG        lwGblTicksToGrab;

#endif

/* End of VECTREX.H */
