#ifndef VESA_H
# define VESA_H

# ifdef __cplusplus
 extern "C" {
# endif

# include <standard.h>

# define MAX_ZEILEN_LEN 1600

/* VESA 256 Farben MODI */
# define VESA_640X400 0x100
# define VESA_640X480 0x101
# define VESA_800X600 0x103
# define VESA_1024X768 0x105
# define VESA_1280X1024 0x107

/* Fehlermeldungen */
# define VESA_NICHT_GEFUNDEN 200
# define NICHT_INITIALISIERT 201
# define MODUS_NICHT_GEFUNDEN 202
# define NICHTS_FREIGEGEBEN 203
# define MODUS_NICHT_VERFUEGBAR 204
# define VESA_TREIBER_FEHLER 205
# define KEIN_VESA_MODUS_AKTIV 206
# define KEIN_ALTER_MODUS_DA 207
# define KEIN_256_FARBEN_MODUS 208
# define KEIN_MAUSTREIBER_DA 209
# define KEINE_MAUS_INSTALLIERT 210
# define MAUS_POS_OUT_OF_SCREEN 211
# define KEIN_RECTANGLE 212
# define ZEILE_GROESSE_MAX_ZEILEN_LEN 213

# define VESA_DATEN_FREIGEGEBEN (1*1)
# define VESA_MODUS_DATEN_FREIGEGEBEN (1*10)
# define VESA_MODUS_UND_DATEN_FREIGEGEBEN (VESA_DATEN_FREIGEGEBEN+VESA_MODUS_DATEN_FREIGEGEBEN)

/* Print Modi */
# define PIC_ON_BACK 1
# define PIC_FILLED 2
# define MULTI_COLOR 1
# define SINGLE_COLOR 2

	
	typedef struct
	{
		BYTE	signatur[4];
		BYTE	vesa_versionsnummer_lo;
		BYTE	vesa_versionsnummer_hi;
		char *herstellername;
		DWORD	leistungsflag;
		WORD *modi_liste;
		BYTE	fuellen[256-18];
	} VESA_DATEN;

	typedef struct
	{
		int	modus_flag;
		BYTE	flag_fenster_1;
		BYTE	flag_fenster_2;
		int	granularitaet;
		WORD	fenster_groesse;
		WORD	seg_adresse_fenster_1;
		WORD seg_adresse_fenster_2;
		void *fenster_anzeige_routine;
		int	bytes_pro_zeile;
		int	x_aufloesung;
		int	y_aufloesung;
		BYTE	breite_zeichenmatrix;
		BYTE hoehe_zeichenmatrix;
		BYTE	anzahl_bitplanes;
		BYTE	anzahl_bits_pro_punkt;
		BYTE	anzahl_speicherbloecke;
		BYTE	speichermodell;
		BYTE	groesse_speicherbloecke;
		BYTE	fuellen[256-29];
	} VESA_MODUS_DATEN;

	typedef struct {
								int   groesse_x;
								int   groesse_y;
								BYTE *maus_zeiger;
	}MOUSE_POINTER;

	typedef struct {
								int   x;
								int   y;
								int   len_x;
								int   len_y;
								BYTE *buffer;
	}RECTANGLE_SAVE;

	extern VESA_DATEN *vesa_daten;
	extern VESA_MODUS_DATEN *vesa_modus_daten;
	extern FONT default_font;
	extern FONT *aktueller_font;
	extern WORD vesa_error;
	extern BYTE *zeile;
	extern BYTE *zeile_hilf;
 extern volatile int vesa_mouse_x;
 extern volatile int vesa_mouse_y;
 extern volatile int vesa_mouse_button;
	extern int vesa_da;
 extern int vesa_modus_aktiv;
 extern int vesa_aufloesung_x;
 extern int vesa_aufloesung_y;
 extern int vesa_mouse_initialisiert;
 extern int vesa_print_modus;
 extern int vesa_back_color;
 extern int vesa_text;
	extern int vesa_text_color;
	extern int vesa_lese_fenster;
 extern int vesa_schreib_fenster;
	extern WORD vesa_segment_adresse_fenster[2];
	extern WORD vesa_fenster_adresse[2];
	//
	extern volatile int vesa_mouse_interrupt;
	//


	int vesa_karte_da(void);			                  /* ALLE_OK, VESA_NICHT_GEFUNDEN, KEIN_SPEICHER_DA */
 int vesa_existiert_modus(int modus);  		      /* MODUS_NICHT_VERFUEGBAR, MODUS_VERFUEGBAR, VESA_TREIBER_FEHLER */
 int vesa_lese_modus_daten(int modus);  	      /* ALLES_OK, VESA_NICHT_GEFUNDEN, KEIN_SPEICHER_DA */
 int vesa_anzeige_modus_daten(int modus);  	    /* return werte wie 'lese_vesa_modus_daten()' */
 int vesa_anzeige_daten(void);  		            /* Wert der Variablen vesa_da = ist_vesa_karte_da() */
	int vesa_gebe_datenspeicher_zurueck(void); 	  /* NICHTS_FREIGEGEBEN, VESA_DATEN_FREIGEGEBEN, VESA_MODUS_DATEN_FREIGEGEBEN, VESA_MODUS_UND_DATEN_FREIGEGEBEN */
	int vesa_alten_modus_speichern(void);		      /* ALLES_OK, KEIN_SPEICHER_DA */
	int vesa_alten_modus_setzen(void);		          /* ALLES_OK, KEIN_ALTER_MODUS_DA */
	int vesa_alten_modus_freigeben(void);         	/* ALLES_OK, KEIN_ALTER_MODUS_DA */
	int vesa_setze_modus(int modus);		            /* MODUS_NICHT_VERFUEGBAR, MODUS_VERFUEGBAR, Rueckgabe siehe ist_vesa_karte_da(), MODUS_NICHT_VERFUEGBAR, MODUS_VERFUEGBAR, VESA_TREIBER_FEHLER */
	int vesa_plot256(int x,int y,int farbe);
	int vesa_print_zeile256(int x,int y,BYTE *zeile, WORD zeilen_laenge);
	int vesa_mouse_init(void);
	int vesa_mouse_deinit(void);
	int vesa_mouse_setze_cursor(int x, int y);
	int vesa_mouse_setze_rect(int x_min, int y_min,int x_max, int y_max);
	void vesa_mouse_verstecke_cursor(void);
	void vesa_mouse_zeige_cursor(void);
	void vesa_mouse_mickey_pro_8punkte(int mickey_x, int mickey_y);
	int vesa_set_default_mouse_color(BYTE outline,BYTE fill);
	int vesa_get_zeile(int x1, int y1, int len,BYTE *zeilen_buffer);

	void vesa_line_h(int x1,int y1,int len, BYTE color);
	void vesa_line_v(int x1,int y1,int len, BYTE color);
	void vesa_rect(int x1,int y1,int len_x, int len_y, BYTE color);
	void vesa_rect_fill(int x1,int y1,int len_x, int len_y,BYTE color);
	RECTANGLE_SAVE *vesa_rect_save(int x1,int y1,int len_x, int len_y,BYTE color);
	int vesa_remove_rectangle(RECTANGLE_SAVE *rectangle);

	void vesa_set_dac_register(int color_nummer,BYTE r,BYTE g,BYTE b);
	int vesa_get_dac_register(int color_nummer,BYTE *r, BYTE *g, BYTE *b);
	int vesa_set_mehrere_dac(int start,int anzahl, BYTE *buffer);
	int vesa_get_mehrere_dac(int start, int anzahl, BYTE *buffer);
	int vesa_save_mehrere_dac(int start, int anzahl, char *dateiname);
	int vesa_load_mehrere_dac(int start, char *dateiname);

	int vesa_get_dot_col256(int x,int y);

//	int vesa_print_string(int x, int y,char *string);
//	int vesa_print_string_max(int x, int y,int len_x,int len_y,char *string);
//	int vesa_print_string_max2(int x, int y,int len_x, int len_y,char *string);
//	int vesa_print_char(int x, int y,BYTE zeichen);
// #define vesa_print_char(x,y,z) vesa_print_char_max(x,y,PMAX,PMAX,z)
//	int vesa_print_char_max(int x, int y,int len_x,int len_y, BYTE zeichen);
//#define vesa_print_char(x,y,z) vesa_print_char_max3(x,y,PMAX,PMAX,z)

	int vesa_print_string_max3(int x, int y,int len_x, int len_y,char *string);

int _ch_vpsm3_(int x,int y,char z);
int _chm_vpsm3_(int x,int y,int len_x,int len_y,char z);
#define vesa_print_string_max(x,y,len_x,len_y,string) vesa_print_string_max3(x,y,len_x,len_y,string)
#define vesa_print_string_max2(x,y,len_x,len_y,string) vesa_print_string_max3(x,y,len_x,len_y,string)

#define vesa_print_char(x,y,z) _ch_vpsm3_(x,y,z)
#define vesa_print_char_max(x,y,len_x,len_y,z) _chm_vpsm3_(x,y,len_xlen_y,z)
#define vesa_print_string(x,y,string) vesa_print_string_max3(x,y,PMAX,PMAX,string)



	int vesa_print_int(int x, int y, long zahl);
	void vesa_draw_shaded_rectangle(int x, int y, int len_x, int len_y,BYTE col1, BYTE col2, int thick);

	int vesa_load_font(char *dateiname,FONT *ein_font);
	int vesa_free_font(FONT *ein_font);
	int vesa_get_text_len(char *text, int *len_x, int *len_y);
	void sync_display(void);
	void clear_screen(void);

 int vesa_mouse_get_irq(void);



# ifdef __cplusplus
	}
# endif

#endif
