#ifdef DEBUG_MSG
/*****************************************************************************/
/*                                                                           */
/*   VECTREX HP3000 Software Emulator              Copyright K.Wilkins 1996  */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Title      : Vectrex emulator graphics window header file               */
/*                                                                           */
/*   File Name  : WINDOW.H                                                   */
/*                                                                           */
/*   Author     : Keith Wilkins                                              */
/*                                                                           */
/*   Version    : 1.00                                                       */
/*                                                                           */
/*   Desciption : This file contains the header information for other        */
/*                modules so that they can use the window functions.         */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   Revision History:                                                       */
/*                                                                           */
/*   Version    Date    Who  Description of changes                          */
/*   -------    ----    ---  ----------------------                          */
/*                                                                           */
/*    0.01    05/04/93  K.W  Creation                                        */
/*    1.00    05/08/96  K.W  Public release of DVE V1.0                      */
/*                                                                           */
/*****************************************************************************/

#if !defined WINDOW_H
   #define WINDOW_H

#define WIN_MIN_SIZE                15
#define WIN_MAX_TITLE_LENGTH        30
#define WIN_DEFAULT_BORDER_WIDTH    2

typedef struct GWINDOW
{
   ULONG              tMagicNumber;

   SWORD              iXAnchorPoint;
   SWORD              iYAnchorPoint;

   SWORD              iXSizeInternal;
   SWORD              iYSizeInternal;

   SWORD              iBorderWidth;
   SWORD              iBorderColour;

   SWORD              iBackgroundColour;

   SWORD              iTitleBarHeight;
   SWORD              iTitleBarTextColour;
   SWORD              iTitleBarBackgroundColour;
   UBYTE              acTitle[WIN_MAX_TITLE_LENGTH+1];

   /* Under no circulstances let the user program manipulate the cursor     */
   /* position directly, this is an ABSOLUTE cursor position with referance */
   /* to the screen origin at 0,0. Use fWinSetCursorPos and fWinGetCursorPos*/
   /* to do your manipulation                                               */

   PIXEL            tCursorPos;

   BOOL             bHidden;

   BITBLOCKPTR      tScreenSave;
   BITBLOCKPTR      tWindowSave;

   struct GWINDOW   *ptNextWindow;
   struct GWINDOW   *ptPrevWindow;

}GWINDOW,*GWINDOWPTR;


/* Define function prototypes */

GWINDOWPTR  fWinOpen(SWORD iXpos,SWORD iYpos,SWORD iXsize,SWORD iYsize,SWORD iForegroundColour,SWORD iBackgroundColour,SWORD iBorderWidth,SWORD iBorderColour);
BOOL        fWinClose(GWINDOWPTR tWorkWin);
BOOL        fWinTitle(GWINDOWPTR tWorkWin,UBYTE *pcTitle,SWORD FontSize,SWORD iTitleHeight,SWORD iTitleColour,SWORD iBackgroundColour);
BOOL        fWinHide(GWINDOWPTR tWorkWin);
BOOL        fWinRestore(GWINDOWPTR tWorkWin);
BOOL        fWinActivate(GWINDOWPTR tWorkWin);
BOOL        fWinMove(GWINDOWPTR tWorkWin,SWORD iXpos,SWORD iYpos);
BOOL        fWinSetCursorPos(GWINDOWPTR tWorkWin,SWORD iXpos,SWORD iYpos,SWORD iColour);
PIXEL       fWinGetCursorPos(GWINDOWPTR tWorkWin);
BOOL        fWinPrintf(GWINDOWPTR tWorkWin,SWORD iFontSize,UBYTE *pcFormat,...);
BOOL        fWinOutText(GWINDOWPTR tWorkWin,SWORD iFontSize,UBYTE *pcText);
BOOL        fWinOutChar(GWINDOWPTR tWorkWin,SWORD iFontSize,UBYTE cData);
BOOL        fWinClear(GWINDOWPTR tWorkWin);
SWORD       fWinInput(GWINDOWPTR tWorkWin,SWORD iFontSize,UBYTE *pcBuffer,SWORD iMaxLength);
BOOL        fWinPlot(GWINDOWPTR tWorkWin,SWORD iXpos,SWORD iYpos,SWORD iColour);
BOOL        fWinLine(GWINDOWPTR tWorkWin,SWORD iX1,SWORD iY1,SWORD iX2,SWORD iY2,SWORD iColour);
GWINDOWPTR  fQuickWin(SWORD iColour,UBYTE *pcFormat,...);
GWINDOWPTR  fQuickWinPos(SWORD iXpos,SWORD iYpos,SWORD iColour,UBYTE *pcFormat,...);
SWORD       fAlertBox(SWORD iColour,UBYTE *pcFormat,...);


#endif

/* End of WINDOW.H */


#endif
